/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "animation.h"


Animation::Animation (ResourcePool* resources, AnimationType index, string defFile){
	this->notifiedObservers = false;
	this->interruptibleBy = INTERRUPT_ANYONE;
	this->timeLength = 0; this->timePassed = 0;
    this->repeating = false; this->animationType = index;
    this->stampSet = NULL;
    this->resources = resources;
    this->load (defFile);
}
Animation::Animation (AnimationSettings settings){
	this->notifiedObservers = settings.getNotifiedObservers();
    this->timeLength = settings.getTimeLength();
    this->timePassed = settings.getTimePassed();
    this->repeating = settings.getRepeating();
    this->interruptibleBy = settings.getInterruptibleBy();
    this->animationType = settings.getAnimationType();
    this->animationObservers = settings.getAnimationObservers();
    this->stampSet = settings.getStampSet();
    this->frames = settings.getFrames();
    this->resources = settings.getResources();
}

Animation::~Animation(){}

Interface_Cloneable* Animation::clone() const{
	vector<Interface_AnimationObserver*> newObservers;
    AnimationSettings settings (timeLength, timePassed, repeating, interruptibleBy, animationType,
							   	newObservers, stampSet, frames, resources, notifiedObservers);
    return new Animation (settings);
}

bool Animation::update (unsigned int passed){
	//cout << "Animation is updating: " << passed << ". Frame: " << getCurrentFrame() << " Repeating? " << repeating << endl;
	//cout << "Time passed: " << timePassed << " of time length " << timeLength << endl;
	//cout << "Source rect is " << getRect().x << ", " << getRect().y << ", "
	//	<< getRect().w << "," << getRect().h << "." << endl;
    bool dirty = false;

    int oldFrame = this->getCurrentFrame();

	if (timePassed < timeLength){
	    this->timePassed += passed;
    	int newFrame = this->getCurrentFrame();
    	if (oldFrame != newFrame){
        	dirty = true;
    	}
	}
	if (timePassed >= timeLength){
        if (this->repeating){
            this->timePassed = 0;
        }
		if (!notifiedObservers){
			//cout << "Animation at " << this << " notifying observers." << endl;
			notifiedObservers = true;
			for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin(); 
					next != animationObservers.end(); next++){
   		    	(*next)->animationComplete (NULL, animationType);
			}
		}
		else{
			//cout << "Animation at " << this << " already notified observers. Ignoring." << endl;
		}
    }

    return dirty;
}

void Animation::addAnimationObserver (Interface_AnimationObserver* observer){
	if (! hasAnimationObserver (observer)){
		animationObservers.push_back (observer);
	}
}
bool Animation::hasAnimationObserver (Interface_AnimationObserver* observer){
	for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			next != animationObservers.end(); next++){
		if (*next == observer){
			return true;
		}
	}
	return false;
}
void Animation::removeAnimationObserver (Interface_AnimationObserver* observer){
	bool found = false;
	for (vector <Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			!found && next != animationObservers.end(); next++){
		if (*next == observer){
			found = true;
			animationObservers.erase (next);
		}
	}
}

int Animation::getAnimationTimeRemaining() const{
	if (getRepeating()){ 
		//cout << "animation is repeating" << endl;
		return -1;
	}
	if (getTimeLength() == 0) return 0;
	return max (0, int (getTimeLength()) - int (getTimePassed()));
}

SDL_Rect Animation::getRect(){
    int frame = getCurrentStampIndex();

	if (frame != -1){
		SDL_Rect retRect = stampSet->getRect (frame);
		//cout << "Animation::getRect() returning: " << retRect.x << ", " << retRect.y << ", " 
		//	<< retRect.w << ", " << retRect.h << "." << endl;
		return retRect;
	}
	else{
		//cout << "Animation::getRect() frame is -1." << endl;
		SDL_Rect retRect;
		retRect.x = 0; retRect.y = 0; retRect.w = 0; retRect.h = 0;
		//cout << "Animation::getRect() returning: " << retRect.x << ", " << retRect.y << ", " 
		//	<< retRect.w << ", " << retRect.h << "." << endl;
		return retRect;
	}
}

SDL_Surface* Animation::getImage(){
	if (stampSet){
		return stampSet->getCurrentSurface();
	}
	else{
		return NULL;
	}
}

void Animation::load (string definitionFile){
	//cout << "Loading animation file." << endl;
    // This def file should be a ".anm" file with a list of indexes and a sprite sheet def file (".def" file).
	frames.clear();
    ifstream defFile;
    defFile.open (definitionFile.c_str());
    string spriteSheetFile;
    AnimationType index;
	defFile >> spriteSheetFile;
	if (stampSet){
		delete stampSet; stampSet = NULL;
	}
	//stampSet = new StampSet (resources, spriteSheetFile);
	stampSet = resources->loadStampSet (spriteSheetFile);
    defFile >> index;
    while (! defFile.eof()){
        //cout << "Animation's next index: " << index << " and file: '" << definitionFile << "'." << endl;

        // Now set up this animation set's next animation.
        frames.push_back (index);
        defFile >> index;
    }
    defFile.close();
	//cout << "Done loading animation file." << endl;
}

int Animation::getCurrentStampIndex() const{
	int frame = getCurrentFrame();
	//cout << "Animation::getCurrentStampIndex() frame is " << frame << endl;
	int maxIndex = getFrameCount() - 1;
	if (frame > maxIndex) frame = maxIndex;
	//cout << "Animation::getCurrentStampIndex() frame is, after adjustment, " << frame << endl;
	if (frame != -1 && stampSet){
		//cout << "Animation::getCurrentStampIndex() returning "; 
		//cout << frames [frame] << endl;
		return frames [frame];
	}
	return -1;
}

int Animation::getCurrentFrame() const{
    unsigned int numFrames = this->getFrameCount();
    if (numFrames > 0){
		unsigned int width = timeLength / numFrames;
		if (width > 0){
			//cout << "Animation::getCurrentFrame() timeLength is " << timeLength << " and numFrames is " << numFrames << endl;
			return timePassed / width;
		}
		else if (width == 0){
			//cout << "Animation::getCurrentFrame() width is 0!" << endl;
			return 0;
		}
		else{
			//cout << "Animation::getCurrentFrame() there is no current frame! returning -1 " << endl;
			return -1;
		}
    }
	else{
    	return -1;
	}
}
