#include "audioManager.h"
#include "math.h"
#include <SDL/SDL.h>
#include <SDL/SDL_audio.h>
#include <iostream>
#include <algorithm>
#include "soundInstance.h"
#include "soundInstanceReference.h"
#include "lock.h"
#include "tryLock.h"
#include "interface_lockable.h"
#include "audioProcessingManager.h"
#include "audioEffect.h"

using namespace std;


namespace Audio{
	#define NUM_SOUNDS 1024
	struct sample {
		AudioData* data;
		Uint32 dlen;
	} sounds[NUM_SOUNDS];
	double mainVolume_; Interface_Lockable mainVolumeLock_;

	AudioManager audioManager_;

	void mixaudio(void *unused, Uint8 *stream, int amountToBuffer)
	{
		audioManager_.mixAudio (unused, stream, amountToBuffer);
	}

	double AudioManager::getMainVolume(){
		Lock volumeLock (&mainVolumeLock_);
		return mainVolume_;
	}
	void AudioManager::setMainVolume (double volumeZeroToOne){
		Lock volumeLock (&mainVolumeLock_);
		volumeZeroToOne = min (1.0, max (0.0, volumeZeroToOne));
		mainVolume_ = volumeZeroToOne;
	}

	void AudioManager::applyPanning16 (Uint32* start, unsigned int numSamples, double panZeroToOne){
		if (fabs (panZeroToOne - 0.5) < 0.05) return; // Just skip if pan is already centered.
		//cout << "num samples is " << numSamples << endl;
		short* where = (short*) start;
		//cout << "where: " << where << ", end: " << end << endl;
		double leftMult = 1.0;
		double rightMult = 1.0;
		if (panZeroToOne <= 0.5) rightMult = panZeroToOne / 0.5;
		else leftMult = (1.0 - panZeroToOne) / 0.5;
		double leftMultInv = 1.0 - leftMult;
		double rightMultInv = 1.0 - rightMult;
		for (unsigned int x = 0; x < numSamples; x++){
			//where [2 * x] /= 2;
			//where [2 * x + 1] /= 2;
			int left = where [2 * x] * leftMult + where [2 * x + 1] * rightMultInv;
			if (left < -32500) left = -32500; if (left > 32500) left = 32500;
			int right = where [2 * x] * leftMultInv + where [2 * x + 1] * rightMult;
			if (right < -32500) right = -32500; if (right > 32500) right = 32500;
			where [2 * x] = left;
			where [2 * x + 1] = right;
		}
	}

	void AudioManager::mixAudio (void *unused, Uint8 *stream, int amountToBuffer)
	{
		//cout << "mixaudio called" << endl;
		Lock externalInstancesLock (externalInstancesLock_);
		Lock internalInstancesLock (internalInstancesLock_);
		Lock loadedSoundsLock (loadedSoundsLock_);

		for (ExternalInstancesList::iterator iter = externalInstances_.begin(); 
				iter != externalInstances_.end(); iter++){
			//cout << "mixing sound instance for file " << iter->first->getName() << endl;
			if (iter->first->getState() != SoundInstance::statePlaying
					&& iter->first->getState() != SoundInstance::stateRepeating){
				//cout << "instance no longer playing. skipping" << endl;
				continue;
			}
			Audio::AudioData* dataToBuffer = NULL;
			// If there are effects, use the effects' processed data buffer.
			if (iter->first->getEffects().size() > 0){
				dataToBuffer = &(iter->second);
			}
			else{
				int soundIndex = loadedSounds_ [iter->first->getName()];
				dataToBuffer = sounds [soundIndex].data;
			}
			Audio::AudioData extraSamples (dataToBuffer->getBitSize(), dataToBuffer->getRate());
			Uint32 dlen = dataToBuffer->getNumberOfSamples() * dataToBuffer->getBitSize() / 8 * 2;
			Uint32 len = min ((Uint32) amountToBuffer, dlen);
			Uint32 dpos = iter->first->getBufferPosition();
			Uint32 amount = min (len, max (Uint32 (0), dlen - dpos));
			// Need to add extra samples to buffer after dataToMix's samples AND set buffer 
			//	position that much farther into beginning of dataToBuffer
			if (dpos + (Uint32) amountToBuffer > dlen && 
					iter->first->getState() == SoundInstance::stateRepeating){
				//cout << "repeating" << endl;
				// Add some samples to buffer at the end
				unsigned int numSamplesToCopy = 
					((dpos + (Uint32) amountToBuffer) - dlen) * 8 / 2 / dataToBuffer->getBitSize();
				//cout << "extra samples is " << numSamplesToCopy << endl;
				extraSamples.copySamplesFrom (*dataToBuffer, 0, numSamplesToCopy);
			}
			//cout << "amountToBuffer " << amountToBuffer << " dlen " << dlen << endl;
			double volume = iter->first->getVolume();
			int mixedVolume = int (SDL_MIX_MAXVOLUME * volume * getMainVolume());
			//cout << "num samples is " << dataToBuffer->getNumberOfSamples() << endl;
			Uint8* where = (Uint8*) &(dataToBuffer->getRaw() [0]);
			where += dpos;
			{
				//cout << "amount: " << amount << ", bitsize: " << dataToBuffer->getBitSize() << endl;
				Uint32* pannedData = (Uint32*) malloc (amount);
				memcpy ((void*) pannedData, where, amount);
				applyPanning16 (pannedData, (amount * 8) / (dataToBuffer->getBitSize() * 2), 
					iter->first->getPan());
				//cout << "pan is " << iter->first->getPan() << endl;
				SDL_MixAudio(stream, (Uint8*) pannedData, amount, mixedVolume);
				free (pannedData); pannedData = NULL;
			}
			unsigned int newBufferPos = dpos + amount;
			if (extraSamples.getNumberOfSamples() > 0){
				unsigned int bytesMissed = 
					extraSamples.getNumberOfSamples() * extraSamples.getBitSize() * 2 / 8;
				//cout << "mixing " << bytesMissed << " missed bytes" << endl;
				where = (Uint8*) &(dataToBuffer->getRaw() [0]);
				{
					//cout << "bytes missed is " << bytesMissed << endl;
					Uint32* pannedData = (Uint32*) malloc (bytesMissed);
					memcpy ((void*) pannedData, where, bytesMissed);
					applyPanning16 (pannedData, (bytesMissed * 8) / (dataToBuffer->getBitSize() * 2), 
						iter->first->getPan());
					//cout << "pan is " << iter->first->getPan() << endl;
					SDL_MixAudio(stream + amount, (Uint8*) pannedData, bytesMissed, mixedVolume);
					free (pannedData); pannedData = NULL;
				}
				newBufferPos = bytesMissed;
				//cout << "setting new buffer position to " << newBufferPos << endl;
			}
			//cout << "len is " << len << " dpos is " << dpos << " amount is " << amount
			//     << " volume is " << mixedVolume << endl;
			iter->first->setBufferPosition (newBufferPos);
		}
		//cout << "mixaudio done" << endl;
	}

	void AudioManager::refreshInstances (unsigned int timePassed){
		/*
		TryLock externalInstancesLock (externalInstancesLock_);
		if (!externalInstancesLock.tryLock()){ 
			//cout << "missed a trylock" << endl;
			return;
		}
		else{
			//cout << "got a trylock" << endl;
		}
		*/
		Lock externalInstancesLock (externalInstancesLock_);
		Lock internalInstancesLock (internalInstancesLock_);
		Lock loadedSoundsLock (loadedSoundsLock_);

		std::vector<std::tr1::shared_ptr <SoundInstanceReference> > donePlaying;
		// First, make a list of all the sounds that are done playing
		for (std::vector<std::tr1::shared_ptr <SoundInstanceReference> >::iterator iter = internalInstances_.begin();
				iter != internalInstances_.end(); iter++){
			if ((*iter)->getState() != SoundInstanceReference::statePlaying){ 
				//cout << "sound " << (*iter)->getName() << " no longer playing" << endl;
				donePlaying.push_back (*iter);
			}
		}
		// Next, remove the sounds that are done playing
		for (std::vector<std::tr1::shared_ptr <SoundInstanceReference> >::iterator iter = donePlaying.begin();
				iter != donePlaying.end(); iter++){
			std::vector<std::tr1::shared_ptr <SoundInstanceReference> >::iterator found = 
				std::find (internalInstances_.begin(), internalInstances_.end(), *iter);
			if (found != internalInstances_.end()){
				internalInstances_.erase (found);
			}
		}

		// Clean up all external instances that are no longer in use.
		for (size_t x = 0; x < externalInstances_.size(); ){
			if (!externalInstances_ [x].first->getInUse()){ 
				//cout << "deleting sound instance for sound " << externalInstances_ [x].first->getName() << endl;
				SoundInstance* dying = externalInstances_ [x].first;
				externalInstances_.erase (externalInstances_.begin() + x);
				delete dying;
			}
			else x++;
		}

		// Now update all sound instances
		for (ExternalInstancesList::iterator iter = externalInstances_.begin();
				iter != externalInstances_.end(); iter++){
			iter->first->update (timePassed);
		}
	}

	void AudioManager::update (unsigned int timePassed){
		//cout << "audio manager updating" << endl;
		if (timePassed < 1) return;

		refreshInstances (timePassed);

		Interface_AudioProcessingManager* processor = new AudioProcessingManager (16, 44100); // HAX
		std::vector<SoundInstance*> instances;
		std::vector<Audio::AudioData> data;
		{
			/*
			TryLock externalInstancesLock (externalInstancesLock_);
			if (!externalInstancesLock.tryLock()) return;
			*/
			Lock externalInstancesLock (externalInstancesLock_);
			Lock internalInstancesLock (internalInstancesLock_);
			Lock loadedSoundsLock (loadedSoundsLock_);

			for (ExternalInstancesList::iterator iter = externalInstances_.begin(); 
					iter != externalInstances_.end(); iter++){
				instances.push_back (iter->first);
				Audio::AudioData samples (iter->second.getBitSize(), iter->second.getRate());
				samples.copySamplesFrom (iter->second);
				data.push_back (samples);
			}
		}

		for (unsigned int index = 0; index < data.size(); index++){
			unsigned int windowStart = data [index].getNumberOfSamples();
			unsigned int msToBuffer = 200;
			unsigned int windowEnd = 
				windowStart + (msToBuffer * data [index].getRate() / 1000);
			windowEnd = std::min (
				windowEnd, instances [index]->getTimeLength() * data [index].getRate() / 1000
			);

			if (windowStart >= windowEnd) continue;

			std::vector<AudioEffect*> effects = instances [index]->getEffects();
			//cout << "get effects returned " << effects.size() << " effects" << endl;

			// We only need to buffer if it's possible effects will alter audio data
			if (effects.size() > 0){
				Audio::AudioData* sourceData = NULL;
				{
					Lock externalInstancesLock (externalInstancesLock_);
					Lock internalInstancesLock (internalInstancesLock_);
					Lock loadedSoundsLock (loadedSoundsLock_);
					int soundIndex = loadedSounds_ [instances [index]->getName()];
					sourceData = sounds [soundIndex].data;
				}

				processor->startEffectStack (*sourceData, windowStart, windowEnd);
				for (std::vector<AudioEffect*>::iterator effect = effects.begin(); effect != effects.end(); effect++){
					//cout << "processing audio, window: " << windowStart << ", " << windowEnd << endl;
					// Have instances' effects process some audio for our buffers
					(*effect)->processAudio (processor);
				}
				processor->endEffectStack();
				sourceData = NULL;

				Lock externalInstancesLock (externalInstancesLock_);

				AudioData composite (processor->getCompositeAudioData());
				AudioData existing (data [index]);
				//cout << "existing audio data has rate " << data [index].getRate() 
				//	<< " bitsize " << data [index].getBitSize() << " sample count " 
				//	<< existing.getNumberOfSamples() << endl;
				//cout << "composite audio data has rate " << composite.getRate() 
				//	<< " bitsize " << composite.getBitSize() << " sample count " 
				//	<< composite.getNumberOfSamples() << endl;

				if (composite.getNumberOfSamples() > windowStart){
					existing.copySamplesFrom (composite, windowStart, composite.getNumberOfSamples() - windowStart);
				}

				{
					for (ExternalInstancesList::iterator found = externalInstances_.begin();
							found != externalInstances_.end(); found++){
						if (found->first == instances [index]){ 
							found->second = existing;
							break;
						}
					}
				}
				/*
				cout << "samples are ";
				for (unsigned int x = 0; x < existing.getNumberOfSamples(); x++){
					cout << "[" << existing.getSample (x).getLeftSample() << "," 
						<< existing.getSample (x).getRightSample() << "] ";
				}
				cout << endl << endl;
				*/
			}
		}
		delete processor; processor = NULL;
	}

	AudioManager::AudioManager(){
		// Lock in this order, and always lock all lockables above and equal to the one you want. 
		// e.g. If you need loadedSounds locked, lock it and internal and external instances.
		externalInstancesLock_ = new Interface_Lockable();
		internalInstancesLock_ = new Interface_Lockable();
		loadedSoundsLock_ = new Interface_Lockable();

		mainVolume_ = 1.0;

		SDL_Init (SDL_INIT_AUDIO);
		for (int x = 0; x < NUM_SOUNDS; x++){ 
			sounds [x].dlen = 0;
		}

		/* Set 16-bit stereo audio at 44Khz */
		fmt.freq = 44100;
		fmt.format = AUDIO_S16SYS;
		fmt.channels = 2;
		fmt.samples = 2048;        /* A good value for games */
		fmt.callback = mixaudio;
		fmt.userdata = NULL;

		bytesPerMs_ = fmt.freq * (fmt.format == AUDIO_S16SYS ? 2 : 1) * fmt.channels / 1000;

		/* Open the audio device and start playing sound! */
		if ( SDL_OpenAudio(&fmt, NULL) < 0 ) {
			fprintf(stderr, "Unable to open audio: %s\n", SDL_GetError());
			exit(1);
		}

		SDL_PauseAudio (0);
	}
	AudioManager::~AudioManager(){
		delete internalInstancesLock_;
		delete externalInstancesLock_;
		delete loadedSoundsLock_;

		SDL_CloseAudio();
	}

	std::tr1::shared_ptr<SoundInstanceReference> AudioManager::playSound (
			const std::string& filename, double volumeZeroToOne){
		//cout << "Playing sound" << endl;
		std::tr1::shared_ptr<SoundInstanceReference> newInstance = loadSound (filename);
		newInstance->setVolume (volumeZeroToOne);
		newInstance->play();
		Lock externalInstancesLock (externalInstancesLock_);
		Lock internalInstanceLock (internalInstancesLock_);
		internalInstances_.push_back (newInstance);
		//cout << "Returning sound" << endl;
		return newInstance;
	}

	std::tr1::shared_ptr<SoundInstanceReference> AudioManager::loadSound(const std::string& filename)
	{
		Lock externalInstancesLock (externalInstancesLock_);
		Lock internalInstancesLock (internalInstancesLock_);
		Lock loadedSoundsLock (loadedSoundsLock_);

		std::map<std::string, int>::iterator loadedSound = loadedSounds_.find (filename);
		int index = 0;
		if (loadedSound == loadedSounds_.end()){
			index = loadedSounds_.size(); if (index >= NUM_SOUNDS) throw ("Can't load sound: no buffers left!");
			SDL_AudioSpec wave;
			Uint8 *data;
			Uint32 dlen;
			SDL_AudioCVT cvt;

			/* Load the sound file and convert it to 16-bit stereo at 44kHz */
			if ( SDL_LoadWAV(filename.c_str(), &wave, &data, &dlen) == NULL ) {
				fprintf(stderr, "Couldn't load %s: %s\n", filename.c_str(), SDL_GetError());
				throw ("Couldn't load a file!");
			}
			//cout << "Loaded sound file " << filename << endl;
			if (SDL_BuildAudioCVT(&cvt, wave.format, wave.channels, wave.freq,
				AUDIO_S16SYS, 2, fmt.freq) == -1) cout << "Couldn't build converter" << endl;
			cvt.buf = (Uint8*) malloc(dlen*cvt.len_mult);
			memcpy(cvt.buf, data, dlen);
			cvt.len = dlen;
			SDL_ConvertAudio(&cvt);
			SDL_FreeWAV(data);

			/* Put the sound data in the slot */
			unsigned int bitSize = 16;
			unsigned int rate = 44100;
			sounds[index].data = new AudioData (bitSize, rate, (char*) cvt.buf, cvt.len_cvt);
			sounds[index].dlen = cvt.len_cvt;
		
			loadedSounds_ [filename] = index;
			loadedSound = loadedSounds_.find (filename);
		}
		index = loadedSound->second;
		SoundInstance* newInstance = new SoundInstance (filename, sounds [index].dlen, bytesPerMs_);
		SoundInstanceReference* newInstanceReference = new SoundInstanceReference (newInstance);
		unsigned int bitSize = 16; // HAX make this dynamic
		unsigned int rate = 44100; // HAX
		externalInstances_.push_back (std::pair<SoundInstance*, AudioData> (
			newInstance, AudioData (bitSize, rate)
			//newInstance, AudioData (bitSize, rate, (char*) *sounds [index].data, sounds [index].dlen)
		));

		return std::tr1::shared_ptr<SoundInstanceReference> (newInstanceReference);
	}
}
