#ifndef BINLIST_H
#define BINLIST_H


#include <vector>
#include "utils.h"


template <class T>
class Interface_IndexEvaluator{
	public:
		Interface_IndexEvaluator(){}
		virtual ~Interface_IndexEvaluator(){}
		virtual bool lessThan (const T& first, const T& second) const = 0;
		virtual bool equals (const T& first, const T& second) const = 0;
};

template <class T>
class BinListNode{
	public:
		BinListNode (const T& item) : item_ (item), left_ (NULL), right_ (NULL) { }
		~BinListNode(){
			if (left_) delete left_;
			if (right_) delete right_;
		}

		void setLeft (BinListNode<T>* left){ left_ = left; }
		BinListNode<T>* getLeft(){ return left_; }
		void setRight (BinListNode<T>* right){ right_ = right; }
		BinListNode<T>* getRight(){ return right_; }
		void setItem (const T& item){ item_ = item; }
		const T getItem() const{ return item_; }

		void append (const T& item, const Interface_IndexEvaluator<T>& indexEvaluator){
			if (indexEvaluator.lessThan (item, item_)){
				if (!left_) left_ = new BinListNode<T> (item);
				else left_->append (item, indexEvaluator);
			}
			else{
				if (!right_) right_ = new BinListNode<T> (item);
				else right_->append (item, indexEvaluator);
			}
		}
		bool hasItem (const T& item, const Interface_IndexEvaluator<T>& indexEvaluator) const{
			if (indexEvaluator.equals (item, item_)) return true;
			if (indexEvaluator.lessThan (item, item_)){
				if (left_) return left_->hasItem (item, indexEvaluator);
				return false;
			}
			if (right_) return right_->hasItem (item, indexEvaluator);
			return false;
		}
		T popFront(){
			return pop (true);
		}
		T pop (const bool& front = false){
			if (!front){
				if (!right_->getRight()){
					T item = right_->getItem();
					BinListNode<T>* newRight = right_->getLeft();
					right_->setLeft (NULL);
					delete right_;
					right_ = newRight;
					return item;
				}
				else return right_->pop (front);
			}
			else{
				if (!left_->getLeft()){
					T item = left_->getItem();
					BinListNode<T>* newLeft = left_->getRight();
					left_->setRight (NULL);
					delete left_;
					left_ = newLeft;
					return item;
				}
				else return left_->pop (front);
			}
		}
		bool remove (const T& item, const Interface_IndexEvaluator<T>& indexEvaluator){
			BinListNode<T>** dyingChild = NULL;
			if (indexEvaluator.lessThan (item, item_)){
				if (left_){
					if (indexEvaluator.equals (item, left_->getItem())) dyingChild = &left_;
					else return left_->remove (item, indexEvaluator);
				}
				else return false;
			}
			else{
				if (right_){
					if (indexEvaluator.equals (item, right_->getItem())) dyingChild = &right_;
					else return right_->remove (item, indexEvaluator);
				}
				else return false;
			}
			// Dying child will always be set!
			BinListNode<T>* dyingLeft = (*dyingChild)->getLeft();
			BinListNode<T>* dyingRight = (*dyingChild)->getRight();
			(*dyingChild)->setLeft (NULL); (*dyingChild)->setRight (NULL); delete (*dyingChild);
			*dyingChild = NULL;
			if (dyingLeft) insertNode (dyingLeft, indexEvaluator);
			if (dyingRight) insertNode (dyingRight, indexEvaluator);
			return true;
		}
		void getAll (std::vector<T>& runningList){
			if (left_) left_->getAll (runningList);
			runningList.push_back (item_);
			if (right_) right_->getAll (runningList);
		}

		void insertNode (BinListNode<T>* node, const Interface_IndexEvaluator<T>& indexEvaluator){
			if (indexEvaluator.lessThan (node->getItem(), item_)){
				if (left_) left_->insertNode (node, indexEvaluator);
				else left_ = node;
			}
			else{
				if (right_) right_->insertNode (node, indexEvaluator);
				else right_ = node;
			}
		}
	
	private:
		T item_;
		BinListNode<T>* left_;
		BinListNode<T>* right_;
};

template <class T>
class BinList{
	public:
		BinList (const Interface_IndexEvaluator<T>* indexEvaluator) : indexEvaluator_ (indexEvaluator), root_ (NULL), itemCount_ (0) { }
		~BinList(){
			if (root_) delete root_;
		}

		bool remove (const T& item){
			if (root_){
				if (indexEvaluator_->equals (item, root_->getItem())){
					BinListNode<T>* newRoot (NULL);
					BinListNode<T>* movingNode (NULL);
					if (root_->getLeft()){
						if (root_->getRight()){
							if (rand() % 2){
								newRoot = root_->getLeft(); 
								movingNode = root_->getRight();
							}
							else{
								newRoot = root_->getRight();
								movingNode = root_->getLeft();
							}
						}
						else{
							newRoot = root_->getLeft();
						}
					}
					else newRoot = root_->getRight();
					itemCount_--;
					root_->setLeft (NULL); root_->setRight (NULL); delete root_; root_ = newRoot;
					if (movingNode) root_->insertNode (movingNode, *indexEvaluator_);
					return true;
				}
				else{
					if (root_->remove (item, *indexEvaluator_)){
						itemCount_--;
						return true;
					}
				}
			}
			return false;
		}
		void append (const T& item){
			itemCount_++;
			if (root_){
				root_->append (item, *indexEvaluator_);
			}
			else{
				root_ = new BinListNode<T> (item);
			}
		}
		bool hasItem (const T& item) const{
			if (!root_) return false;
			return root_->hasItem (item, *indexEvaluator_);
		}
		T popFront(){
			return pop (true);
		}
		T pop (const bool& front = false){
			itemCount_--;
			if (!front){
				if (!root_->getRight()){
					T item = root_->getItem();
					BinListNode<T>* newRoot = root_->getLeft();
					root_->setLeft (NULL);
					delete root_;
					root_ = newRoot;
					return item;
				}
				else return root_->pop (front);
			}
			else{
				if (!root_->getLeft()){
					T item = root_->getItem();
					BinListNode<T>* newRoot = root_->getRight();
					root_->setRight (NULL);
					delete root_;
					root_ = newRoot;
					return item;
				}
				else return root_->pop (front);
			}
		}
		void getAll (std::vector<T>& runningList){
			if (root_) root_->getAll (runningList);
		}
		unsigned int size() const{
			return itemCount_;
		}

	private:
		const Interface_IndexEvaluator<T>* indexEvaluator_;
		BinListNode<T>* root_;
		unsigned int itemCount_;
};


#endif
