#include "geometry.h"
#include <math.h>
#include "utils.h"


Vector2::Vector2 (const double& x, const double& y) : x_ (x), y_ (y) { }
Vector2::~Vector2(){}

Vector2 Vector2::getNormalized(){
	if (getLength() == 0) return Vector2 (0, 0);
	double multiplier = 1.0 / getLength();
	return Vector2 (x_ * multiplier, y_ * multiplier);
}
double Vector2::getLength(){
	//if (!length_.isSet()) length_ = Optional<double> (Utils::fastFloatSqrt (x_ * x_ + y_ * y_));
	if (!length_.isSet()) length_ = Optional<double> (sqrt (x_ * x_ + y_ * y_));
	return length_.get();
}
Vector2 Vector2::getInverted() const{
	return Vector2 (x_ * -1, y_ * -1);
}
double Vector2::dotProduct (Vector2& otherVector){
	double lengthSquared = x_ * x_ + y_ * y_;
	double otherLengthSquared = otherVector.x_ * otherVector.x_ + otherVector.y_ * otherVector.y_;
	//if (getLength() == 0 || otherVector.getLength() == 0) return 0;
	if (lengthSquared == 0 || otherLengthSquared == 0) return 0;
	//double dotNumerator = (x_ * otherVector.x_ + y_ * otherVector.y_);
	//return (dotNumerator * dotNumerator) / (lengthSquared * otherLengthSquared);
	//return (getX() * otherVector.getX() + getY() * otherVector.getY()) / (getLength() * otherVector.getLength());
	return (getX() * otherVector.getX() + getY() * otherVector.getY()) / (sqrt (lengthSquared) * sqrt (otherLengthSquared));

	//return (getX() * otherVector.getX() + getY() * otherVector.getY()) / (Utils::fastFloatSqrt (lengthSquared) * Utils::fastFloatSqrt (otherLengthSquared));

	//return (x_ * otherVector.x_ + y_ * otherVector.y_) / (Utils::fastFloatSqrt (lengthSquared) * Utils::fastFloatSqrt (otherLengthSquared));
}
bool Vector2::equals (const Vector2& otherVector) const{
	return fabs (getX() - otherVector.getX()) < .0001 && fabs (getY() - otherVector.getY()) < .0001;
}
Vector2 Vector2::getRotated (const double& angle) const{
	return Vector2 (cos (angle) * getX() - sin (angle) * getY(), sin (angle) * getX() + cos (angle) * getY());
}
std::string Vector2::toString() const{
	char msg [1024];
	sprintf (msg, "(%f,%f)", x_, y_);
	return std::string (msg);
}
double Vector2::getX() const{ return x_; }
double Vector2::getY() const{ return y_; }


Point::Point (const double& x, const double& y) : x_ (x), y_ (y) { }
Point::Point() : x_ (0), y_ (0) { }
Point::~Point(){}

bool Point::operator< (const Point& rhs) const{
    return (y_ < rhs.y_) || (y_ == rhs.y_ && x_ < rhs.x_);
	/*
    int compareY = Utils::compareDoubles (y_, rhs.y_);
    int compareX = Utils::compareDoubles (x_, rhs.x_);
    return (compareY == -1) || (compareY == 0 && compareX == -1);
	*/
}
bool Point::operator== (const Point& rhs) const{
	return y_ == rhs.y_ && x_ == rhs.x_;
	/*
    int compareY = Utils::compareDoubles (y_, rhs.y_);
    int compareX = Utils::compareDoubles (x_, rhs.x_);
    return (compareY == 0) && (compareX == 0);
	*/
}

double Point::getX() const{ return x_; }
double Point::getY() const{ return y_; }
		
Vector2 Point::vectorToOtherPoint (const Point& otherPoint) const{
	return Vector2 (otherPoint.getX() - getX(), otherPoint.getY() - getY());
}
bool Point::equals (const Point& otherPoint) const{
	return getX() == otherPoint.getX() && getY() == otherPoint.getY();
}
std::string Point::toString() const{
	char x [1024];
	char y [1024];
	sprintf (x, "%f", x_);
	sprintf (y, "%f", y_);
	return std::string ("(") + std::string (x) + std::string (", ") + std::string (y) + std::string (")");
}


Line::Line (const Point& point1, const Point& point2) : point1_ (point1), point2_ (point2) { }
Line::Line() : point1_ (0, 0), point2_ (0, 0) { }
Line::~Line(){}

bool Line::equals (const Line& otherLine) const{
	if (point1_.equals (otherLine.point1_) && point2_.equals (otherLine.point2_)) return true;
	if (point2_.equals (otherLine.point1_) && point2_.equals (otherLine.point1_)) return true;
	return false;
}
bool Line::intersectsLine (const Line& otherLine) const{
	Optional<Point> intersectPoint = getLineIntersectionPoint (otherLine);
	if (intersectPoint.isSet()) return true;
	return false;
}
double Line::getLength() const{
	return point1_.vectorToOtherPoint (point2_).getLength();
}
double Line::distanceToPoint (const Point& point, const double& tolerance) const{
	Vector2 p1ToPoint (point1_.vectorToOtherPoint (point));
	Vector2 p2ToPoint (point2_.vectorToOtherPoint (point));
	double dot1 = point1_.vectorToOtherPoint (point2_).dotProduct (p1ToPoint);
	double dot2 = point2_.vectorToOtherPoint (point1_).dotProduct (p2ToPoint);
	if (1.0 - fabs (dot1) < tolerance || 1.0 - fabs (dot2) < tolerance){
		if ( point.getX() <= max (point1_.getX(), point2_.getX()) && point.getX() >= min (point1_.getX(), point2_.getX()) 
			&& point.getY() <= max (point1_.getY(), point2_.getY()) && point.getY() >= min (point1_.getY(), point2_.getY()) ){
			return 0;
		}
	}
	//if (fabs (dot1) >= sqrt (2) / 2.0 || fabs (dot2) >= sqrt (2) / 2.0){
	if (dot1 < 0.0 || dot2 < 0.0){
		//cout << ", dot1 " << dot1 << ", dot2 " << dot2 << endl;
		return min (point.vectorToOtherPoint (point1_).getLength(), point.vectorToOtherPoint (point2_).getLength());
	}
	double angle = acos (dot1);
	double hypotenuse = point.vectorToOtherPoint (point1_).getLength();
	//cout << "angle " << angle << ", hypotenuse " << hypotenuse << ", dot1 " << dot1 << ", dot2 " << dot2 << ", sin (angle) " << sin (angle) << endl;
	return sin (angle) * hypotenuse;
}
Optional<Point> Line::getLineIntersectionPoint (const Line& otherLine) const{
	if (point1_.getX() == point2_.getX() && point1_.getY() == point2_.getY()){
		if (otherLine.point1_.getX() == otherLine.point2_.getX() && otherLine.point1_.getY() == otherLine.point2_.getY()){
			if ((fabs (point1_.getX() - otherLine.point1_.getX()) < .0001) && (fabs (point1_.getY() - otherLine.point1_.getY()) < .0001)) return point1_;
			else return Optional<Point>();
		}
		return otherLine.getLineIntersectionPoint (*this);
	}
	double a1 = point2_.getY() - point1_.getY();
	double b1 = point1_.getX() - point2_.getX();
	double c1 = (a1 * point1_.getX()) + (b1 * point1_.getY());
	double a2 = otherLine.getPoint2().getY() - otherLine.getPoint1().getY();
	double b2 = otherLine.getPoint1().getX() - otherLine.getPoint2().getX();
	double c2 = (a2 * otherLine.getPoint1().getX()) + (b2 * otherLine.getPoint1().getY());
	double det = (a1 * b2) - (a2 * b1);
	if (det == 0){ // If the lines are parallel (det == 0), there's still a chance they overlap.
		const Point& p1 = point1_;
		const Point& p2 = point2_;
		Point op1 = otherLine.getPoint1();
		Point op2 = otherLine.getPoint2();
		const Point& pa = p1;
		const Point& pb = p2;
		std::vector<Point> points;
		points.push_back (op1);
		points.push_back (op2);
		for (std::vector<Point>::iterator op = points.begin(); op != points.end(); op++){
			Vector2 vec1 = pa.vectorToOtherPoint (*op).getNormalized();
			Vector2 vec2 = op->vectorToOtherPoint (pb).getNormalized();
			if (vec1.equals (vec2) || vec1.equals (vec2.getInverted()) || op->equals (pa) || op->equals (pb)){
				if (p1.vectorToOtherPoint (op1).getLength() < p1.vectorToOtherPoint (op2).getLength()){
					return Optional<Point> (op1);
				}
				else{
					return Optional<Point> (op2);
				}
			}
		}
		return Optional<Point>();
	}
	else{
		double xIntersect = ((b2 * c1) - (b1 * c2)) / det;
		double yIntersect = ((a1 * c2) - (a2 * c1)) / det;
		bool xOkay = true;
		bool yOkay = true;
		std::vector<Point> pointPairs;
		pointPairs.push_back (point1_);
		pointPairs.push_back (point2_);
		pointPairs.push_back (otherLine.getPoint1());
		pointPairs.push_back (otherLine.getPoint2());				
		for (std::vector<Point>::iterator op = pointPairs.begin(); op != pointPairs.end(); op += 2){
			Point point1 (*op);
			Point point2 (*(op + 1));
			xOkay = xOkay && std::min (point1.getX(), point2.getX()) <= xIntersect && xIntersect <= std::max (point1.getX(), point2.getX());
			yOkay = yOkay && std::min (point1.getY(), point2.getY()) <= yIntersect && yIntersect <= std::max (point1.getY(), point2.getY());
		}
		if (xOkay && yOkay){
			Point intersect (xIntersect, yIntersect);
			return Optional<Point> (intersect);
		}
		else{
			return Optional<Point>();
		}
	}
}
std::string Line::toString() const{
	return "[" + point1_.toString() + ", " + point2_.toString() + "]";
}
Point Line::getPoint1() const{ return point1_; }
Point Line::getPoint2() const{ return point2_; }
