#include <iostream>
#include <SDL/SDL.h>
#include "imageProcessingSurfaceManager.h"
#include "rect.h"
#include "screen.h"
using namespace std;


ImageProcessingSurfaceManager::ImageProcessingSurfaceManager (SDL_Surface* originalSurface, const Rect& screenRect){
	construct (originalSurface, screenRect);
}
ImageProcessingSurfaceManager::~ImageProcessingSurfaceManager(){
	destruct();
}
ImageProcessingSurfaceManager::ImageProcessingSurfaceManager (const ImageProcessingSurfaceManager& model){
	construct (model.getTargetSurface(), getScreenRect());
}
ImageProcessingSurfaceManager& ImageProcessingSurfaceManager::operator= (const ImageProcessingSurfaceManager& model){
	if (&model == this) return *this;
	destruct();
	construct (model.getTargetSurface(), model.getScreenRect());
	return *this;
}

void ImageProcessingSurfaceManager::construct (SDL_Surface* targetSurface, const Rect& screenRect){
	reset();

	this->targetSurface = targetSurface;
	
	effectSurface = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, targetSurface->w, targetSurface->h, 
		targetSurface->format->BitsPerPixel, 
		targetSurface->format->Rmask, targetSurface->format->Gmask, targetSurface->format->Bmask, targetSurface->format->Amask);
	SDL_SetColorKey (effectSurface, SDL_SRCCOLORKEY, EFFECT_SURFACE_COLORKEY);
	SDL_FillRect (effectSurface, NULL, EFFECT_SURFACE_COLORKEY);
	
	this->screenRect = screenRect;
}
void ImageProcessingSurfaceManager::destruct(){
	SDL_FreeSurface (effectSurface);
}

void ImageProcessingSurfaceManager::growCompositeRect (const int& x, const int& y){
	growCompositeRect (Rect (x, y, 1, 1));
}
void ImageProcessingSurfaceManager::growCompositeRect (const Rect& include){
	//cout << "growing to include " << include.toString() << endl;
	if (compositeRect.getWidth() < 1 || compositeRect.getHeight() < 1) compositeRect = include;
	else compositeRect.stretchToInclude (include);
	//cout << "grew to " << compositeRect.toString() << endl;
}
void ImageProcessingSurfaceManager::setCompositePixel (const unsigned int& x, const unsigned int& y, const Uint16& newPixel){
	if (x >= (unsigned int) effectSurface->w || y >= (unsigned int) effectSurface->h) return;
	//cout << "Setting pixel at " << x << "," << y << " to " << newPixel << endl;
	growCompositeRect (x, y);
	Utils::setPixel (effectSurface, x, y, newPixel);
}
void ImageProcessingSurfaceManager::blitToComposite (SDL_Surface* sourceSurface, Rect sourceRect, Rect targetRect){
	SDL_Rect source = sourceRect.toSDL_Rect();
	SDL_Rect target = targetRect.toSDL_Rect();
	SDL_BlitSurface (sourceSurface, &source, effectSurface, &target);
	//cout << "blitToComposite sees source " << sourceRect.toString() << endl;
	//cout << "blitToComposite sees target " << targetRect.toString() << endl;
	//cout << "blitToComposite effect surface size is " << effectSurface->w << "," << effectSurface->h << endl;
	growCompositeRect (Rect (targetRect.getX(), targetRect.getY(), sourceRect.getWidth(), sourceRect.getHeight()));
}
Uint16 ImageProcessingSurfaceManager::getBlittingPixel (const unsigned int& x, const unsigned int& y) const{
	if (blittingSurface == NULL) return 0;
	if ((int) x < blittingSurfaceRect.x || (int) y < blittingSurfaceRect.y 
			|| (int) x >= blittingSurfaceRect.x + blittingSurfaceRect.w || (int) y >= blittingSurfaceRect.y + blittingSurfaceRect.h){
		return blittingSurface->format->colorkey;
	}
	//cout << "getting blitting pixel at " << x - blittingSurfaceRect.x + blittingSurfaceClipRect.x << "," 
	//	<< y - blittingSurfaceRect.y + blittingSurfaceClipRect.y << endl;
	return Utils::getPixel (blittingSurface, x - blittingSurfaceRect.x + blittingSurfaceClipRect.x, y - blittingSurfaceRect.y + blittingSurfaceClipRect.y);
}
Uint16 ImageProcessingSurfaceManager::getEffectPixel (const unsigned int& x, const unsigned int& y) const{
	if (x >= (unsigned int) effectSurface->w || y >= (unsigned int) effectSurface->h) return EFFECT_SURFACE_COLORKEY;
	return Utils::getPixel (effectSurface, x, y);
}
Uint16 ImageProcessingSurfaceManager::getCompositePixel (const unsigned int& x, const unsigned int& y) const{
	Uint16 foundPixel = getEffectPixel (x, y);
	if (foundPixel != EFFECT_SURFACE_COLORKEY) return foundPixel;
	return getBlittingPixel (x, y);
}
bool ImageProcessingSurfaceManager::blittingPixelIsTransparent (unsigned int x, unsigned int y) const{
	return !blittingSurface || (getBlittingPixel (x, y) == blittingSurface->format->colorkey);
}
bool ImageProcessingSurfaceManager::compositePixelIsTransparent (unsigned int x, unsigned int y) const{
	//cout << "effect pixel is " << getEffectPixel (x, y);
	//cout << " blitting pixel is " << getBlittingPixel (x, y) << " is transparent? " << endl;
	if (getEffectPixel (x, y) != EFFECT_SURFACE_COLORKEY) return false;
	return blittingPixelIsTransparent (x, y);
}
Uint16 ImageProcessingSurfaceManager::getTargetPixel (const unsigned int& x, const unsigned int& y) const{
	//cout << "Hit a set pixel at " << x << "," << y << endl;
	if (x >= (unsigned int) getTargetSurface()->w || y >= (unsigned int) getTargetSurface()->h) return getTargetSurface()->format->colorkey;
	return Utils::getPixel (getTargetSurface(), x, y);
}

void ImageProcessingSurfaceManager::endEffectStack (const vector<Rect>& viewports, const double& offsetX, const double& offsetY){
	//cout << "Composite rect is " << compositeRect.toString() << endl;
	Rect drawBox = compositeRect;
	drawBox.setXY (drawBox.getX() + offsetX, drawBox.getY() + offsetY);
	//cout << "Draw box is " << drawBox.toString() << endl;
	//cout << "I have " << viewports.size() << " viewports." << endl;
	for (vector<Rect>::const_iterator iter = viewports.begin(); iter != viewports.end(); iter++){
		//cout << "Next viewport is " << iter->toString() << endl;
		drawBox.trimTo (*iter);
	}
	//cout << "Draw box was trimmed to " << drawBox.toString() << endl;
	if (drawBox.getWidth() > 0 && drawBox.getHeight() > 0){
		//cout << "Target at " << drawBox.getX() << "," << drawBox.getY() << " is " << Utils::getPixel (targetSurface, int (drawBox.getX()), int (drawBox.getY())) << endl;
		SDL_BlitSurface (effectSurface, &(drawBox.toSDL_Rect()), targetSurface, &(drawBox.toSDL_Rect()));
		//cout << "Target at " << drawBox.getX() << "," << drawBox.getY() << " now " << Utils::getPixel (targetSurface, int (drawBox.getX()), int (drawBox.getY())) << endl;
		dirtyEffectRects.push_back (drawBox.toSDL_Rect());
	}
	SDL_FillRect (effectSurface, &(compositeRect.toSDL_Rect()), EFFECT_SURFACE_COLORKEY);
	compositeRect = Rect (0, 0, 0, 0);
}

void ImageProcessingSurfaceManager::startEffectStack (SDL_Surface* blittingSurface, SDL_Rect blittingSurfaceRect, SDL_Rect blittingSurfaceClipRect, 
		const vector<Rect*>* dirtyScreenRects){

	//cout << "blitting surface rect is " << blittingSurfaceRect.x << "," << blittingSurfaceRect.y << endl;

	this->blittingSurface = blittingSurface;
	this->blittingSurfaceRect.x = blittingSurfaceRect.x;
	this->blittingSurfaceRect.y = blittingSurfaceRect.y;
	this->blittingSurfaceRect.w = blittingSurfaceRect.w;
	this->blittingSurfaceRect.h = blittingSurfaceRect.h;
	this->blittingSurfaceClipRect.x = blittingSurfaceClipRect.x;
	this->blittingSurfaceClipRect.y = blittingSurfaceClipRect.y;
	this->blittingSurfaceClipRect.w = blittingSurfaceClipRect.w;
	this->blittingSurfaceClipRect.h = blittingSurfaceClipRect.h;

	this->dirtyScreenRects.clear();
	if (dirtyScreenRects != NULL) this->dirtyScreenRects = *dirtyScreenRects;
}

void ImageProcessingSurfaceManager::reset(){
	compositeRect = Rect (0, 0, 0, 0);
	dirtyEffectRects.clear();
	dirtyScreenRects.clear();
}
