#ifndef MENU_H
#define MENU_H


#include <string>
#include "sprite.h"
#ifdef WIN32
	#include <memory>
#else
	#include <tr1/memory>
#endif
#include "text.h"
#include "effectFadeInOut.h"
#include "interface_inputHandlerObserver.h"
#include "interface_menuObserver.h"
#include <algorithm>

class MenuOption{
	public:
		MenuOption (const std::string& id, const std::string& displayName) : id_ (id), displayName_ (displayName) { }
		~MenuOption(){}

		std::string getId() const{ return id_; }
		std::string getDisplayName() const{ return displayName_; }

	private:
		std::string id_;
		std::string displayName_;
};

class InputHandler;
class Menu : public Sprite, public Interface_InputHandlerObserver{
	public:
		Menu (Surface* owner, double x, double y, const std::string& fontSet) : Sprite (owner){
			setXY (x, y);
			fontSet_ = fontSet;
			currentSelection_ = 0;
			resetDisplay();
		}
		virtual ~Menu(){}

		virtual void inputHandlerUpdate (const KeyStatesSet& keyStates);

		void addOption (const MenuOption& newOption){
			menuOptions_.push_back (newOption);
			resetDisplay();
		}
		void selectNext(){
			if (menuOptions_.size() == 0) return;
			setCurrentSelection ((currentSelection_ + 1) % menuOptions_.size());
			resetDisplay();
		}
		void selectPrevious(){
			if (menuOptions_.size() == 0) return;
			if (currentSelection_ == 0) setCurrentSelection (menuOptions_.size() - 1);
			else setCurrentSelection (currentSelection_ - 1);
			resetDisplay();
		}
		void chooseSelected(){
			if (currentSelection_ < 0 || menuOptions_.size() < 1 || currentSelection_ >= menuOptions_.size()) return;
			std::vector<Interface_MenuObserver*> observersCopy = observers_;
			for (std::vector<Interface_MenuObserver*>::iterator iter = observersCopy.begin(); iter != observersCopy.end(); iter++){
				(*iter)->menuOptionChosen (this, &(menuOptions_ [currentSelection_]));
			}
		}
		void addObserver (Interface_MenuObserver* newObserver){
			observers_.push_back (newObserver);
		}
		void removeObserver (Interface_MenuObserver* observer){
			std::vector<Interface_MenuObserver*>::iterator found = std::find (observers_.begin(), observers_.end(), observer);
			if (found != observers_.end()) observers_.erase (found);
		}
		MenuOption getSelected() const{
			return menuOptions_ [currentSelection_];
		}

	private:
		void setCurrentSelection (unsigned int selection){
			if (selection < 0 || selection >= menuOptions_.size()) return;
			currentSelection_ = selection;
			//cout << "current selection is now " << currentSelection_ << endl;
		}
		void clearOptionDisplays(){
			for (std::vector<Text*>::iterator iter = myOptionDisplays_.begin(); iter != myOptionDisplays_.end(); iter++){
				if (*iter) (*iter)->setLive (false);
			}
			myOptionDisplays_.clear();
		}
		void resetDisplay(){
			clearOptionDisplays();
			double currentHeight = 0;
			double yMargin = 10;
			unsigned int optionIndex = 0;
			double maxX = 0;
			for (std::vector<MenuOption>::iterator iter = menuOptions_.begin(); iter != menuOptions_.end(); iter++){
				Text* newText = new Text (this, fontSet_, iter->getDisplayName());
				newText->setXY (0, currentHeight);
				currentHeight += newText->getRect()->getHeight() + yMargin;
				maxX = max (maxX, newText->getX() + newText->getRect()->getWidth());
				EffectFadeInOut* existingEffect = appliedEffects_ [iter->getId()];
				EffectFadeInOut* newEffect = NULL;
				if (currentSelection_ != optionIndex){
					//cout << "making option translucent" << endl;
					if (existingEffect)	newEffect = existingEffect->clone();
					else newEffect = new EffectFadeInOut (0.5, 100000.0, 0.5, 1.0, 0.5);
					newText->addVisualEffect (newEffect);
				}
				appliedEffects_ [iter->getId()] = newEffect;
				myOptionDisplays_.push_back (newText);
				optionIndex++;
			}
			getRect()->setWidth (maxX);
			getRect()->setHeight (currentHeight);
		}

		std::vector<Interface_MenuObserver*> observers_;
		std::vector<Text*> myOptionDisplays_;
		std::vector<MenuOption> menuOptions_;
		std::string fontSet_;
		unsigned int currentSelection_;
		std::map<std::string,EffectFadeInOut*> appliedEffects_;
};



#endif
