#include "microTileField.h"
#include "math.h"
#include <list>
#include <algorithm>
using namespace std;


#ifdef MICROTILEFIELD_MEMCHECK
	#define MALLOC myMalloc
	#define FREE myFree
#else
	#define MALLOC malloc
	#define FREE free
#endif


MicroTileField::MicroTileField (unsigned int width, unsigned int height, unsigned int widthInTiles, unsigned int heightInTiles){
	runningHorizontalMask = (TILE_ENTRY_TYPE (0x01)) << (BITS_PER_TILE_ENTRY - 1);
	hasCompiled = false;
	this->width = width; this->height = height;
	this->widthInTiles = widthInTiles; this->heightInTiles = heightInTiles;
	myPerimeter = Rect (0, 0, width, height);
    unsigned int horizontalTileSize = width / widthInTiles;
    unsigned int verticalTileSize = height / heightInTiles;
    if (horizontalTileSize < 1 || verticalTileSize < 1){
        //string msg = "MicroTileField height less than one!";
		char msg [1000]; 
        sprintf (msg, "MicroTileField::MicroTileField (%d, %d, %d, %d) makes the tile width or height less than one!", width, height, widthInTiles, heightInTiles);
		cout << msg << endl;
        throw msg;
    }
    else{
		initLookup();
		initMarkField();
		initCompilation();
		initDirtyMaskLookup();
    }
}
MicroTileField::~MicroTileField(){ 
	reset();
	killMarkField(); killLookup(); killCompilation(); killDirtyMaskLookup(); 
	#ifdef MICROTILEFIELD_MEMCHECK
		checkMem();
	#endif
}

void MicroTileField::initMarkField(){
	unsigned int tileEntries = (widthInTiles / BITS_PER_TILE_ENTRY) + (widthInTiles % BITS_PER_TILE_ENTRY > 0);
	markField = (TILE_ENTRY_TYPE**) MALLOC (sizeof (TILE_ENTRY_TYPE*) * (heightInTiles + 1));
	for (unsigned int y = 0; y < heightInTiles; y++){
		markField [y] = (TILE_ENTRY_TYPE*) MALLOC (sizeof (TILE_ENTRY_TYPE) * (tileEntries + 1));
		for (unsigned int x = 0; x < tileEntries; x++){
			markField [y][x] = 0x0;
		}
	}
}
void MicroTileField::killMarkField(){
	for (unsigned int y = 0; y < heightInTiles; y++){
		if (markField [y]) FREE (markField [y]);
		markField [y] = NULL;
	}
	FREE (markField); markField = NULL;
}
void MicroTileField::initCompilation(){
	compilation = new RectQuadTree();
	#ifdef MICROTILEFIELD_MEMCHECK
		iAllocated (compilation);
		//cout << "Allocated " << compilation << " at " << __LINE__ << endl;
	#endif
}
void MicroTileField::killCompilation(){
	#ifdef MICROTILEFIELD_MEMCHECK
		iDeallocated (compilation);
	#endif
	delete compilation; compilation = NULL;
}
void MicroTileField::initLookup(){
	for (unsigned int x = 0; x < TILE_ENTRY_STATES; x++){
		bool running = false;
		int index = -1;
		TILE_ENTRY_TYPE mask = 0x00;
		for (int y = 0; y < BITS_PER_TILE_ENTRY / 2 + 1; y++){
			lookup [x][y].start = -1; 
		}
		for (int y = BITS_PER_TILE_ENTRY - 1; y >= 0; y--){
			mask = (0x01 << y);
			if (x & mask){
				if (!running){
					index++;
					lookup [x][index].start = (BITS_PER_TILE_ENTRY - 1) - y;
					lookup [x][index].width = 0;
					running = true;
				}
				lookup [x][index].mask |= mask;
				lookup [x][index].width++;
			}
			else{
				running = false;
			}
		}
	}
}
void MicroTileField::killLookup(){}

void MicroTileField::initDirtyMaskLookup(){
	TILE_ENTRY_TYPE mask = ~((TILE_ENTRY_TYPE) 0x00);
	for (int x = 0; x <= BITS_PER_TILE_ENTRY; x++){
		dirtyMaskLookup [0][x] = mask;
		mask = mask >> 1;
	}
	mask = ~(TILE_ENTRY_TYPE (0x0));
	for (int x = BITS_PER_TILE_ENTRY; x >= 0; x--){
		dirtyMaskLookup [1][x] = mask;
		mask = mask << 1;
	}
}
void MicroTileField::killDirtyMaskLookup(){
}

void MicroTileField::dirty (const vector<Rect>& areas){
	//cout << "Dirtying" << endl;
	for (vector<Rect>::const_iterator next = areas.begin(); next != areas.end(); next++){
		//cout << "next is " << next->toString() << endl;
		//cout << "my perimeter is " << myPerimeter.toString() << endl;
		if (next->intersects (myPerimeter) && next->getWidth() > 0 && next->getHeight() > 0){
			hasCompiled = false;
			Rect intersecting = getBoxesIntersecting (*next);
			//cout << "intersecting is " << intersecting.toString() << endl;
			int startY = int (intersecting.getY());
			int widthY = int (intersecting.getHeight());
			int endY = min (startY + widthY, int (height));
			int startX = max (0, int (intersecting.getX()));
			int startXByte = startX / BITS_PER_TILE_ENTRY;
			short int startXBitOffset = startX % BITS_PER_TILE_ENTRY;
			int widthX = max (0, int (intersecting.getWidth()));
			int endX = min (startX + widthX, int (width));
			int endXByte = endX / BITS_PER_TILE_ENTRY;
			short int endXBitOffset = endX % BITS_PER_TILE_ENTRY;
			//cout << "startY " << startY << " endY " << endY << " startXByte " << startXByte << " endXByte " << endXByte 
			//	 << " startX " << startX << " endX " << endX << endl;
			//cout << "startXBitOffset " << startXBitOffset << " " << tileEntryToString (dirtyMaskLookup [0][startXBitOffset]) 
			//	 << " endXBitOffset " << endXBitOffset << " " << tileEntryToString (dirtyMaskLookup [1][endXBitOffset]) << endl;
			for (int y = startY; y < endY; y++){
				// Handle the first part of bitstring.
				if (startXByte == endXByte){
					markField [y][startXByte] |= (dirtyMaskLookup [0][startXBitOffset] & dirtyMaskLookup [1][endXBitOffset]);
				}
				else{
					markField [y][startXByte] |= dirtyMaskLookup [0][startXBitOffset];
					for (int x = startXByte + 1; x < endXByte; x++){
						// Handle middle (all set bits) of bitstring.
						markField [y][x] |= dirtyMaskLookup [0][0];
					}
					// Handle the remaining part of bitstring.
					markField [y][endXByte] |= (dirtyMaskLookup [0][0] & dirtyMaskLookup [1][endXBitOffset]);
					//00000000
					//11111111 10000000
				}
			}
		}
		//else cout << "MicroTileField::dirty(): doesn't intersect!" << endl;
	}
}
Rect MicroTileField::getBoxesIntersecting (const Rect& intersecting){
	int xPixels = getHorizontalPixelsPerBox();
	int yPixels = getVerticalPixelsPerBox();
	int startX = max (int (intersecting.getX()) / xPixels, 0);
	int startY = max (int (intersecting.getY()) / yPixels, 0);
	int endX = min (int (intersecting.getX() + intersecting.getWidth()), int (width));
	endX = endX / xPixels + (endX % xPixels > 0);
	int endY = min (int (intersecting.getY() + intersecting.getHeight()), int (height)); 
	endY = endY / yPixels + (endY % yPixels > 0);
	int width = endX - startX;
	int height = endY - startY;
	Rect retRect (startX, startY, width, height);
	//cout << "Boxes intersecting: " << retRect.toString() << endl;
	return retRect;
}

void MicroTileField::placeInCompilation (Rect* toPlace){
	//cout << "Placing " << toPlace->toString() << " in compilation " << endl;
	compilation->insert (toPlace);
	//cout << "Done placing in compilation " << endl;
}

void MicroTileField::compile(){
	//cout << "Compile called " << endl;
	unsigned int tilesPerRow = (widthInTiles / BITS_PER_TILE_ENTRY) + (widthInTiles % BITS_PER_TILE_ENTRY > 0);

	Rect** rects = (Rect**) calloc (tilesPerRow * BITS_PER_TILE_ENTRY, sizeof (Rect*));
	for (unsigned int y = 0; y < tilesPerRow * BITS_PER_TILE_ENTRY; y++){
		rects [y] = NULL;
	}
	unsigned int tileWidth = width / widthInTiles;
	unsigned int tileHeight = height / heightInTiles;

	//cout << "AAAAAAAHHHH" << endl;
	//cout << markFieldToString (markField) << endl;

	//cout << "tilesPerRow " << tilesPerRow << " heightInTiles " << heightInTiles << " widthInTiles " << widthInTiles << endl;

	Rect* runningHorizontalRect = NULL;

	// Start moving down.
	for (unsigned int y = 0; y < heightInTiles; y++){
		//cout << "y is " << y << endl;
		if (runningHorizontalRect){
			#ifdef MICROTILEFIELD_MEMCHECK
				iDeallocated (runningHorizontalRect);
			#endif
			delete runningHorizontalRect;
			runningHorizontalRect = NULL;
		}
		for (unsigned int xTile = 0; xTile < tilesPerRow; xTile++){
			//cout << "xTile is " << xTile << endl;
			// If the entire byte is clear, skip it; there are no set tiles in that group.
			if (!markField [y][xTile]){
				if (runningHorizontalRect){
					int slot = int (runningHorizontalRect->getX()) / tileWidth;
					if (rects [slot]){
						if ((rects [slot]->getWidth() == runningHorizontalRect->getWidth())
								&& (rects [slot]->getHeight() + rects [slot]->getY() == runningHorizontalRect->getY())){
							rects [slot]->setHeight (rects [slot]->getHeight() + tileHeight);
							#ifdef MICROTILEFIELD_MEMCHECK
								iDeallocated (runningHorizontalRect);
							#endif
							delete runningHorizontalRect; runningHorizontalRect = NULL;
						}
						else{
							placeInCompilation (rects [slot]);
							rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
						}
					}
					else{ 
						rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
					}
				}
				continue;
			}
			LookupRec* nextLookupRec = &(lookup [ markField [y][xTile] ][0]);
			if (runningHorizontalRect){
				if (xTile > 0 && (markField [y][xTile] & runningHorizontalMask) && (markField [y][xTile - 1] & 0x01)){
					runningHorizontalRect->setWidth (runningHorizontalRect->getWidth() + (nextLookupRec->width * tileWidth));
					nextLookupRec++;
					if (nextLookupRec->start == -1){ // In this case, the entire xTile is occupied by this rect, so just move on.
						continue;
					}
				}
				int slot = int (runningHorizontalRect->getX()) / tileWidth;
				if (rects [slot]){
					if ((rects [slot]->getWidth() == runningHorizontalRect->getWidth())
							&& (rects [slot]->getHeight() + rects [slot]->getY() == runningHorizontalRect->getY())){
						rects [slot]->setHeight (rects [slot]->getHeight() + tileHeight);
						#ifdef MICROTILEFIELD_MEMCHECK
							iDeallocated (runningHorizontalRect);
						#endif
						delete runningHorizontalRect; runningHorizontalRect = NULL;
					}
					else{
						placeInCompilation (rects [slot]);
						rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
					}
				}
				else{ 
					rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
				}
			}
			// We ONLY get this far if we are NOT tracking a horizontal run and/or if a horizontal run just ENDED. 
			while (nextLookupRec->start != -1){
				//cout << "nextLookupRec->start " << nextLookupRec->start << endl;
				if (runningHorizontalRect){
					int slot = int (runningHorizontalRect->getX()) / tileWidth;
					if (rects [slot]){
						if ((rects [slot]->getWidth() == runningHorizontalRect->getWidth())
								&& (rects [slot]->getHeight() + rects [slot]->getY() == runningHorizontalRect->getY())){
							rects [slot]->setHeight (rects [slot]->getHeight() + tileHeight);
							#ifdef MICROTILEFIELD_MEMCHECK
								iDeallocated (runningHorizontalRect);
							#endif
							delete runningHorizontalRect; runningHorizontalRect = NULL;
						}
						else{
							placeInCompilation (rects [slot]);
							rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
						}
					}
					else{ 
						rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
					}
				}
				runningHorizontalRect = new Rect (
					(nextLookupRec->start * tileWidth) + (xTile * BITS_PER_TILE_ENTRY * tileWidth), 
					y * tileHeight, nextLookupRec->width * tileWidth, tileHeight
				);
				#ifdef MICROTILEFIELD_MEMCHECK
					iAllocated (runningHorizontalRect);
					//cout << "Allocated " << runningHorizontalRect << " at " << __LINE__ << endl;
				#endif
				nextLookupRec++;
			}
		}
		// We're done with the row.
		if (runningHorizontalRect){
			int slot = int (runningHorizontalRect->getX()) / tileWidth;
			if (rects [slot]){
				if ((rects [slot]->getWidth() == runningHorizontalRect->getWidth())
						&& (rects [slot]->getHeight() + rects [slot]->getY() == runningHorizontalRect->getY())){
					rects [slot]->setHeight (rects [slot]->getHeight() + tileHeight);
					#ifdef MICROTILEFIELD_MEMCHECK
						iDeallocated (runningHorizontalRect);
					#endif
					delete runningHorizontalRect; runningHorizontalRect = NULL;
				}
				else{
					placeInCompilation (rects [slot]);
					rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
				}
			}
			else{ 
				rects [slot] = runningHorizontalRect; runningHorizontalRect = NULL;
			}
		}
	}
	for (unsigned int y = 0; y < tilesPerRow * BITS_PER_TILE_ENTRY; y++){
		if (rects [y]) placeInCompilation (rects [y]);
	}
	if (runningHorizontalRect){
		#ifdef MICROTILEFIELD_MEMCHECK
			iDeallocated (runningHorizontalRect);
		#endif
		delete runningHorizontalRect;
		runningHorizontalRect = NULL;
	}

	free (rects); rects = NULL;

	hasCompiled = true;
	//cout << "Done compiling" << endl;
}

void MicroTileField::getConsolidatedRects (const Rect& intersecting, vector<Rect*>& rects){
	if (!hasCompiled){
		resetCompilation();
		compile();
	}

	compilation->getItems (&intersecting, rects);

	//cout << rects.size() << " rects are dirty. Returning." << endl;
	//for (unsigned int x = 0; x < rects.size(); x++) cout << rects [x]->toString() << " " << endl;
}

void MicroTileField::reset(){
	resetMarkField();
	resetCompilation();
}
void MicroTileField::resetMarkField(){
	unsigned int tileEntries = (widthInTiles / BITS_PER_TILE_ENTRY) + (widthInTiles % BITS_PER_TILE_ENTRY > 0);
	for (unsigned int y = 0; y < heightInTiles; y++){
		for (unsigned int x = 0; x < tileEntries; x++){
			markField [y][x] = 0x0;
		}
	}
}
void MicroTileField::resetCompilation(){
	vector<Rect*> gets;
	Rect perimeter (0, 0, width, height);
	compilation->getItems (&perimeter, gets);
	for (vector<Rect*>::iterator iter = gets.begin(); iter != gets.end(); iter++){
		#ifdef MICROTILEFIELD_MEMCHECK
			iDeallocated (*iter);
		#endif
		delete *iter;
	}
	compilation->clear();
	hasCompiled = false;
}











#ifdef TEST_MICROTILEFIELD
string MicroTileField::test(){
	string retVal = testLookups() + "\n" + testDirty() + "\n" + testCompile() + "\n"; 
	retVal += testGet() + "\n";
	return retVal;
}
string MicroTileField::testLookups(){
	string retString = "";
	
	// tileEntryToString() tests can only easily test one byte of the TILE_ENTRY_TYPE
	retString += check ("tileEntryToString() test 1", "11111111", tileEntryToString (0xff));
	retString += check ("tileEntryToString() test 2", "00000000", tileEntryToString (0x00));
	retString += check ("tileEntryToString() test 3", "00000001", tileEntryToString (0x01));;
	retString += check ("tileEntryToString() test 4", "10000000", tileEntryToString (0x01 << 7));
	retString += check ("tileEntryToString() test 5", "00100100", tileEntryToString ((0x01 << 5) | (0x01 << 2))); 

	// dirty mask lookup tests only currently work for 8-bit TILE_ENTRY_TYPE. other types have not been implemented.  :P
	retString += check ("dirty mask lookup test 1", "11111111", tileEntryToString (dirtyMaskLookup [0][0]));
	retString += check ("dirty mask lookup test 2", "01111111", tileEntryToString (dirtyMaskLookup [0][1]));
	retString += check ("dirty mask lookup test 2.5", "00111111", tileEntryToString (dirtyMaskLookup [0][2]));
	retString += check ("dirty mask lookup test 3", "00001111", tileEntryToString (dirtyMaskLookup [0][BITS_PER_TILE_ENTRY - 4]));
	retString += check ("dirty mask lookup test 4", "00000001", tileEntryToString (dirtyMaskLookup [0][BITS_PER_TILE_ENTRY - 1]));
	retString += check ("dirty mask lookup test 5", "00000000", tileEntryToString (dirtyMaskLookup [0][BITS_PER_TILE_ENTRY]));
	retString += check ("dirty mask lookup test 6", "00000000", tileEntryToString (dirtyMaskLookup [1][0]));
	retString += check ("dirty mask lookup test 7", "10000000", tileEntryToString (dirtyMaskLookup [1][1]));
	retString += check ("dirty mask lookup test 8", "11110000", tileEntryToString (dirtyMaskLookup [1][BITS_PER_TILE_ENTRY - 4]));
	retString += check ("dirty mask lookup test 9", "11111110", tileEntryToString (dirtyMaskLookup [1][BITS_PER_TILE_ENTRY - 1]));
	retString += check ("dirty mask lookup test 10", "11111111", tileEntryToString (dirtyMaskLookup [1][BITS_PER_TILE_ENTRY]));

	// lookup tests only currently work for 8-bit TILE_ENTRY_TYPE. other types have not been implemented.
	retString += check ("tile state lookup test 1", -1, lookup [0][0].start);
	retString += check ("tile state lookup test 2", -1, lookup [0][1].start);
	retString += check ("tile state lookup test 3", -1, lookup [0][2].start);
	retString += check ("tile state lookup test 4", -1, lookup [0][3].start);
	retString += check ("tile state lookup test 5", -1, lookup [0][4].start);
	retString += check ("tile state lookup test 10", 7, lookup [1][0].start);
	retString += check ("tile state lookup test 11", 1, lookup [1][0].width);
	retString += check ("tile state lookup test 12", -1, lookup [1][1].start);
	retString += check ("tile state lookup test 13", 0, lookup [TILE_ENTRY_STATES - 1][0].start);
	retString += check ("tile state lookup test 14", BITS_PER_TILE_ENTRY, lookup [TILE_ENTRY_STATES - 1][0].width); 
	retString += check ("tile state lookup test 15", -1, lookup [TILE_ENTRY_STATES - 1][1].start);
	retString += check ("tile state lookup test 16", 1, lookup [0x55][0].start); // 01010101
	retString += check ("tile state lookup test 17", 1, lookup [0x55][0].width); // 01010101
	retString += check ("tile state lookup test 18", 3, lookup [0x55][1].start); // 01010101
	retString += check ("tile state lookup test 19", 1, lookup [0x55][1].width); // 01010101
	retString += check ("tile state lookup test 20", 5, lookup [0x55][2].start); // 01010101
	retString += check ("tile state lookup test 21", 1, lookup [0x55][2].width); // 01010101
	retString += check ("tile state lookup test 22", 7, lookup [0x55][3].start); // 01010101
	retString += check ("tile state lookup test 23", 1, lookup [0x55][3].width); // 01010101
	retString += check ("tile state lookup test 24", -1, lookup [0x55][4].start); // 01010101

	return retString;
}
string MicroTileField::testDirty(){
	string retString = "";
	vector<Rect> dirties;

	unsigned int tileEntries = (widthInTiles / BITS_PER_TILE_ENTRY) + (widthInTiles % BITS_PER_TILE_ENTRY > 0);
	TILE_ENTRY_TYPE** testField = (TILE_ENTRY_TYPE**) MALLOC (sizeof (TILE_ENTRY_TYPE*) * heightInTiles);
	TILE_ENTRY_TYPE** testFieldSet = (TILE_ENTRY_TYPE**) MALLOC (sizeof (TILE_ENTRY_TYPE*) * heightInTiles);
	for (unsigned int y = 0; y < heightInTiles; y++){
		testField [y] = (TILE_ENTRY_TYPE*) MALLOC (sizeof (TILE_ENTRY_TYPE) * tileEntries);
		testFieldSet [y] = (TILE_ENTRY_TYPE*) MALLOC (sizeof (TILE_ENTRY_TYPE) * tileEntries);
		for (unsigned int x = 0; x < tileEntries; x++){
			testField [y][x] = 0x00;
			testFieldSet [y][x] = ~((TILE_ENTRY_TYPE) 0x00);
			if (x == tileEntries - 1) testFieldSet [y][x] &= (~((TILE_ENTRY_TYPE) 0x00) << (widthInTiles % BITS_PER_TILE_ENTRY));
		}
	}

	unsigned int tileWidth = width / widthInTiles;
	unsigned int tileHeight = height / heightInTiles;

	dirty (dirties);
	retString += check ("dirty test 1", markFieldToString (testField), markFieldToString (markField));
	resetMarkField();

	dirties.push_back (Rect (-1, 0, 1, 1));
	dirty (dirties);
	retString += check ("dirty test 2", markFieldToString (testField), markFieldToString (markField));
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (0, -1, 1, 1)); 
	dirty (dirties);
	retString += check ("dirty test 3", markFieldToString (testField), markFieldToString (markField));
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (0, height, 1, 1)); 
	dirty (dirties);
	retString += check ("dirty test 4", markFieldToString (testField), markFieldToString (markField));
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (width, 0, 1, 1)); 
	dirty (dirties);
	retString += check ("dirty test 5", markFieldToString (testField), markFieldToString (markField));
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (0, 0, 1, 1));
	dirty (dirties);
	testField [0][0] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 6", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (width - 1, 0, 1, 1));
	dirty (dirties);
	testField [0][(widthInTiles - 1) / BITS_PER_TILE_ENTRY] = 0x01 << widthInTiles % BITS_PER_TILE_ENTRY;
	retString += check ("dirty test 7", markFieldToString (testField), markFieldToString (markField));
	testField [0][(widthInTiles - 1) / BITS_PER_TILE_ENTRY] = 0x00 << widthInTiles % BITS_PER_TILE_ENTRY;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (0, height - 1, 1, 1));
	dirty (dirties);
	testField [heightInTiles - 1][0] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 8", markFieldToString (testField), markFieldToString (markField));
	testField [heightInTiles - 1][0] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (width - 1, height - 1, 1, 1));
	dirty (dirties);
	testField [heightInTiles - 1][(widthInTiles - 1) / BITS_PER_TILE_ENTRY] = 0x01 << (widthInTiles % BITS_PER_TILE_ENTRY);
	retString += check ("dirty test 9", markFieldToString (testField), markFieldToString (markField));
	testField [heightInTiles - 1][(widthInTiles - 1) / BITS_PER_TILE_ENTRY] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (0, 0, width, height));
	dirty (dirties);
	retString += check ("dirty test 10", markFieldToString (testFieldSet), markFieldToString (markField));
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, 1, 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	retString += check ("dirty test 11", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (8 * tileWidth, 0, 1, 1));
	dirty (dirties);
	testField [0][1] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 12", markFieldToString (testField), markFieldToString (markField));
	testField [0][1] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth + 1, 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	testField [0][1] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 13", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	testField [0][1] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth, 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	retString += check ("dirty test 14", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth, 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	retString += check ("dirty test 15", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth * 9 + 1, 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	testField [0][1] = 0xff;
	testField [0][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 16", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	testField [0][1] = 0x00;
	testField [0][2] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth * 9 + 1, tileHeight + 1));
	dirty (dirties);
	testField [0][0] = 0x01;
	testField [0][1] = 0xff;
	testField [0][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	testField [1][0] = 0x01;
	testField [1][1] = 0xff;
	testField [1][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 17", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	testField [0][1] = 0x00;
	testField [0][2] = 0x00;
	testField [1][0] = 0x00;
	testField [1][1] = 0x00;
	testField [1][2] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 1, tileWidth * 9 + 1, tileHeight));
	dirty (dirties);
	testField [0][0] = 0x01;
	testField [0][1] = 0xff;
	testField [0][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	testField [1][0] = 0x01;
	testField [1][1] = 0xff;
	testField [1][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 18", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	testField [0][1] = 0x00;
	testField [0][2] = 0x00;
	testField [1][0] = 0x00;
	testField [1][1] = 0x00;
	testField [1][2] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, 0, tileWidth * 9 + 1, tileHeight));
	dirty (dirties);
	testField [0][0] = 0x01;
	testField [0][1] = 0xff;
	testField [0][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 19", markFieldToString (testField), markFieldToString (markField));
	testField [0][0] = 0x00;
	testField [0][1] = 0x00;
	testField [0][2] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (7 * tileWidth, height - (2 * tileHeight) + 1, tileWidth * 9 + 1, tileHeight));
	dirty (dirties);
	testField [heightInTiles - 2][0] = 0x01;
	testField [heightInTiles - 2][1] = 0xff;
	testField [heightInTiles - 2][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	testField [heightInTiles - 1][0] = 0x01;
	testField [heightInTiles - 1][1] = 0xff;
	testField [heightInTiles - 1][2] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 20", markFieldToString (testField), markFieldToString (markField));
	testField [heightInTiles - 2][0] = 0x00;
	testField [heightInTiles - 2][1] = 0x00;
	testField [heightInTiles - 2][2] = 0x00;
	testField [heightInTiles - 1][0] = 0x00;
	testField [heightInTiles - 1][1] = 0x00;
	testField [heightInTiles - 1][2] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (tileWidth * 8, 50, 2, 30));
	dirties.push_back (Rect (tileWidth * 8 - 1, 50, 2, 30));
	dirty (dirties);
	testField [1][0] = 0x01;
	testField [1][1] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	testField [2][0] = 0x01;
	testField [2][1] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 21", markFieldToString (testField), markFieldToString (markField));
	testField [1][0] = 0x00;
	testField [1][1] = 0x00;
	testField [2][0] = 0x00;
	testField [2][1] = 0x00;
	dirties.clear();
	resetMarkField();

	dirties.push_back (Rect (tileWidth * 6, tileHeight, 30, 30));
	dirties.push_back (Rect (tileWidth * 8, tileHeight, 30, 30));
	dirty (dirties);
	testField [1][0] = 0x01 << 1;
	testField [1][1] = 0x01 << (BITS_PER_TILE_ENTRY - 1);
	retString += check ("dirty test 22", markFieldToString (testField), markFieldToString (markField));
	testField [1][0] = 0x00;
	testField [1][1] = 0x00;
	dirties.clear();
	resetMarkField();

	for (unsigned int y = 0; y < heightInTiles; y++){
		FREE (testField [y]);
		FREE (testFieldSet [y]);
	}
	FREE (testField);
	FREE (testFieldSet);

	return retString;
}
string MicroTileField::testCompile(){
	vector<Rect*> gets;
	vector<Rect*> expects;
	vector<Rect> dirties;

	unsigned int tileWidth = width / widthInTiles;
	unsigned int tileHeight = height / heightInTiles;

	Rect scraps [10];

	string retString = "";

	dirties.push_back (Rect (0, 0, 1, 1));
	scraps [0] = Rect (0, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	gets.clear();
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 1", expects, gets);
	dirties.clear();
	expects.clear();
	reset();

	dirties.push_back (Rect (width - 1, 0, 1, 1));
	scraps [0] = Rect (width - tileWidth, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	gets.clear();
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 2", expects, gets);
	dirties.clear();
	expects.clear();
	reset();

	dirties.push_back (Rect (0, height - 1, 1, 1));
	scraps [0] = Rect (0, height - tileHeight, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	gets.clear();
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 3", expects, gets);
	dirties.clear();
	expects.clear();
	reset();

	dirties.push_back (Rect (width - 1, height - 1, 1, 1));
	scraps [0] = Rect (width - tileWidth, height - tileHeight, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	gets.clear();
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 4", expects, gets);
	dirties.clear();
	expects.clear();
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (-1, 0, 1, 1));
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 5", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (width, 0, 1, 1));
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 6", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, -1, 1, 1));
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 7", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, height, 1, 1));
	dirty (dirties);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 8", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, width, height));
	dirty (dirties);
	scraps [0] = Rect (0, 0, 640, 480);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 9", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (0, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 10", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (width - 1, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (width - tileWidth, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 11", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, height - 1, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (0, height - tileHeight, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 12", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (width - 1, height - 1, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (width - tileWidth, height - tileHeight, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 13", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (width - 1, 0, 1, height));
	dirty (dirties);
	scraps [0] = Rect (width - tileWidth, 0, tileWidth, height);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 14", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, height - 1, width, 1));
	dirty (dirties);
	scraps [0] = Rect (0, height - tileHeight, width, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 15", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, 1, height));
	dirty (dirties);
	scraps [0] = Rect (0, 0, tileWidth, height);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 16", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, width, 1));
	dirty (dirties);
	scraps [0] = Rect (0, 0, width, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 17", expects, gets);
	reset();

	expects.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 2, tileHeight * 2, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 2, tileHeight * 2, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	gets.clear();
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 18", expects, gets);
	gets.clear();
	getConsolidatedRects (Rect (tileWidth * 2, tileHeight * 2, 1, 1), gets);
	retString += check ("compile test 19", expects, gets);
	gets.clear();
	getConsolidatedRects (Rect (tileWidth * 2 - 1, tileHeight * 2, 1, 1), gets);
	expects.clear();
	retString += check ("compile test 20", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, 1, 1));
	dirties.push_back (Rect (tileWidth, 0, 1, 1));
	dirties.push_back (Rect (tileWidth * 2, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (0, 0, tileWidth * 3, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 21", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, 1, 1));
	dirties.push_back (Rect (tileWidth, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (0, 0, tileWidth * 2, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 22", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (0, 0, 1, 1));
	dirties.push_back (Rect (tileWidth * 2, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (0, 0, tileWidth, tileHeight);
	scraps [1] = Rect (tileWidth * 2, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	expects.push_back (&scraps [1]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 23", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 7, 0, tileWidth + 1, 1));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 7, 0, tileWidth * 2, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 24", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 7, 0, tileWidth * 2 + 1, 1));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 7, 0, tileWidth * 3, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 25", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 7, 0, tileWidth * 3 + 1, 1));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 7, 0, tileWidth * 4, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 26", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 8, 0, 1, 1));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 8, 0, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 27", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 8, 50, 2, 30));
	dirties.push_back (Rect (tileWidth * 8 - 1, 50, 2, 30));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 7, tileHeight, tileWidth * 2, tileHeight * 2);
	expects.push_back (&scraps [0]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 28", expects, gets);
	reset();

	expects.clear();
	gets.clear();
	dirties.clear();
	dirties.push_back (Rect (tileWidth * 6, tileHeight, 30, 30));
	dirties.push_back (Rect (tileWidth * 8, tileHeight, 30, 30));
	dirty (dirties);
	scraps [0] = Rect (tileWidth * 6, tileHeight, tileWidth, tileHeight);
	scraps [1] = Rect (tileWidth * 8, tileHeight, tileWidth, tileHeight);
	expects.push_back (&scraps [0]);
	expects.push_back (&scraps [1]);
	getConsolidatedRects (Rect (0, 0, width, height), gets);
	retString += check ("compile test 29", expects, gets);
	reset();


	return retString;
}
string MicroTileField::testGet(){
	return "";
}
string MicroTileField::check (string testName, string expected, string got){
	//cout << "checking " << testName << endl;
	if (expected == got) return "Passed " + testName + "\n";
	return "Failed " + testName + ". Expected " + expected + ", got " + got + "\n";
}
string MicroTileField::check (string testName, int expected, int got){
	char gotString [1000];
	sprintf (gotString, "%d", got);
	char expectedString [1000];
	sprintf (expectedString, "%d", expected);
	return check (testName, string (expectedString), string (gotString));
}
string MicroTileField::check (const char* testName, vector<Rect*>& expected, vector<Rect*>& got){
	return check (string (testName), expected, got);
}
string MicroTileField::check (string testName, vector<Rect*>& expected, vector<Rect*>& got){
	list<Rect> expectedList;
	for (vector<Rect*>::iterator iter = expected.begin(); iter != expected.end(); iter++){
		expectedList.push_back (**iter);
	}
	expectedList.sort();
	list<Rect> gotList;
	for (vector<Rect*>::iterator iter = got.begin(); iter != got.end(); iter++){
		gotList.push_back (**iter);
	}
	gotList.sort();
	string expectedString = "[";
	for (list<Rect>::iterator iter = expectedList.begin(); iter != expectedList.end(); iter++){
		if (iter != expectedList.begin()) expectedString += ", ";
		expectedString += iter->toString();
	}
	expectedString += "]";
	string gotString = "[";
	for (list<Rect>::iterator iter = gotList.begin(); iter != gotList.end(); iter++){
		if (iter != gotList.begin()) gotString += ", ";
		gotString += iter->toString();
	}
	gotString += "]";
	return check (testName, expectedString, gotString);
}
string MicroTileField::tileEntryToString (TILE_ENTRY_TYPE toString){
	string retString = "";
	for (int x = BITS_PER_TILE_ENTRY - 1; x >= 0; x--) 
		retString += ((0x01 << x) & toString) ? "1" : "0";
	return retString;
}
string MicroTileField::markFieldToString (TILE_ENTRY_TYPE** marked){
	string retString = "";
	unsigned int tileEntries = (widthInTiles / BITS_PER_TILE_ENTRY) + (widthInTiles % BITS_PER_TILE_ENTRY > 0);
	for (unsigned int y = 0; y < heightInTiles; y++){
		for (unsigned int x = 0; x < tileEntries; x++){
			retString += tileEntryToString (marked [y][x]);
		}
		retString += "\n";
	}
	return retString;
}




#ifdef MICROTILEFIELD_MEMCHECK
void* MicroTileField::myMalloc (size_t s){
	void* allocated = malloc (s);
	iAllocated (allocated);
	//cout << "Allocated " << allocated << " at " << __LINE__ << endl;
	return allocated;
}
void MicroTileField::myFree (void* i){
	if (iDeallocated (i)) free (i);
}	
void MicroTileField::checkMem(){
	if (allocatedMemory.size() > 0 || doubleFrees.size() > 0){ 
		cout << "Failed memcheck.";
		if (allocatedMemory.size() > 0){
			cout << " Leaked: ";
			for (vector<void*>::iterator iter = allocatedMemory.begin(); iter != allocatedMemory.end(); iter++){
				if (iter != allocatedMemory.begin()) cout << ", ";
				cout << *iter;
			}
		}
		if (doubleFrees.size() > 0){
			cout << " Freed unallocated: ";
			for (vector<void*>::iterator iter = doubleFrees.begin(); iter != doubleFrees.end(); iter++){
				if (iter != doubleFrees.begin()) cout << ", ";
				cout << *iter;
			}
		}
		cout << endl;
	}
	else{
		cout << "Passed memcheck" << endl;
	}
}
void MicroTileField::iAllocated (void* i){
	allocatedMemory.push_back (i);
}
bool MicroTileField::iDeallocated (void* i){
	if (find (allocatedMemory.begin(), allocatedMemory.end(), i) == allocatedMemory.end()){
		doubleFrees.push_back (i);
		return false;
	}
	allocatedMemory.erase (find (allocatedMemory.begin(), allocatedMemory.end(), i));
	return true;
}
#endif





#endif
