#ifndef QUAD_PATH_FINDER_H
#define QUAD_PATH_FINDER_H


#include <list>
#include <list>
#include <math.h>
#include <queue>
#include <stdexcept>
#include <sstream>
#include <algorithm>
#include "interface_segmentContainerListener.h"
#include "binList.h"
#include "rect.h"
#include "segmentContainer.h"
#include "utils.h"
#include "utilTypes.h"

#define QUAD_PATH_FINDER_MAX_CACHED_PATHS 3


class IntRect{
	public:
		IntRect (const int x, const int y, const int width = 1, const int height = 1) : x_ (x), y_ (y), width_ (width), height_ (height) { }
		IntRect (const Rect& source){ init_ (source.getX(), source.getY(), source.getWidth(), source.getHeight()); }
		IntRect() : x_ (0), y_ (0), width_ (0), height_ (0) { }
		bool operator!= (const IntRect& other) const{
			return x_ != other.x_ || y_ != other.y_ || width_ != other.width_ || height_ != other.height_;
		}
		bool operator== (const IntRect& other) const{
			return !(other != *this);
		}
		~IntRect(){}

		int getX() const{ return x_; }
		int getY() const{ return y_; }
		int getWidth() const{ return width_; }
		int getHeight() const{ return height_; }

	private:
		void init_ (const double x, const double y, const double width, const double height){ 
			x_ = int (x); 
			y_ = int (y); 
			width_ = int (ceil (x + width)) - int (x); 
			height_ = int (ceil (y + height)) - int (y);
		}

		int x_;
		int y_;
		int width_;
		int height_;
};

class PathUnit{
	public:
		PathUnit (const int x, const int y, const int distanceTravelled, const int distanceLeft, PathUnit* parent = NULL) 
				: x_ (x), y_ (y), distanceTravelled_ (distanceTravelled), distanceLeft_ (distanceLeft), parent_ (parent) { }
		~PathUnit(){}

		int x() const{ return x_; }
		int y() const{ return y_; }
		int distanceTravelled() const{ return distanceTravelled_; }
		int distanceLeft() const{ return distanceLeft_; }
		PathUnit* parent() const{ return parent_; }

	private:
		int x_;
		int y_;
		int distanceTravelled_;
		int distanceLeft_;
		PathUnit* parent_;
};
class ComparePathUnitPtr{
	public:
		bool operator() (PathUnit*& first, PathUnit*& second){
			int sum1 = first->distanceTravelled() + first->distanceLeft();
			int sum2 = second->distanceTravelled() + second->distanceLeft();
			if (sum1 > sum2) return true;
			if (sum1 == sum2 && first->distanceLeft() > second->distanceLeft()) return true;
			return false;
		}
};

class IndexByHashEvaluator : public Interface_IndexEvaluator<PathUnit*>{
	public:
		IndexByHashEvaluator(){}
		virtual ~IndexByHashEvaluator(){}
		virtual bool lessThan (PathUnit* const& first, PathUnit* const& second) const{
			if (first->y() < second->y()) return true;
			if (first->y() > second->y()) return false;
			return first->x() < second->x();
			
			/*
			unsigned int firstHash = (unsigned int) (first->y() ^ first->x());
			unsigned int secondHash = (unsigned int) (second->y() ^ second->x());
			if (firstHash < secondHash) return true;
			return false;
			*/

			/*
			unsigned int firstHash = ((unsigned int) (first->x()) && 0x0f0f0f0f) | (unsigned int) (first->y()) && 0xf0f0f0f0;
			unsigned int secondHash = ((unsigned int) (second->x()) && 0x0f0f0f0f) | (unsigned int) (second->y()) && 0xf0f0f0f0;
			return firstHash < secondHash;
			*/
		}
		virtual bool equals (PathUnit* const& first, PathUnit* const& second) const{
			return first->y() == second->y() && first->x() == second->x();
		}
	private:
};

class CachedPath{
	public:
		CachedPath (const std::list<IntRect>& path, int characterWidth = 1, int characterHeight = 1) : path_ (path), 
				characterWidth_ (characterWidth), characterHeight_ (characterHeight){
			if (path.size() > 0){ 
				std::list<IntRect>::const_iterator iter = path_.end();
				iter--;
				endRect_ = *iter;
			}
		}
		~CachedPath(){}
		bool hasPoint (const Point& point) const{
			std::list<IntRect>::const_iterator loc = std::find (path_.begin(), path_.end(), IntRect (int (point.getX()), int (point.getY()), 1, 1));
			return loc != path_.end();
		}
		bool satisfies (const Point& startPoint, const Point& endPoint, int characterWidth = 1, int characterHeight = 1) const{
			return IntRect (int (endPoint.getX()), int (endPoint.getY()), 1, 1) == endRect_ && hasPoint (startPoint);
		}
		void getPath (std::list<IntRect>& runningList, const Point& startPoint, const Point& endPoint) const{
			IntRect endRect (int (endPoint.getX()), int (endPoint.getY()), 1, 1);
			if (endRect != endRect_) return;
			std::list<IntRect>::const_iterator loc = std::find (path_.begin(), path_.end(), IntRect (int (startPoint.getX()), int (startPoint.getY()), 1, 1));
			if (loc == path_.end()) return;
			for ( ; loc != path_.end(); loc++){
				runningList.push_back (*loc);
			}
		}
	private:
		std::list<IntRect> path_;
		IntRect endRect_;
		int characterWidth_;
		int characterHeight_;
};

class PathUnitList{
	public:
		PathUnitList(){
			hashEvaluator_ = new IndexByHashEvaluator();
			listByHash_ = new BinList<PathUnit*> (hashEvaluator_);
		}
		~PathUnitList(){
			delete listByHash_;
			delete hashEvaluator_;
		}
		void append (PathUnit* pathUnit){
			listByHash_->append (pathUnit);
			listByDistance_.push (pathUnit);
		}
		PathUnit* pop(){
			PathUnit* popped = listByDistance_.top();
			listByDistance_.pop();
			listByHash_->remove (popped);
			return popped;
		}
		bool hasItem (PathUnit* pathUnit) const{
			return listByHash_->hasItem (pathUnit);
		}
		unsigned int size() const{
			return listByHash_->size();
		}
		void getAll (std::vector<PathUnit*>& runningList){
			listByHash_->getAll (runningList);
		}
	private:
		std::priority_queue<PathUnit*, std::vector<PathUnit*>, ComparePathUnitPtr> listByDistance_;
		BinList<PathUnit*>* listByHash_;
		IndexByHashEvaluator* hashEvaluator_;
};

class PathFinderException : public std::runtime_error{
	public:
		PathFinderException (const std::string& what) : std::runtime_error (what) { }
		virtual ~PathFinderException() throw() { }
};
class PathFinderPathNotFoundException : public PathFinderException{
	public:
		PathFinderPathNotFoundException (const std::string what = "No path found.") : PathFinderException (what) { }
		virtual ~PathFinderPathNotFoundException() throw() { }
};

class QuadPathFinder : public Interface_SegmentContainerListener{
	friend class QuadPathFinderTester;

	public:
		QuadPathFinder (SegmentContainer& walls)
			: walls_ (walls){ 
			walls_.addListener (this);
		}
		~QuadPathFinder(){ 
			cleanupAllPathUnits_();
		}

		virtual void segmentAdded (const SegmentContainer* which, const Line& newSegment){
			if (which == &walls_) cachedPaths_.clear();
		}
		virtual void segmentsCleared (const SegmentContainer* which){
			if (which == &walls_) cachedPaths_.clear();
		}
		void findPath (std::list<IntRect>& runningList, const Point& startPointOriginal, const Point& endPoint, 
					   unsigned int characterWidth = 1, unsigned int characterHeight = 1, double closeEnough = -1.0)
		{
			runningList.clear();
			Point startPoint (int (startPointOriginal.getX()), int (startPointOriginal.getY()));
			for (std::list<CachedPath>::const_iterator iter = cachedPaths_.begin(); iter != cachedPaths_.end(); iter++){
				if (iter->satisfies (startPoint, endPoint)){
					iter->getPath (runningList, startPoint, endPoint);
					//cout << "returning cached path" << endl;
					return;
				}
			}
			//cout << "finding path from " << startPoint.toString() << " , " << endPoint.toString() << endl;
			closeEnough = max (closeEnough, double (0));
			if (characterWidth < 1) characterWidth = 1;
			if (characterHeight < 1) characterHeight = 1;
			if (!pointPassable_ (startPoint, characterWidth, characterHeight)) throw PathFinderException ("Can not start path on a wall!");
			PathUnitList possibles;
			PathUnitList closed;
			int startX = int (startPoint.getX());
			int startY = int (startPoint.getY());
			int endX = int (endPoint.getX());
			int endY = int (endPoint.getY());
//#define SHOW_ADJACENTS
//#define SHOW_POSSIBLES
			PathUnit* startCell = createPathUnit_ (startX, startY, 0, estimateDistance_ (startX, startY, endX, endY));
			possibles.append (startCell);
			PathUnit* possible = possibles.pop();
			while (possible->distanceLeft() > closeEnough){
				if (possible->distanceLeft() > closeEnough){
					#ifdef SHOW_POSSIBLES
					cout << "[POSSIBLE] ";
					cout << "possible x,y distanceTravelled,distanceLeft, sum: " << possible->x() << "," << possible->y() << " " 
						 << possible->distanceTravelled() << "," << possible->distanceLeft() << " " << possible->distanceTravelled() + possible->distanceLeft()
						 << endl;
					#endif
					std::list<PathUnit*> consider;
					getAdjacentPathUnits_ (consider, possible, endX, endY, characterWidth, characterHeight);
					#ifdef SHOW_ADJACENTS
					std::cout << "[ADJACENTS_START] adjacents for " << possible->x() << "," << possible->y() << " are: " << endl;
					#endif
					for (std::list<PathUnit*>::const_iterator con = consider.begin(); con != consider.end(); con++){
						#ifdef SHOW_ADJACENTS
						std::cout << "[ADJACENT] " << (*con)->x() << "," << (*con)->y() << " ";
						#endif
						//cout << "considering cell at " << (*con)->x() << "," << (*con)->y();
						PathUnit* conPath = *con;
						bool passable = pointPassable_ (Point (conPath->x(), conPath->y()), characterWidth, characterHeight);
						if (!passable || closed.hasItem (conPath) || possibles.hasItem (conPath)){
							#ifdef SHOW_ADJACENTS
							if (!passable) cout << " but it's not passable" << endl;
							else cout << " but i've already tried it" << endl;
							#endif
							continue;
						}
						#ifdef SHOW_ADJACENTS
						cout << ". not considered yet, adding to possibles" << endl;
						#endif
						possibles.append (conPath);
					}
					#ifdef SHOW_ADJACENTS
					std::cout << "[ADJACENTS_DONE]" << std::endl;
					cout << "[POSSIBLE_DONE]" << endl;
					#endif
				}
				closed.append (possible);
				if (possibles.size() > 0) possible = possibles.pop();
				else throw PathFinderPathNotFoundException();
			}
			PathUnit* next = possible;
			while (next){
				//cout << "building path. next is at " << next->x() << "," << next->y() << endl;
				PathUnit* nextParent = next->parent();
				if (nextParent && abs (nextParent->x() - next->x()) + abs (nextParent->y() - next->y()) > 1){
					list<Point> rasterizedLine;
					Utils::rasterizeLine (rasterizedLine, Line (Point (next->x(), next->y()), Point (nextParent->x(), nextParent->y())));
					list<Point>::const_iterator lastItem = rasterizedLine.end(); lastItem--;
					for (list<Point>::const_iterator iter = rasterizedLine.begin(); iter != lastItem; iter++){
						runningList.insert (runningList.begin(), IntRect (int (iter->getX()), int (iter->getY()), 1, 1));
					}
					//cout << "rasterized from " << next->x() << "," << next->y() << " to " << nextParent->x() << "," << nextParent->y() << endl;
				}
				else{
					runningList.insert (runningList.begin(), IntRect (next->x(), next->y(), 1, 1));
				}
				next = nextParent;
			}
			if (cachedPaths_.size() > QUAD_PATH_FINDER_MAX_CACHED_PATHS) cachedPaths_.erase (cachedPaths_.begin());
			cachedPaths_.push_back (CachedPath (runningList));
			cleanupAllPathUnits_();
		}
		enum EdgeFacing{ facing_up, facing_right, facing_down, facing_left };
	
	private:
		int estimateDistance_ (const int x1, const int y1, const int x2, const int y2) const{
			return int ( sqrt ( double( ((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1)) ) ) );
		}
		bool pointPassable_ (const Point& checkPoint, unsigned int characterWidth = 1, unsigned int characterHeight = 1) const{
			int checkPointX = int (checkPoint.getX()) - (characterWidth / 2);
			int checkPointY = int (checkPoint.getY()) - (characterHeight / 2);
			list <Line> nearbyWalls;
			getWallsForBounds_ (nearbyWalls, IntRect (checkPointX, checkPointY, characterWidth, characterHeight));
			return nearbyWalls.size() == 0;
		}
		void cleanupAllPathUnits_(){
			for (std::list<PathUnit*>::iterator iter = allPathUnits_.begin(); iter != allPathUnits_.end(); iter++){
				delete *iter;
			}
			allPathUnits_.clear();
		}
		PathUnit* createPathUnit_ (const int x, const int y, const int distanceTravelled, const int distanceLeft){
			return createPathUnit_ (x, y, distanceTravelled, distanceLeft, NULL);
		}
		PathUnit* createPathUnit_ (const int x, const int y, const int distanceTravelled, const int distanceLeft, PathUnit* parent){
			PathUnit* newPathUnit = new PathUnit (x, y, distanceTravelled, distanceLeft, parent);
			allPathUnits_.push_back (newPathUnit);
			return newPathUnit;
		}
		void getAdjacentPathUnits_ (std::list<PathUnit*>& runningList, PathUnit* source, const int targetX, const int targetY, int characterWidth, int characterHeight){
			getAdjacentPathUnits_ (runningList, source, targetX, targetY, IntRect (walls_.getBounds()), characterWidth, characterHeight);
		}
		void getAdjacentPathUnits_ (std::list<PathUnit*>& runningList, PathUnit* source, const int targetX, const int targetY, const IntRect& startBounds, int characterWidth, int characterHeight){
			if (source->x() == targetX && source->y() == targetY) return; // If this is the end point, skip it.
			int width = startBounds.getWidth();
			int height = startBounds.getHeight();
			int startX = startBounds.getX();
			int startY = startBounds.getY();
			width = height = max (width, height); // Force square bounds.
			IntRect correctedBounds (startBounds.getX(), startBounds.getY(), width, height);
			//std::cout << "getting adjacents for source: " << source << ", startBounds x,y,w,h: " << startBounds.getX() << "," << startBounds.getY() << "," << startBounds.getWidth() << "," << startBounds.getHeight() << ", width,height: " << width << "," << height << std::endl;
			if (width < 1 || height < 1) return;
			if (width == 1 || height == 1){
				std::list<Line> nearbyWalls;
				getWallsForBounds_ (nearbyWalls, correctedBounds);
				if (nearbyWalls.size() == 0 && (startX != source->x() || startY != source->y())){
					runningList.push_back (createPathUnit_ (
						startX, startY, source->distanceTravelled() + 
						//abs (source->x() - startX) + abs (source->y() - startY),
						estimateDistance_ (source->x(), source->y(), startX, startY),
						estimateDistance_ (startX, startY, targetX, targetY), source
					));
				}
				//std::cout << "reached width or height 1 scenario" << std::endl;
				return;
			}
			int sourceX = source->x();
			int sourceY = source->y();
			int endX = startX + width;
			int endY = startY + height;
			if (sourceX < startX || sourceY < startY || sourceX >= endX || sourceY >= endY){
				//std::cout << "outside start bounds" << endl;
				int toAddX;
				int toAddY;
				toAddX = sourceX - 1;
				toAddY = sourceY;
				if (toAddX >= startX && toAddX < endX && toAddY >= startY && toAddY < endY){
					runningList.push_back (createPathUnit_ (
						toAddX, toAddY, source->distanceTravelled() + 
						//abs (sourceX - toAddX) + abs (sourceY - toAddY),
						estimateDistance_ (sourceX, sourceY, toAddX, toAddY),
						estimateDistance_ (toAddX, toAddY, targetX, targetY), source
					));
				}
				toAddX = sourceX + 1;
				toAddY = sourceY;
				if (toAddX >= startX && toAddX < endX && toAddY >= startY && toAddY < endY){
					runningList.push_back (createPathUnit_ (
						toAddX, toAddY, source->distanceTravelled() + 
						//abs (sourceX - toAddX) + abs (sourceY - toAddY),
						estimateDistance_ (sourceX, sourceY, toAddX, toAddY),
						estimateDistance_ (toAddX, toAddY, targetX, targetY), source
					));
				}
				toAddX = sourceX;
				toAddY = sourceY - 1;
				if (toAddX >= startX && toAddX < endX && toAddY >= startY && toAddY < endY){
					runningList.push_back (createPathUnit_ (
						toAddX, toAddY, source->distanceTravelled() + 
						//abs (sourceX - toAddX) + abs (sourceY - toAddY),
						estimateDistance_ (sourceX, sourceY, toAddX, toAddY),
						estimateDistance_ (toAddX, toAddY, targetX, targetY), source
					));
				}
				toAddX = sourceX;
				toAddY = sourceY + 1;
				if (toAddX >= startX && toAddX < endX && toAddY >= startY && toAddY < endY){
					runningList.push_back (createPathUnit_ (
						toAddX, toAddY, source->distanceTravelled() + 
						//abs (sourceX - toAddX) + abs (sourceY - toAddY),
						estimateDistance_ (sourceX, sourceY, toAddX, toAddY),
						estimateDistance_ (toAddX, toAddY, targetX, targetY), source
					));
				}
				return;
			}
			std::list<Line> nearbyWalls;
			getWallsForBounds_ (nearbyWalls, correctedBounds);
			if (nearbyWalls.size() == 0){
				// Sectorize and grab adjacent path units.
				//bool endPointInside = ((targetX > startX) && (targetX + 1 < startX + width) && (targetY > startY) && (targetY + 1 < startY + height));
				bool endPointInside = ((targetX >= startX) && (targetX < startX + width) && (targetY >= startY) && (targetY < startY + height));
				if (endPointInside){ 
					runningList.push_back (createPathUnit_ (
						targetX, targetY, source->distanceTravelled() + 
						//abs (sourceX - targetX) + abs (sourceY - targetY),
						estimateDistance_ (sourceX, sourceY, targetX, targetY),
						0, source
					));
					return;
				}
				list<IntRect> sectorized;
				sectorize_ (sectorized, correctedBounds, characterWidth, characterHeight);
				for (list<IntRect>::iterator iter = sectorized.begin(); iter != sectorized.end(); iter++){
					//std::cout << "sourceX,Y: " << sourceX << "," << sourceY << std::endl;
					if (iter->getX() != sourceX || iter->getY() != sourceY){
						runningList.push_back (createPathUnit_ (
							iter->getX(), iter->getY(), source->distanceTravelled() + 
							//abs (sourceX - iter->getX()) + abs (sourceY - iter->getY()),
							estimateDistance_ (sourceX, sourceY, iter->getX(), iter->getY()),
							estimateDistance_ (iter->getX(), iter->getY(), targetX, targetY), source
						));
					}
				}
			}
			else{
				vector<IntRect> quads;
				quadrisect_ (quads, correctedBounds);
				int quadBoundX = quads [0].getX() + quads [0].getWidth();
				int quadBoundY = quads [0].getY() + quads [0].getHeight();
				if (sourceX - 1 < quadBoundX && sourceY - 1 < quadBoundY){
					getAdjacentPathUnits_ (runningList, source, targetX, targetY, quads [0], characterWidth, characterHeight);
				}
				if (sourceX + 1 >= quadBoundX && sourceY - 1 < quadBoundY){
					getAdjacentPathUnits_ (runningList, source, targetX, targetY, quads [1], characterWidth, characterHeight);
				}
				if (sourceX - 1 < quadBoundX && sourceY + 1 >= quadBoundY){
					getAdjacentPathUnits_ (runningList, source, targetX, targetY, quads [2], characterWidth, characterHeight);
				}
				if (sourceX + 1 >= quadBoundX && sourceY + 1 >= quadBoundY){
					getAdjacentPathUnits_ (runningList, source, targetX, targetY, quads [3], characterWidth, characterHeight);
				}
			}
		}
		void getWallsForBounds_ (std::list<Line>& runningList, const IntRect& bounds) const{
			Point center (bounds.getX() + (bounds.getWidth() / 2.0), bounds.getY() + (bounds.getHeight() / 2.0));
			Rect sectorBounds (bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight());
			std::vector<Line> nearbyWalls;
			walls_.getSegmentsInRadius (nearbyWalls, center, Utils::SQRT_2() * max (bounds.getWidth(), bounds.getHeight()) / 2.0);
			for (std::vector<Line>::const_iterator iter = nearbyWalls.begin(); iter != nearbyWalls.end(); iter++){
				//cout << "checking " << iter->toString() << endl;
				if (sectorBounds.intersectsLine (*iter)) runningList.push_back (*iter);
			}
		}
		void quadrisect_ (std::vector<IntRect>& runningList, const IntRect& sectorBounds) const{
			int startX = sectorBounds.getX();
			int startY = sectorBounds.getY();
			int width = sectorBounds.getWidth();
			int height = sectorBounds.getHeight();
			int midX = startX + (width / 2);
			int midY = startY + (height / 2);
			int endX = startX + width;
			int endY = startY + height;
			if (width < 1 || height < 1){
			}
			else if (width == 1 && height == 1){
				runningList.push_back (sectorBounds);
			}
			else if (width == 1){
				for (int y = startY; y < endY; y++){
					runningList.push_back (IntRect (startX, y, width, midY - startY));
				}
			}
			else if (height == 1){
				for (int x = startX; x < endX; x++){
					runningList.push_back (IntRect (x, startY, midX - startX, height));
				}
			}
			else{
				int offsetX = (width % 2);
				int offsetY = (height % 2);
				runningList.push_back (IntRect (startX, startY, midX - startX, midY - startY));
				runningList.push_back (IntRect (midX, startY, midX + offsetX - startX, midY - startY));
				runningList.push_back (IntRect (startX, midY, midX - startX, midY + offsetY - startY));
				runningList.push_back (IntRect (midX, midY, midX + offsetX - startX, midY + offsetY - startY));
			}
		}
		void getClearedEdgeRects_ (list<IntRect>& runningList, const IntRect& edge, const EdgeFacing facing, int characterWidth, int characterHeight) const{
			if (max (edge.getHeight(), edge.getWidth()) > 3){
				IntRect toCheck;
				int charLeft = int (characterWidth / 2);
				int charTop = int (characterHeight / 2);
				if (facing == facing_up) toCheck = IntRect (edge.getX() - charLeft, edge.getY() - charTop - 1, 
					edge.getWidth() + characterWidth - 1, characterHeight);
				else if (facing == facing_down) toCheck = IntRect (edge.getX() - charLeft, edge.getY() - charTop + 1, 
					edge.getWidth() + characterWidth - 1, characterHeight);
				else if (facing == facing_right) toCheck = IntRect (edge.getX() - charLeft + 1, edge.getY() - charTop, 
					characterWidth, edge.getHeight() + characterHeight - 1);
				else if (facing == facing_left) toCheck = IntRect (edge.getX() - charLeft - 1, edge.getY() - charTop, 
					characterWidth, edge.getHeight() + characterHeight - 1);
				//cout << "edge is " << edge.getX() << "," << edge.getY() << "," << edge.getWidth() << "," << edge.getHeight() << endl;
				//cout << "toCheck is " << toCheck.getX() << "," << toCheck.getY() << "," << toCheck.getWidth() << "," << toCheck.getHeight() << endl;
				list<Line> nearbyWalls;
				getWallsForBounds_ (nearbyWalls, toCheck);
				/*for (list<Line>::const_iterator iter = nearbyWalls.begin(); iter != nearbyWalls.end(); iter++){
					cout << "wall " << iter->toString() << endl;
				}*/
				if (nearbyWalls.size() == 0){
					if (facing == facing_up || facing == facing_down){
						runningList.push_back (IntRect (edge.getX(), edge.getY(), 1, 1));
						runningList.push_back (IntRect (edge.getX() + (edge.getWidth() / 2), edge.getY(), 1, 1));
						runningList.push_back (IntRect (edge.getX() + edge.getWidth() - 1, edge.getY(), 1, 1));
					}
					else if (facing == facing_right || facing == facing_left){
						runningList.push_back (IntRect (edge.getX(), edge.getY(), 1, 1));
						runningList.push_back (IntRect (edge.getX(), edge.getY() + (edge.getHeight() / 2), 1, 1));
						runningList.push_back (IntRect (edge.getX(), edge.getY() + edge.getHeight() - 1, 1, 1));
					}
				}
				else{
					if (facing == facing_up || facing == facing_down){
						IntRect newEdgeLeft (edge.getX(), edge.getY(), edge.getWidth() / 2, 1);
						IntRect newEdgeRight (edge.getX() + (edge.getWidth() / 2), edge.getY(), edge.getWidth() / 2 + (edge.getWidth() % 2), 1);
						getClearedEdgeRects_ (runningList, newEdgeLeft, facing, characterWidth, characterHeight);
						getClearedEdgeRects_ (runningList, newEdgeRight, facing, characterWidth, characterHeight);
					}
					else if (facing == facing_right || facing == facing_left){
						IntRect newEdgeTop (edge.getX(), edge.getY(), 1, edge.getHeight() / 2);
						IntRect newEdgeBottom (edge.getX(), edge.getY() + (edge.getHeight() / 2), 1, edge.getHeight() / 2 + (edge.getHeight() % 2));
						getClearedEdgeRects_ (runningList, newEdgeTop, facing, characterWidth, characterHeight);
						getClearedEdgeRects_ (runningList, newEdgeBottom, facing, characterWidth, characterHeight);
					}
				}
			}
			else{
				if (facing == facing_up || facing == facing_down) 
					for (int x = edge.getX(); x < edge.getX() + edge.getWidth(); x++) runningList.push_back (IntRect (x, edge.getY(), 1, 1));
				if (facing == facing_right || facing == facing_left) 
					for (int y = edge.getY(); y < edge.getY() + edge.getHeight(); y++) runningList.push_back (IntRect (edge.getX(), y, 1, 1));
			}
		}
		void getInnerPerimeterRects_ (std::list<IntRect>& runningList, const IntRect& sectorBounds, int characterWidth, int characterHeight) const{
			if (sectorBounds.getWidth() != sectorBounds.getHeight()){
				ostringstream msg;
				msg << "QuadPathFinder::getInnerPerimeterRects() was not passed a square sectorBounds! Width: " 
					<< sectorBounds.getWidth() << ", height: " << sectorBounds.getHeight();
				cout << msg.str() << endl;
				return;
			}
			if (sectorBounds.getWidth() < 1) return;
			if (sectorBounds.getWidth() == 1){
				runningList.push_back (sectorBounds);
				return;
			}
			// We always add the corners.
			runningList.push_back (IntRect (sectorBounds.getX(), sectorBounds.getY(), 1, 1));
			runningList.push_back (IntRect (sectorBounds.getX() + sectorBounds.getWidth() - 1, sectorBounds.getY(), 1, 1));
			runningList.push_back (IntRect (sectorBounds.getX() + sectorBounds.getWidth() - 1, sectorBounds.getY() + sectorBounds.getHeight() - 1, 1, 1));
			runningList.push_back (IntRect (sectorBounds.getX(), sectorBounds.getY() + sectorBounds.getHeight() - 1, 1, 1));
			if (sectorBounds.getWidth() == 2) return;
			// Add the rest of the edges (don't include the corners, which we already added).
			getClearedEdgeRects_ (runningList, IntRect (sectorBounds.getX() + 1, sectorBounds.getY(), sectorBounds.getWidth() - 2, 1), facing_up, characterWidth, characterHeight);
			getClearedEdgeRects_ (runningList, IntRect (sectorBounds.getX() + sectorBounds.getWidth() - 1, sectorBounds.getY() + 1, 1, sectorBounds.getHeight() - 2), facing_right, characterWidth, characterHeight);
			getClearedEdgeRects_ (runningList, IntRect (sectorBounds.getX() + 1, sectorBounds.getY() + sectorBounds.getHeight() - 1, sectorBounds.getWidth() - 2, 1), facing_down, characterWidth, characterHeight);
			getClearedEdgeRects_ (runningList, IntRect (sectorBounds.getX(), sectorBounds.getY() + 1, 1, sectorBounds.getHeight() - 2), facing_left, characterWidth, characterHeight);
		}
		void sectorize_ (std::list<IntRect>& runningList, const IntRect& sectorBounds, int characterWidth, int characterHeight) const{
			int width = sectorBounds.getWidth();
			int height = sectorBounds.getHeight();
			if (width != height){
				std::ostringstream msg;
				msg << "sectorize_ was not passed a square sectorBounds! Width: " << width << ", height: " << height << endl;
				std::cout << msg.str() << endl;
			}
			if (width < 1 || height < 1){
			}
			else if (height < 4 || width < 4){
				for (int y = 0; y < height; y++){
					for (int x = 0; x < width; x++){
						runningList.push_back (IntRect (sectorBounds.getX() + x, sectorBounds.getY() + y));
					}
				}
			}
			else{
				getInnerPerimeterRects_ (runningList, sectorBounds, characterWidth, characterHeight);
			}
		}

		SegmentContainer& walls_;
		std::list<PathUnit*> allPathUnits_;
		std::list<CachedPath> cachedPaths_;
};


#endif
