#ifndef SEGMENT_CONTAINER_H
#define SEGMENT_CONTAINER_H


#include "geometry.h"
#include "rect.h"
#include "interface_segmentContainerListener.h"
#include "utils.h"
#include "utilTypes.h"
#include <algorithm>
#include <vector>
#include <iostream>
#include <map>
#ifdef WIN32
	#include <memory>
#else
	#include <tr1/memory>
#endif


class SegmentContainerEntry{
	public:
		SegmentContainerEntry (const Line& line) : line_ (line), checked_ (false) { }
		~SegmentContainerEntry(){}

		void setChecked (const bool& checked){ checked_ = checked; }
		bool getChecked() const{ return checked_; }

		Line getLine() const{ return line_; }

	private:
		Line line_;
		bool checked_;
};

class SegmentIntersection{
	public:
		SegmentIntersection (const Line& line, const Point& point) : line_ (line), point_ (point) { }
		~SegmentIntersection(){}

		Line getLine(){ return line_; }
		Point getPoint(){ return point_; }

	private:
		Line line_;
		Point point_;
};
class SegmentContainerEntryIntersection{
	public:
		SegmentContainerEntryIntersection (SegmentContainerEntry* entry, const Point& intersectionPoint) : entry_ (entry), point_ (intersectionPoint) { }
		~SegmentContainerEntryIntersection(){}

		SegmentContainerEntry* getEntry(){ return entry_; }
		Point getPoint(){ return point_; }

	private:
		SegmentContainerEntry* entry_;
		Point point_;
};

class SegmentContainerNode{
	public:
		SegmentContainerNode (const Rect& rect, const unsigned int& maxSegments) 
			: rect_ (rect), center_ (rect.getCenter()), maxSegments_ (maxSegments), split_ (false) {
			for (int x = 0; x < 4; x++){
				nodes_ [x] = NULL;
			}
		}
		~SegmentContainerNode(){
			for (int x = 0; x < 4; x++){
				delete nodes_ [x]; 
			}
		}

		void getSegmentsInRadius (std::vector<SegmentContainerEntry*>& runningList, const Point& point, const double& radius){
			//cout << "Looking for point " << point.toString() << ", radius " << radius << ", center " << center_.toString() << endl;
			if (split_){
				std::vector<SegmentContainerNode*> eligible;
				if (point.getX() - radius <= center_.getX() && point.getY() - radius <= center_.getY()){
					//cout << "Node 0 eligible" << endl;
					eligible.push_back (nodes_ [0]);
				}
				if (point.getX() + radius >= center_.getX() && point.getY() - radius <= center_.getY()){	
					//cout << "Node 1 eligible" << endl;
					eligible.push_back (nodes_ [1]);
				}
				if (point.getX() - radius <= center_.getX() && point.getY() + radius >= center_.getY()){
					//cout << "Node 2 eligible" << endl;
					eligible.push_back (nodes_ [2]);
				}
				if (point.getX() + radius >= center_.getX() && point.getY() + radius >= center_.getY()){
					//cout << "Node 3 eligible" << endl;
					eligible.push_back (nodes_ [3]);
				}
				for (std::vector<SegmentContainerNode*>::const_iterator iter = eligible.begin(); iter != eligible.end(); iter++){
					(*iter)->getSegmentsInRadius (runningList, point, radius);
				}
			}
			else{
				//cout << "found a leaf with " << entries_.size() << " entries:" << endl;
				for (std::vector<SegmentContainerEntry*>::const_iterator iter = entries_.begin(); iter != entries_.end(); iter++){
					//cout << (*iter)->getLine().toString();
					//cout << ", " << ((*iter)->getChecked() ? "" : "not") << " checked" << endl;
					if (!(*iter)->getChecked() && (*iter)->getLine().distanceToPoint (point) <= radius){
						runningList.push_back (*iter);
					}
					(*iter)->setChecked (true);
				}
			}
		}		
		void getIntersects (std::vector<SegmentContainerEntryIntersection>& runningList, const Line& line){
			if (split_){
				std::vector<SegmentContainerNode*> nodesTouched;
				getNodesTouched (nodesTouched, line);
				for (std::vector<SegmentContainerNode*>::iterator iter = nodesTouched.begin(); iter != nodesTouched.end(); iter++){
					(*iter)->getIntersects (runningList, line);
				}
			}
			else{
				for (std::vector<SegmentContainerEntry*>::iterator iter = entries_.begin(); iter != entries_.end(); iter++){
					if ((*iter)->getChecked()) continue;
					Optional<Point> intersectPoint = (*iter)->getLine().getLineIntersectionPoint (line);
					if (intersectPoint.isSet()){
						SegmentContainerEntryIntersection intersect (*iter, intersectPoint.get());
						runningList.push_back (intersect);
					}
					(*iter)->setChecked (true);
				}
			}
		}
		void insert (SegmentContainerEntry* entry, const bool& canSplit = true){
			if (split_){
				std::vector<SegmentContainerNode*> nodesTouched;
				getNodesTouched (nodesTouched, entry->getLine());
				if (nodesTouched.size() == 0){ 
					std::cout << "line " << entry->getLine().toString() << " could not find a home among nodes. . ." << std::endl;
					for (int x = 0; x < 4; x++){
						std::cout << nodes_ [x]->getRect().toString() << std::endl;
					}
				}
				for (std::vector<SegmentContainerNode*>::iterator iter = nodesTouched.begin(); iter != nodesTouched.end(); iter++){
					(*iter)->insert (entry, canSplit);
				}
			}
			else if (entries_.size() >= maxSegments_ && canSplit){
				// Split into parts.
				entries_.push_back (entry);
				double midX = rect_.getCenter().getX();
				double midY = rect_.getCenter().getY();
				double midWidth = rect_.getWidth() / 2;
				double midHeight = rect_.getHeight() / 2;
				Rect rect1 = Rect (rect_.getX(), rect_.getY(), midWidth, midHeight);
				Rect rect2 = Rect (midX, rect_.getY(), midWidth, midHeight);
				Rect rect3 = Rect (rect_.getX(), midY, midWidth, midHeight);
				Rect rect4 = Rect (midX, midY, midWidth, midHeight);
				nodes_ [0] = new SegmentContainerNode (rect1, maxSegments_);
				nodes_ [1] = new SegmentContainerNode (rect2, maxSegments_);
				nodes_ [2] = new SegmentContainerNode (rect3, maxSegments_);
				nodes_ [3] = new SegmentContainerNode (rect4, maxSegments_);
				split_ = true;
				for (std::vector<SegmentContainerEntry*>::iterator iter = entries_.begin(); iter != entries_.end(); iter++){
					insert ((*iter), false);
				}
				entries_.clear();
			}
			else{
				entries_.push_back (entry);
			}
		}

		Rect getRect() const{ return rect_; }

	private:
		void getNodesTouched (std::vector<SegmentContainerNode*>& runningList, const Line& line){
			
			for (int x = 0; x < 4; x++){
				if (nodes_ [x]->getRect().intersectsLine (line)) runningList.push_back (nodes_ [x]);
			}
			return;
		}
	
		SegmentContainerNode* nodes_ [4];
		Rect rect_;
		Point center_;
		unsigned int maxSegments_;
		bool split_;
		std::vector<SegmentContainerEntry*> entries_;
};

class SegmentsInRadiusCacheEntry{
	public: 
		SegmentsInRadiusCacheEntry (const Point& point, double radius) : first (point), second (radius){ }
		~SegmentsInRadiusCacheEntry(){}

		bool operator< (const SegmentsInRadiusCacheEntry& rhs) const{
			return (first < rhs.first) || (first == rhs.first && Utils::compareDoubles (second, rhs.second) == 1);
		}
		bool operator== (const SegmentsInRadiusCacheEntry& rhs) const{
			return (first == rhs.first) && (Utils::compareDoubles (second, rhs.second) == 0);
		}

		Point first;
		double second;
};

//typedef std::pair<Point, double> SegmentsInRadiusCacheEntry;
class SegmentsInRadiusCacheEntryList{
	public:
		SegmentsInRadiusCacheEntryList (const SegmentsInRadiusCacheEntry& entry) : entry_ (entry) { }
		~SegmentsInRadiusCacheEntryList(){}

		const SegmentsInRadiusCacheEntry& getEntry() const{ return entry_; }
		void pop(){ if (previous_) previous_->setNext (next_); if (next_) next_->setPrevious (previous_); }
		void placeBetween (std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> thisPtr, 
				std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> previous, std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> next){ 
			if (previous == thisPtr) previous = std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList>();
			if (next == thisPtr) next = std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList>();
			pop();
			if (previous) previous->setNext (thisPtr);
			if (next) next->setPrevious (thisPtr);
			previous_ = previous;
			next_ = next;
		}
		std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> getPrevious() const{ return previous_; }
		void setPrevious (std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> previous){ previous_ = previous; }
		std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> getNext() const{ return next_; }
		void setNext (std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> next){ next_ = next; }

	private:
		SegmentsInRadiusCacheEntry entry_;
		std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> next_;
		std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> previous_;
};
typedef std::tr1::shared_ptr<SegmentsInRadiusCacheEntryList> SegmentsInRadiusCacheEntryListPtr;

//#define SEG_CONT_CACHE_MASSIVE_DEBUG

typedef std::pair< std::vector<Line>, SegmentsInRadiusCacheEntryListPtr > SegmentsInRadiusCacheEntryResult;
typedef std::map< SegmentsInRadiusCacheEntry, SegmentsInRadiusCacheEntryResult > SegmentsInRadiusCacheContainer;
class SegmentsInRadiusCache{
	public:
		SegmentsInRadiusCache (unsigned int maxCached = 2000) : maxCached_ (maxCached) { }
		~SegmentsInRadiusCache(){}
		Optional <std::vector <Line> > get (const Point& point, double radius){
			#ifdef SEG_CONT_CACHE_MASSIVE_DEBUG
				cout << "retrieving " << point.toString() << ", " << radius << " from cache" << endl;
			#endif
			SegmentsInRadiusCacheContainer::iterator iter = cache_.find (SegmentsInRadiusCacheEntry (point, radius));
			if (iter != cache_.end()){ 
				#ifdef SEG_CONT_CACHE_MASSIVE_DEBUG
					cout << "cache hit" << endl;
					cout << "\tcache is now: " << endl;
					for (SegmentsInRadiusCacheEntryListPtr next = head_; next; next = next->getNext()){
						cout << "\t" << next->getEntry().first.toString() << ", " << next->getEntry().second << endl;
					}
					cout << "\tunordered cache is: " << endl;
					for (SegmentsInRadiusCacheContainer::const_iterator iter2 = cache_.begin(); iter2 != cache_.end(); iter2++){
						cout << "\t" << iter2->first.first.toString() << ", " << iter2->first.second << endl;
					}
					cout << "\thead is " << (head_ ? head_->getEntry().first.toString() : "NULL") << endl;
					cout << "\ttail is " << (tail_ ? tail_->getEntry().first.toString() : "NULL") << endl;
				#endif
				SegmentsInRadiusCacheEntryListPtr result = iter->second.second;
				if (tail_ == result) tail_ = result->getPrevious();
				if (head_ != result){
					result->placeBetween (result, SegmentsInRadiusCacheEntryListPtr(), head_);
					head_ = result;
				}
				#ifdef SEG_CONT_CACHE_MASSIVE_DEBUG
					cout << "cache lru updated" << endl;
					cout << "\tcache is now: " << endl;
					for (SegmentsInRadiusCacheEntryListPtr next = head_; next; next = next->getNext()){
						cout << "\t" << next->getEntry().first.toString() << ", " << next->getEntry().second << endl;
					}
					cout << "\tunordered cache is: " << endl;
					for (SegmentsInRadiusCacheContainer::const_iterator iter2 = cache_.begin(); iter2 != cache_.end(); iter2++){
						cout << "\t" << iter2->first.first.toString() << ", " << iter2->first.second << endl;
					}
					cout << "\thead is " << (head_ ? head_->getEntry().first.toString() : "NULL") << endl;
					cout << "\ttail is " << (tail_ ? tail_->getEntry().first.toString() : "NULL") << endl;
				#endif
				return Optional< std::vector <Line> > (iter->second.first);
			}
			#ifdef SEG_CONT_CACHE_MASSIVE_DEBUG
				cout << "cache miss" << endl;
				cout << "\tcache is now: " << endl;
				for (SegmentsInRadiusCacheEntryListPtr next = head_; next; next = next->getNext()){
					cout << "\t" << next->getEntry().first.toString() << ", " << next->getEntry().second << endl;
				}
				cout << "\tunordered cache is: " << endl;
				for (SegmentsInRadiusCacheContainer::const_iterator iter = cache_.begin(); iter != cache_.end(); iter++){
					cout << "\t" << iter->first.first.toString() << ", " << iter->first.second << endl;
				}
				cout << "\thead is " << (head_ ? head_->getEntry().first.toString() : "NULL") << endl;
				cout << "\ttail is " << (tail_ ? tail_->getEntry().first.toString() : "NULL") << endl;
			#endif
			return Optional< std::vector <Line> >();
		}
		void set (const Point& point, double radius, const std::vector<Line>& segments){
			//cout << "adding " << point.toString() << ", " << radius << " to cache" << endl;
			SegmentsInRadiusCacheEntry cacheEntry (point, radius);
			SegmentsInRadiusCacheEntryListPtr lruEntry = SegmentsInRadiusCacheEntryListPtr (new SegmentsInRadiusCacheEntryList (cacheEntry));
			cache_ [cacheEntry] = SegmentsInRadiusCacheEntryResult (segments, lruEntry);
			lruEntry->placeBetween (lruEntry, SegmentsInRadiusCacheEntryListPtr(), head_);
			head_ = lruEntry;
			if (!tail_) tail_ = head_;
			if (cache_.size() > maxCached_){
				SegmentsInRadiusCacheEntryListPtr popped = tail_; 
				tail_ = popped->getPrevious();
				popped->pop();
				SegmentsInRadiusCacheContainer::iterator found = cache_.find (popped->getEntry());
				if (found != cache_.end()) cache_.erase (found);
				#ifdef SEG_CONT_CACHE_MASSIVE_DEBUG
					cout << "dumping " << popped->getEntry().first.toString() << endl;
					cout << "\tcache is now: " << endl;
					for (SegmentsInRadiusCacheEntryListPtr next = head_; next; next = next->getNext()){
						cout << "\t" << next->getEntry().first.toString() << ", " << next->getEntry().second << endl;
					}
					cout << "\tunordered cache is now: " << endl;
					for (SegmentsInRadiusCacheContainer::const_iterator iter = cache_.begin(); iter != cache_.end(); iter++){
						cout << "\t" << iter->first.first.toString() << ", " << iter->first.second << endl;
					}
					cout << "\thead is " << (head_ ? head_->getEntry().first.toString() : "NULL") << endl;
					cout << "\ttail is " << (tail_ ? tail_->getEntry().first.toString() : "NULL") << endl;
				#endif
			}
		}
		void clear(){
			cache_.clear();
		}
	private:
		SegmentsInRadiusCacheContainer cache_;
		unsigned int maxCached_;
		SegmentsInRadiusCacheEntryListPtr head_;
		SegmentsInRadiusCacheEntryListPtr tail_;
};

class SegmentContainer{
	public:
		SegmentContainer (const Rect& bounds, const unsigned int& maxSegments = 10) : 
			bounds_ (bounds), maxSegments_ (maxSegments), itemCount_ (0), 
			root_ (new SegmentContainerNode (bounds_, maxSegments)) { }
		~SegmentContainer(){ 
			delete root_; root_ = NULL; 
			freeEntries();
		}
		void addListener (Interface_SegmentContainerListener* newListener){
			if (!hasListener (newListener)) listeners_.push_back (newListener);
		}
		bool hasListener (Interface_SegmentContainerListener* listener){
			std::vector<Interface_SegmentContainerListener*>::iterator loc = std::find (listeners_.begin(), listeners_.end(), listener);
			return loc != listeners_.end();
		}
		void removeListener (Interface_SegmentContainerListener* listener){
			std::vector<Interface_SegmentContainerListener*>::iterator loc = std::find (listeners_.begin(), listeners_.end(), listener);
			if (loc != listeners_.end()) listeners_.erase (loc);
		}
		void clear(){
			segmentsInRadiusCache_.clear();
			delete root_; root_ = new SegmentContainerNode (bounds_, maxSegments_);
			itemCount_ = 0;
			freeEntries();
			for (std::vector<Interface_SegmentContainerListener*>::iterator iter = listeners_.begin(); iter != listeners_.end(); iter++){
				(*iter)->segmentsCleared (this);
			}
		}

		void getAllSegments (std::vector<Line>& runningList){
			for (std::vector<SegmentContainerEntry*>::iterator iter = allEntries_.begin(); 
					iter != allEntries_.end(); iter++){
				runningList.push_back ((*iter)->getLine());
			}
		}

		void getSegmentsInRadius (std::vector<Line>& runningList, const Point& point, const double& radius){
			//cout << "getting segs in radius of " << point.toString() << " with radius " << radius << endl;
			Optional <std::vector <Line> > cachedSegments = segmentsInRadiusCache_.get (point, radius);
			if (cachedSegments.isSet()){
				//cout << "segments in radius cache hit" << endl;
				std::vector<Line>& cached = cachedSegments.get();
				for (std::vector<Line>::iterator iter = cached.begin(); iter != cached.end(); iter++){
					runningList.push_back (*iter);
				}
			}
			else{
				//cout << "segments in radius cache miss" << endl;
				std::vector<SegmentContainerEntry*> entryList;
				root_->getSegmentsInRadius (entryList, point, radius);
				//cout << "entryList has " << entryList.size() << endl;
				for (std::vector<SegmentContainerEntry*>::iterator iter = entryList.begin(); iter != entryList.end(); iter++){
					runningList.push_back ((*iter)->getLine());
				}
				segmentsInRadiusCache_.set (point, radius, runningList);
			}
			setAllChecked (false);
			//cout << "got " << runningList.size() << " segs" << endl;
		}
		void insert (const Line& line){
			segmentsInRadiusCache_.clear();
			//cout << "inserting segment " << line.toString() << endl;
			itemCount_++;
			SegmentContainerEntry* newEntry = new SegmentContainerEntry (line);
			allEntries_.push_back (newEntry);
			root_->insert (newEntry);
			for (std::vector<Interface_SegmentContainerListener*>::iterator iter = listeners_.begin(); iter != listeners_.end(); iter++){
				(*iter)->segmentAdded (this, line);
			}
		}
		void getIntersects (std::vector<SegmentIntersection>& runningList, const Line& line){
			// It's faster to just do a linear search.  :P  Sucks to find out after spending so many hours on this thing!
			for (std::vector<SegmentContainerEntry*>::const_iterator iter = allEntries_.begin(); iter != allEntries_.end(); iter++){
				Optional<Point> intersectPoint ((*iter)->getLine().getLineIntersectionPoint (line));
				if (intersectPoint.isSet()) runningList.push_back (SegmentIntersection ((*iter)->getLine(), intersectPoint.get()));
			}
		}
		unsigned int size() const{ return itemCount_; }

		Rect getBounds() const{ return bounds_; }

		/*
		int absoluteCount(){
			int count = 0;
			root_->markAllChecked();
			for (std::vector<SegmentContainerEntry*>::iterator iter = allEntries_.begin(); iter != allEntries_.end(); iter++){
				if ((*iter)->getChecked()) count++;
				(*iter)->setChecked (false);
			}
			return count;
		}
		*/
		
	private:
		void freeEntries(){
			for (std::vector<SegmentContainerEntry*>::iterator iter = allEntries_.begin(); iter != allEntries_.end(); iter++){
				delete *iter;
			} allEntries_.clear();
		}

		void setAllChecked (bool checked){
			for (std::vector<SegmentContainerEntry*>::iterator iter = allEntries_.begin(); iter != allEntries_.end(); iter++){
				(*iter)->setChecked (checked);
			}
		}

		Rect bounds_;
		unsigned int maxSegments_;
		unsigned int itemCount_;
		SegmentContainerNode* root_;
		std::vector<SegmentContainerEntry*> allEntries_;
		std::vector<Interface_SegmentContainerListener*> listeners_;
		SegmentsInRadiusCache segmentsInRadiusCache_;
};


#endif
