#include "soundInstance.h"
#include "interface_soundInstanceObserver.h"
#include <algorithm>
#include <iostream>
#include "lock.h"
#include "audioEffect.h"
#include <SDL/SDL.h>

using namespace std;


SoundInstance::SoundInstance (const std::string& name, Uint32 length, double bytesPerMs) 
	: name_ (name), bufferPosition_ (0), length_ (length), lastPlayTime_ (0), 
	  lastPlayTimePosition_ (0), state_ (stateStopped), pan_ (0.5), volume_ (1.0), 
	  bytesPerMs_ (bytesPerMs), inUse_ (true), timePassed_ (0)
{ 
	//cout << "sound instance created. length " << length << " bytesPerMs " << bytesPerMs << endl;
}
SoundInstance::~SoundInstance(){
	std::vector<Interface_SoundInstanceObserver*> toNotify;
	{
		Lock observersLock (&observersLock_);
		for (std::vector<Interface_SoundInstanceObserver*>::iterator iter = observers_.begin();
				iter != observers_.end(); iter++){
			toNotify.push_back (*iter);
		}
	}
	for (std::vector<Interface_SoundInstanceObserver*>::iterator iter = toNotify.begin();
			iter != toNotify.end(); iter++){
		(*iter)->soundInstanceDestroying (this);
	}
	{
		Lock effectsLock (&effectsLock_);
		for (std::vector<AudioEffect*>::iterator iter = effects_.begin(); iter != effects_.end(); iter++){
			delete *iter;
		}
		effects_.clear();
	}	
}

unsigned int SoundInstance::getTimePassed(){
	Lock timePassedLock (&timePassedLock_);
	return timePassed_;
}
void SoundInstance::setTimePassed (unsigned int time){
	Lock timePassedLock (&timePassedLock_);
	timePassed_ = time;
}
void SoundInstance::update (unsigned int timePassed){
	setTimePassed (getTimePassed() + timePassed);
}

bool SoundInstance::getInUse(){
	Lock inUseLock (&inUseLock_);
	return inUse_;
}
void SoundInstance::setInUse (bool inUse){
	Lock inUseLock (&inUseLock_);
	inUse_ = inUse;
}

std::vector<Interface_SoundInstanceObserver*>::iterator SoundInstance::findObserver (
		Interface_SoundInstanceObserver* observer){
	std::vector<Interface_SoundInstanceObserver*>::iterator iter = 
		std::find (observers_.begin(), observers_.end(), observer);
	return iter;
}
void SoundInstance::addObserver (Interface_SoundInstanceObserver* observer){
	Lock observersLock (&observersLock_);
	if (findObserver (observer) == observers_.end()) observers_.push_back (observer);
}
void SoundInstance::removeObserver (Interface_SoundInstanceObserver* observer){
	Lock observersLock (&observersLock_);
	std::vector<Interface_SoundInstanceObserver*>::iterator iter = findObserver (observer);
	if (iter != observers_.end()) observers_.erase (iter);
}

double SoundInstance::getPan(){
	Lock panLock (&panLock_);
	return pan_;
}
void SoundInstance::setPan (double pan){
	Lock panLock (&panLock_);
	pan_ = pan;
}
double SoundInstance::getVolume(){
	Lock volumeLock (&volumeLock_);
	return volume_;
}
void SoundInstance::setVolume (double volume){
	Lock volumeLock (&volumeLock_);
	volume_ = volume;
}
Uint32 SoundInstance::getLastPlayTimePosition(){
	Lock lastPlayTimePositionLock (&lastPlayTimePositionLock_);
	return lastPlayTimePosition_;
}
void SoundInstance::setLastPlayTimePosition (Uint32 position){
	Lock lastPlayTimePositionLock (&lastPlayTimePositionLock_);
	lastPlayTimePosition_ = position;
}
Uint32 SoundInstance::getLastPlayTime(){
	Lock lastPlayTimeLock (&lastPlayTimeLock_);
	return lastPlayTime_;
}
void SoundInstance::setLastPlayTime (Uint32 time){
	Lock lastPlayTimeLock (&lastPlayTimeLock_);
	lastPlayTime_ = time;
}
Uint32 SoundInstance::getPlayPosition(){
	if (getState() == stateStopped) return 0;
	if (getState() == statePaused) return lastPlayTimePosition_;
	// Not exact. If sound skips, this will be off! May be able to mitigate by having 
	//	audio manager update play position to buffer position in SDL callback
	Uint32 length = std::min (getTimeLength(), getLastPlayTimePosition() + (SDL_GetTicks() - getLastPlayTime()));
	return length;
}
void SoundInstance::setPlayPosition (Uint32 position){
	setLastPlayTimePosition (position);
	setBufferPosition (Uint32 (position * bytesPerMs_));
}
Uint32 SoundInstance::getBufferPosition(){
	Lock bufferPositionLock (&bufferPositionLock_);
	return bufferPosition_;
}
void SoundInstance::setBufferPosition (Uint32 position){
	Lock bufferPositionLock (&bufferPositionLock_);
	bufferPosition_ = position;
}
Uint32 SoundInstance::getLength(){
	Lock lengthLock (&lengthLock_);
	return length_;
}
Uint32 SoundInstance::getTimeLength(){
	//cout << "in getTimeLength, length is " << getLength() << " bytesPerMs is " << bytesPerMs_ << endl;
	Uint32 length = (Uint32) (getLength() / bytesPerMs_);
	{
		Lock effectsLock (&effectsLock_);
		for (std::vector<AudioEffect*>::iterator iter = effects_.begin(); iter != effects_.end(); iter++){
			length += (*iter)->getTimeLengthOffset (length);
		}
	}
	//cout << "time length is " << length << endl;
	return length;
}
SoundInstance::State SoundInstance::getState(){
	bool shouldStop = false;
	{
		Lock stateLock (&stateLock_);
		if (state_ == statePlaying){
			Uint32 length = getTimeLength();
			//cout << "length " << length << endl;
			Uint32 where = getLastPlayTimePosition() + (SDL_GetTicks() - getLastPlayTime());
			//cout << "where " << where << endl;
			if (where >= length) shouldStop = true;
		}
	}
	if (shouldStop) stop();

	Lock stateLock (&stateLock_);
	return state_;
}
void SoundInstance::setState (SoundInstance::State state){
	Lock stateLock (&stateLock_);
	state_ = state;
}
void SoundInstance::reset(){
	setState (stateStopped);
	setPlayPosition (0);
}
void SoundInstance::play (bool repeating){
	if (getState() != statePaused) reset();
	setLastPlayTime (SDL_GetTicks());
	if (repeating) setState (stateRepeating);
	else setState (statePlaying);
}
void SoundInstance::pause(){
	if (getState() != stateStopped){
		setPlayPosition (getPlayPosition());
		setState (statePaused);
	}
}
void SoundInstance::stop(){
	setPlayPosition (0);
	setState (stateStopped);
}
std::string SoundInstance::getName(){
	// If there's no setting name outside constructor, shouldn't need to lock this.
	return name_;
}
std::vector<AudioEffect*>::iterator SoundInstance::findEffect (AudioEffect* effect){
	return std::find (effects_.begin(), effects_.end(), effect);
}
bool SoundInstance::hasEffect (AudioEffect* effect){
	Lock effectsLock (&effectsLock_);
	return findEffect (effect) != effects_.end();
}
void SoundInstance::addEffect (AudioEffect* effect){
	//cout << "adding effect " << effect << endl;
	if (!hasEffect (effect)){
		Lock effectsLock (&effectsLock_);
		effects_.push_back (effect);
	}
}
void SoundInstance::removeEffect (AudioEffect* effect){
	Lock effectsLock (&effectsLock_);
	std::vector<AudioEffect*>::iterator found = findEffect (effect);
	if (found != effects_.end()) effects_.erase (found);
}
std::vector<AudioEffect*> SoundInstance::getEffects(){
	Lock effectsLock (&effectsLock_);
	return effects_;
}
