/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "surface.h"
#include "assert.h"
#include "animationSet.h"
#include "border.h"
#include "imageProcessingSurfaceManager.h"
#include "camera.h"
#include "transition.h"
#include <algorithm>
#include <list>


Surface::Surface (Surface* parent, SDL_Surface* image){
	camera = NULL;
	debugLevel = 0;
	setColorKeySourcePixelCoords (0, 0);
	visible = true;
	resources = NULL;
	live = true;
	rect = new Rect (0, 0, 0, 0);
	x = 0; y = 0; z = 0;
	useColorKey = false;
	surface = image;
	currentBuffer = 0;
	border = NULL;
	dirty = true;
	this->parent = parent;
    if (parent) parent->addChildSurface (this);
	animations = NULL;
	for (int x = 0; x < DISPLAY_BUFFERS; x++){
		bufferedRects [x] = NULL;
		bufferedViewports [x] = NULL;
		bufferedImageProcessorViewports [x] = NULL;
		bufferedOcclusionports [x] = NULL;
	}
}
Surface::Surface (Surface* parent, string bmpFile, bool getColorKey)
{
	camera = NULL;
	debugLevel = 0;
	setColorKeySourcePixelCoords (0, 0);
	visible = true;
	resources = NULL;
	this->live = true;
    this->rect = new Rect (0, 0, 0, 0);
	this->x = 0; this->y = 0; this->z = 0;
	this->useColorKey = getColorKey;
    this->surface = NULL; 
	this->currentBuffer = 0;
	this->border = NULL; this->dirty = true; this->parent = parent;
	if (parent){
		parent->addChildSurface (this);
	}
	this->animations = NULL;
    if (bmpFile != ""){
        //printf ("Getting bitmap loaded.\n");
        this->loadBitmap (bmpFile, getColorKey);
        //printf ("SDL error: %s .\n", SDL_GetError());
    }
    else{
        this->surface = NULL;
    }
	if (useColorKey && surface != NULL){
		colorKey = deriveColorKey();
		//cout << "Got a color key of " << colorKey << endl;
	}
	for (int x = 0; x < DISPLAY_BUFFERS; x++){
		bufferedRects [x] = NULL;
		bufferedViewports [x] = NULL;
		bufferedImageProcessorViewports [x] = NULL;
		bufferedOcclusionports [x] = NULL;
	}
    //printf ("Surface constructor done.\n");
}

Surface::~Surface()
{
    //printf ("Surface destroying.\n");

	for (vector<Interface_SurfaceEventObserver*>::iterator iter = surfaceEventObservers.begin(); iter != surfaceEventObservers.end(); iter++){
		(*iter)->surfaceDestroying (this);
	}

    this->freeChildren();

	// Tell this surface's parent that we're going to disappear.
	//   The parent needs to report the dying rectangles we've
	//	 collected so it can tell the screen those rects are 
	//	 dirty.
	if (parent){
		for (vector< Rect >::iterator next = dyingRects.begin(); next != dyingRects.end(); next++){
			parent->reportDyingRect (*next);
		}
		//SDL_Rect base = getAbsoluteRect().getRect();
		SDL_Rect base = getAbsoluteVisualRect().toSDL_Rect();
		Rect dying (base);
		parent->reportDyingRect (dying);
		parent->reportDyingChild (this);
	}

    /*if (this->surface){
		// Commented out because we don't want to destroy shared SDL surfaces.
        SDL_FreeSurface (this->surface); this->surface = NULL;
    }*/

    if (this->border){ 
        this->freeBorder();
    }

    if (this->animations){ 
        delete this->animations; this->animations = NULL;
    }

	// Clean up the buffered rects.
	//for (map< int, Rect* >::iterator iter = bufferedRects.begin(); iter != bufferedRects.end(); iter++){
		//delete iter->second; iter->second = NULL;
	//}
	clearBufferedRects();

	// Clean up the buffered viewports.
	//for (map< int, vector< Rect >* >::iterator iter = bufferedViewports.begin(); iter != bufferedViewports.end(); iter++){
	//	delete iter->second; iter->second = NULL;
	//}
	clearBufferedViewports();

	clearBufferedOcclusionports();

	//cout << "Set myself " << this << " to not be live." << endl;
	setLive (false);

    if (this->rect != NULL){ 
        delete this->rect; this->rect = NULL;
    }

	if (camera) delete camera; camera = NULL;

	for (vector<Effect*>::iterator iter = effects.begin(); iter != effects.end(); iter++) delete *iter;

	// NEVER free your surface because surfaces are shared!!!
	//if (surface) SDL_FreeSurface (surface);
}

void Surface::setLive (bool living){
	this->live = living;
}
bool Surface::getLive(){
	return this->live;
}
void Surface::setVisible (bool visible){
	this->visible = visible;
}
bool Surface::getVisible(){
	return this->visible;
}

bool Surface::addPerspectiveTransition (TwoDimensionalTransition* transition){
	if (!camera){ 
		setCamera (new Camera());
		camera->addObserver (this);
	}
	return camera->addTransition (transition);
}
void Surface::clearPerspectiveTransitions(){
	camera->clearTransitions();
}

void Surface::setCamera (Camera* camera){
	if (this->camera) delete this->camera;
	this->camera = camera;
}
Camera* Surface::getCamera(){
	return camera;
}

// This should only be called when a surface is destructing. Do not try to delete the child.
void Surface::reportDyingChild (const Surface* dying){
	vector<Surface*>::iterator it = find (children.begin(), children.end(), dying);
	if (it != children.end()){
		children.erase (it);
	}
}

const Rect* Surface::getScreenRect(){
	if (parent) return parent->getScreenRect();
	return NULL;
}

void Surface::getRectsToDraw (const Rect& intersecting, vector<Rect*>& runningList){
	getParent()->getRectsToDraw (intersecting, runningList);
}

bool Surface::hasVisualEffects(){
	return effects.size() > 0;
}

bool Surface::hasVisualEffect (Effect* effect){
	for (vector< Effect* >::iterator iter = effects.begin(); iter != effects.end(); iter++){
		if (*iter == effect){
			return true;
		}
	}
	return false;
}

void Surface::addVisualEffect (Effect* effect){
	if (! hasVisualEffect (effect)){
		effect->addObserver (this);
		effects.push_back (effect);
	}
}

void Surface::transitionsComplete (Camera* caller){
	for (vector<Interface_SurfaceEventObserver*>::iterator iter = surfaceEventObservers.begin(); iter != surfaceEventObservers.end(); iter++){
		(*iter)->perspectiveTransitionsComplete (this);
	}
}
void Surface::effectComplete (Effect* effect){
	effectsToKill.push_back (effect);
}

void Surface::reportDyingRects (const vector<Rect>& dying){
	//cout << "A rect is dying" << endl;
	for (vector<Rect>::const_iterator iter = dying.begin(); iter != dying.end(); iter++){
		reportDyingRect (*iter);
	}
}

void Surface::reportDyingRect (const Rect& dying){
	dyingRects.push_back (dying);
}

void Surface::setAnimationTimeLength (AnimationType index, unsigned int milliseconds){
	if (animations){
		animations->setTimeLength (index, milliseconds);
	}
}

bool Surface::playAnimationFromStart (const AnimationType& index, const InterruptFlag& interrupter, const bool& continuous){
	if (playAnimation (index, continuous)){
		setDirty (true);
		return resetAnimation (index, interrupter);
	}
	return false;
}

bool Surface::resetAnimation (const AnimationType& index, const InterruptFlag& interrupter){
	if (animations){
		return animations->resetAnimation (index, interrupter);
	}
	return false;
}

bool Surface::playAnimation (AnimationType index, bool continuous){
	//cout << "Surface::playAnimation() entering, surface is " << this << endl;
	if (animations){
		animations->setRepeating (index, continuous);
		setDirty (true);
		return animations->play (index);
	}
	//cout << "Surface::playAnimation() exiting, surface is " << this << endl;
	return false;
}

void Surface::animationComplete (Surface* caller, AnimationType index){
	//cout << "Surface::animationComplete (" << caller << ", " << index << ")" << endl;
	for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			next != animationObservers.end(); next++){
		(*next)->animationComplete (this, index);
	}
}

void Surface::setBorder (Border* border){
	if (this->border) freeBorder();
    this->border = border;
	//cout << "I set my border to " << border << endl;
}
void Surface::removeBorder(){
    this->border = NULL;
}
void Surface::freeBorder(){
    if (this->border){
        delete this->border; 
        this->removeBorder();
    }
}
void Surface::addChildSurface (Surface* child){
    bool found = false;
    vector<Surface*>::iterator iter = this->children.begin();
    if (child){
        for (; iter != this->children.end(); iter++) if ((*iter) == child) found = true;
        if (!found){
            this->children.insert (this->children.end(), child);
            child->setParent (this);
            //printf ("Added child at %p.\n", child);
        }
    }
}
void Surface::cleanUpChildren(){
	vector <Surface*>::iterator next = children.begin();
	bool finished = (next == children.end());
	unsigned int cleanedUp = 0;
	//cout << "Cleaning up " << children.size() << " children." << endl;
	while (!finished){
		next = children.begin();
		finished = (next == children.end());
		bool removedOne = false;
		while (!finished && !removedOne){
			if ( (*next) && ! ((*next)->getLive()) ){
				Surface* child = *next;
				children.erase (next);
				delete (child); 
				//cout << "Killed a child. " << endl;
				cleanedUp++;
				removedOne = true;
			}
			else{
				next++;
				finished = (next == children.end());
			}
		}
	}
	//if (cleanedUp) cout << "Cleaned up " << cleanedUp << " children." << endl;
}
void Surface::showChildren(){
    printf ("My children: ");
    for (unsigned int i = 0; i < this->children.size(); i++){
        printf ("%p, ", *(this->children.begin() + i));
    }
    printf ("\n");
}

void Surface::clearVisualEffects(){
	std::vector<Effect*> effectsCopy = effects;
	for (std::vector<Effect*>::iterator iter = effectsCopy.begin(); iter != effectsCopy.end(); iter++){
		removeVisualEffect (*iter);
	}
}

void Surface::removeVisualEffect (Effect* effect){
	//cout << "Removing effect at " << effect << endl;
	for (vector< Effect* >::iterator iter = effects.begin(); iter != effects.end(); iter++){
		if ((*iter) == effect){
			effects.erase (iter);
			break;
		}
	}
	//cout << "Found effect? " << found << endl;
}

void Surface::removeChildSurface (Surface* child){
    //printf ("Getting ready to erase my kid!\n");
    //this->showChildren();
    vector<Surface*>::iterator iter = this->children.begin();
    bool found = false;
    if (child){
        //printf ("Entering the damn for loop.\n");
        for (iter = this->children.begin(); !found && iter != this->children.end(); iter++){
            //printf ("(*iter) is %p and child is %p.\n", (*iter), child);
            if ((*iter) == child){
                found = true;
                this->children.erase (iter);
                //printf ("I swear I killed him.\n");
            }
        }
    }
    //printf ("Erased my kid at %p.  :'(\n", child);
    //this->showChildren();
    //printf ("Is the little fucker gone yet?!\n");
}
void Surface::freeChildren(){
    vector<Surface*>::iterator iter = this->children.begin();
	while (children.begin() != children.end()){ 
		Surface* deleting = *(children.begin());
		children.erase (children.begin());
		delete deleting;
	}
}

SDL_Surface* Surface::getSurface(){
	if (animations) return animations->getCurrentSurface(); 
	return this->surface;
}

bool Surface::isDirty(){
	/*
	if (animations){
		cout << "Animation is checking if it's dirty." << endl;
	}
	*/
	Rect* oldRect = getBufferedRect (currentBuffer);
	Rect newRect = getAbsoluteVisualRect();
	//if (hasBorder()) newRect.trimTo (border->getViewport());
	if (oldRect){
		if (this->dirty ||
			int (oldRect->getX()) != int (newRect.getX()) || 
			int (oldRect->getY()) != int (newRect.getY())
		){
		//	oldRect->getWidth() != getRect()->getWidth() ||
		//	oldRect->getHeight() != getRect()->getHeight()){
			//if (animations) cout << "Old rect doesn't match new rect. is dirty." << endl;
			return true;
		}
		else{
			//if (animations) cout << "Old rect matches new rect. is not dirty." << endl;
			return false;
		}
	}
	else{
		//if (animations) cout << "Old rect doesn't exist. is dirty." << endl;
		return true;
	}
}

vector< Rect >* Surface::getBufferedImageProcessorViewports (int buffer){
	if (! bufferedImageProcessorViewports [buffer]){
		bufferedImageProcessorViewports [buffer] = new vector< Rect >;
	}
	return bufferedImageProcessorViewports [buffer];
}

vector< Rect >* Surface::getBufferedViewports (int buffer){
	//if (! bufferedViewports [0]) bufferedViewports [0] = new vector< Rect >;
	//bufferedViewports [0]->clear(); bufferedViewports [0]->push_back (getAbsoluteRect());

	//cout << "getBufferedViewports checking buffer " << buffer << endl;
	if (! bufferedViewports [buffer]){
		bufferedViewports [buffer] = new vector< Rect >;
	}
	
	//cout << "The buffer has " << bufferedViewports [buffer]->size() << " viewport(s): " << endl;
	//for (int x = 0; x < bufferedViewports [buffer]->size(); x++){
		//cout << (*bufferedViewports [buffer]) [x].toString() << endl;
	//}
	
	return bufferedViewports [buffer];
}

vector< Rect >* Surface::getBufferedOcclusionports (int buffer){
	//cout << "getBufferedOcclusionports checking buffer " << buffer << endl;
	if (! bufferedOcclusionports [buffer]){
		bufferedOcclusionports [buffer] = new vector< Rect >;
	}
	
	//cout << "The buffer has " << bufferedOcclusionports [buffer]->size() << " occlusionport(s): " << endl;
	//for (int x = 0; x < bufferedOcclusionports [buffer]->size(); x++){
		//cout << (*bufferedViewports [buffer]) [x].toString() << endl;
	//}
	
	return bufferedOcclusionports [buffer];
}

Rect* Surface::getBufferedRect (int buffer){
	if (! this->bufferedRects [buffer]){
		this->bufferedRects [buffer] = new Rect();
		this->bufferedRects [buffer]->copyFrom (* (this->rect));
		Rect absolute = getAbsoluteRect();
		this->bufferedRects [buffer]->setXY (absolute.getX(), absolute.getY());
		//cout << "Setting to dirty at " << __LINE__ << endl;
		this->setDirty (true);
	}
	return this->bufferedRects [buffer];
}

void Surface::forcedAddDirtyRects (const vector<Rect>& newRects){
	if (parent) parent->forcedAddDirtyRects (newRects);
}
void Surface::getDirtyRects (const Rect& perimeter, vector<Rect>& runningList){
	// If this surface is dirty, all its children are dirty as well.
	//   In that case, we don't need to poll children.
	if (this->isDirty()){
		//cout << "A whole surface is dirty." << endl;
		Rect from;
		Rect to;
		from.copyFrom (* getBufferedRect (currentBuffer));
		to.copyFrom (*getRect());
		// Buffered rects have absolute location but this surface's rect 
		//   uses relative values, so we need to convert it.
		to.setX (int (getAbsoluteVisualX()));
		to.setY (int (getAbsoluteVisualY()));
		if (from.getWidth() > 0 && from.getHeight() > 0){
			runningList.push_back (from);
		}
		if (to.getWidth() > 0 && to.getHeight() > 0){
			runningList.push_back (to);
		}
	}
	else{
		for (vector<Surface*>::iterator iter = children.begin(); iter != children.end(); iter++){
			(*iter)->getDirtyRects (perimeter, runningList);
		}
		if (hasBorder()){
			border->getDirtyRects (perimeter, runningList);
		}
	}
	for (vector<Rect>::iterator iter = dyingRects.begin(); iter != dyingRects.end(); iter++){
		//cout << "Dirty because of dying rects" << endl;
		runningList.push_back (*iter);
	}
	dyingRects.clear();

	// Pick up any rects that were found to be dirty in other parts of the execution cycle.
	for (vector< Rect >::iterator iter = nextDirtyRects.begin(); iter != nextDirtyRects.end(); iter++){
		runningList.push_back (*iter);
		//cout << "Dirty in other parts of execution cycle!" << endl; 
	}
	nextDirtyRects.clear();

	// Add dirty rects created by rect changes.
	Rect* oldRect = getBufferedRect (getBuffer());
	Rect myRect = getAbsoluteVisualRect();
	//cout << "Old rect is " << oldRect->toString() << " new rect is " << myRect.toString() << endl;
	//if (hasBorder()) myRect.trimTo (border->getViewport());
	vector< Rect > cutouts = myRect.cutOut (*oldRect);
	for (vector< Rect >::iterator iter = cutouts.begin(); iter != cutouts.end(); iter++){
		//cout << "Dirty created by growing rect!" << endl; 
		runningList.push_back (*iter);
	}
	/*
	if (hasBorder()){
		//cout << "Using border viewport instead of my rect for cutouts" << endl;
		Rect borderViewport = border->getViewport();
		cutouts = oldRect->cutOut (borderViewport);
	}
	*/
	//else{
		cutouts = oldRect->cutOut (myRect);
	//}
	for (vector< Rect >::iterator iter = cutouts.begin(); iter != cutouts.end(); iter++){
		/*
		cout << "Dirty created by shrinking rect!" << endl; 
		cout << "Old rect is " << oldRect->toString() << " new rect is " << myRect.toString() << endl;
		cout << "Resulting dirty rect is " << (*iter).toString();
		*/
		runningList.push_back (*iter);
	}
	Rect absolute = getAbsoluteRect();
	//cout << "Getting dirty rects from effects." << endl;
	/*
	for (vector< Effect* >::iterator currentEffect = effects.begin(); currentEffect != effects.end(); currentEffect++){
		(*currentEffect)->getDirtyRects (absolute, runningList); 
	}
	*/
	/*
	if (rects.size() > 0){
		cout << "Collected " << rects.size() << " dirty rect(s).";
	}
	*/
}

Rect Surface::getAbsoluteVisualRect(){
	//int absX = int (getAbsoluteVisualX()); int absY = int (getAbsoluteVisualY());
	Rect myRect = getVisualRect();
	//return Rect (absX, absY, myRect.getWidth(), myRect.getHeight());
	return Rect (getAbsoluteVisualX(), getAbsoluteVisualY(), myRect.getWidth(), myRect.getHeight());
}
Rect Surface::getAbsoluteRect(){
	/*
	Rect myRect = *getRect();
	if (parent){
		Rect parentRect = parent->getAbsoluteRect();
		return Rect (parentRect.getX() + myRect.getX(), parentRect.getY() + myRect.getY(), myRect.getWidth(), myRect.getHeight());
	}
	return Rect (myRect.getX(), myRect.getY(), 0, 0);
	*/
	double absX = getAbsoluteX(); double absY = getAbsoluteY();
	Rect myRect = *getRect();
	//int width = getRect()->getWidth(); int height = getRect()->getHeight();
	//Rect retRect (absX, absY, width, height);
	return Rect (absX, absY, myRect.getWidth(), myRect.getHeight());
}

Rect Surface::getVisualRect(){
	Rect myRect (getX(), getY(), getRect()->getWidth(), getRect()->getHeight());
	double effectOffsetX = 0;
	double effectOffsetY = 0;
	if (effects.size()){
		for (vector<Effect*>::iterator nextEffect = effects.begin(); nextEffect != effects.end(); nextEffect++){
			if (!(*nextEffect)->getActive()) continue; // Skip inactive effects
			effectOffsetX += (*nextEffect)->getX();
			effectOffsetY += (*nextEffect)->getY();
		}
	}
	myRect.setX (Utils::spaceCoordToScreenCoord (myRect.getX() + effectOffsetX, getZ()));
	myRect.setY (Utils::spaceCoordToScreenCoord (myRect.getY() + effectOffsetY, getZ()));
	return myRect;
}

Rect* Surface::getRect(){
	//cout << "Getting rect." << endl;
	if (animations && animations->getCurrentAnimation() > -1){
		//cout << "Returning animation's rect." << endl;
		SDL_Rect animRect = animations->getCurrentRect();
		rect->setWidth (animRect.w); rect->setHeight (animRect.h);
	}
	return rect;
}

bool Surface::hasBorder(){
	return (border != NULL);
}

// This should be run right after update() so we have a chance to find any dirty rectangles 
//	revealed by changes in the occlusionports.
vector< Rect > Surface::getOcclusionports(){
	vector< Rect > ports = *( getBufferedOcclusionports (getBuffer()) );

	return ports;
}

// This should be run right after update() so we have a chance to find any dirty rectangles 
//	revealed by changes in the viewports.
vector< Rect > Surface::getViewports(){
	//cout << this << " getting viewports." << endl;
	
	vector< Rect > viewports = *( getBufferedViewports (getBuffer()) );

	return viewports;
}
vector< Rect > Surface::getImageProcessorViewports(){
	vector< Rect > viewports = *( getBufferedImageProcessorViewports (getBuffer()) );
	return viewports;
}

void Surface::updateOcclusionports(){
	// We need to find out what occlusion ports disappeared. To do this, we cut the new occlusion ports out of the old 
	//	occlusion ports and what remains are ports that once were occluded but are now visible.
	vector< Rect >* oPorts = getBufferedOcclusionports (getBuffer());
	vector< Rect > dirties;
	for (vector< Rect >::iterator iter = oPorts->begin(); iter != oPorts->end(); iter++){
		for (vector< Rect >::iterator newport = occlusionPorts.begin(); newport != occlusionPorts.end(); newport++){
			vector< Rect > cuts = iter->cutOut (*newport);
			for (vector< Rect >::iterator cut = cuts.begin(); cut != cuts.end(); cut++){
				if (iter->getWidth() > 0 && iter->getHeight() > 0){
					dirties.push_back (*iter);
				}
			}
		}
 	}
	//cout << "Enqueueing at " << __LINE__ << endl;
	enqueueDirtyRects (dirties);
	oPorts->clear();
	// Now we need to make the new occlusion ports the "old" buffered occlusion ports so we can compare again next time.
	for (vector< Rect >::iterator iter = occlusionPorts.begin(); iter != occlusionPorts.end(); iter++){	
		oPorts->push_back (*iter);
	}
	occlusionPorts.clear();
}

void Surface::updateViewports(){
	//cout << this << " updating viewports." << endl;
	vector< Rect > tempList;
	vector< Rect > imageProcessorTempList;
	vector< Rect >* targetList = getBufferedViewports (getBuffer());
	vector< Rect >* imageProcessorTargetList = getBufferedImageProcessorViewports (getBuffer());

	// First, get the viewports from any effects we might have running.
	//Rect absolute = getAbsoluteRect();
	Rect absolute = getAbsoluteVisualRect();
	vector< Rect >::iterator scrapIter;
	vector< Rect >::iterator endIter;
	vector< Rect >::iterator endIter2;
	vector< Rect >::iterator endIter3;
	for (vector< Effect* >::iterator iter = effects.begin(), endIter = effects.end(); iter != endIter; iter++){
		if (!(*iter)->getActive()) continue; // Skip inactive effects.
		vector< Rect > viewports = (*iter)->getViewports (absolute);
		for (scrapIter = viewports.begin(), endIter2 = viewports.end(); scrapIter != endIter2; scrapIter++){
			tempList.push_back (*scrapIter);
			//imageProcessorTempList.push_back (*scrapIter); // I'm going to try out ignoring effect viewports for image processors.
		}
	}
	//cout << "After " << effects.size() << " effects, we have " << tempList.size() << " viewports." << endl;
	// If we didn't get any viewports from our effects, we'll use our rect as a default.
	if (tempList.size() == 0){
		tempList.push_back (absolute);
		// Don't add for image processors because they can ignore our own viewport.
		if (parent){ 
			imageProcessorTempList.push_back (parent->getAbsoluteVisualRect());
			//cout << "using parent's rect for default viewport, which is " << parent->getAbsoluteVisualRect().toString() << endl;
		}
	}
	//cout << "Before parent, we have " << tempList.size() << " viewports." << endl;
	vector< Rect > parentPorts;
	Surface* parent = getParent();
	if (parent){
		parentPorts = parent->getViewports();
	}

	// Next, we have to trim each of our viewports to the viewports of our parent.
	// TODO: This won't work quite as expected- when there are two viewports that don't intersect, the children viewports 
	//	will get squished to nothing because they'll try to trim themselves to each parent viewport! Should probably enforce 
	//	one parent viewport or make an algorithm to intelligently trim to multiple viewports.
	for (vector< Rect >::iterator nextPort = parentPorts.begin(), endIter = parentPorts.end(); nextPort != endIter; nextPort++){
		//cout << "Parent port is " << nextPort->toString() << endl;
		for (scrapIter = tempList.begin(), endIter2 = tempList.end(); scrapIter != endIter2; scrapIter++){
			scrapIter->trimTo (*nextPort);
			//cout << "Trimmed one of my ports to parent's port and got " << scrapIter->toString() << endl;
		}
		for (scrapIter = imageProcessorTempList.begin(), endIter2 = imageProcessorTempList.end(); scrapIter != endIter2; scrapIter++){
			scrapIter->trimTo (*nextPort);
		}
	}

	// Now that we have all the viewports a plain surface should have, we need to trim them to fit within the bordered 
	//	perimeter (that is, if the surface does have a border.)
	Rect bordViewport;
	if (hasBorder()){
		std::vector<Rect> myRects;
		myRects.push_back (absolute);
		//border->buildPerimeter (tempList); // This is an artifact from when borders "stuck" inside their parent's viewport
		border->buildPerimeter (myRects);
		bordViewport = border->getViewport();
		for (scrapIter = tempList.begin(), endIter = tempList.end(); scrapIter != endIter; scrapIter++){
			//cout << "We have a border!" << endl;
			scrapIter->trimTo (bordViewport); 
			// We don't trim image processors to the border because they can bleed outside.
		}
	}	

	//cout << "After parent, we have " << tempList.size() << " viewports." << endl;
	// We need to figure out what rectangles are dirty because the viewports have changed. 
	//	To do that, we'll take each of the viewports we just got, cut out the old ones, 
	//	and what remains will be the newly-revealed rects.
	vector< Rect > dirties;
	vector< Rect > cuts;
	for (vector< Rect >::iterator newRect = tempList.begin(), endIter = tempList.end(); newRect != endIter; newRect++){
		for (scrapIter = targetList->begin(), endIter2 = targetList->end(); scrapIter != endIter2; scrapIter++){
			cuts = newRect->cutOut (*scrapIter);
			for (vector< Rect >::iterator cut = cuts.begin(), endIter3 = cuts.end(); cut != endIter3; cut++){
				dirties.push_back (*cut);
			}
		}
		if (targetList->size() == 0 && newRect->getWidth() > 0 && newRect->getHeight() > 0){
			dirties.push_back (*newRect);
		}
	}
	//cout << "Enqueueing at " << __LINE__ << endl;
	enqueueDirtyRects (dirties);

	vector< Rect > imageProcessorDirties;
	vector< Rect > imageProcessorCuts;
	for (vector< Rect >::iterator newRect = imageProcessorTempList.begin(), endIter = imageProcessorTempList.end(); newRect != endIter; newRect++){
		for (scrapIter = imageProcessorTargetList->begin(), endIter2 = imageProcessorTargetList->end(); scrapIter != endIter2; scrapIter++){
			imageProcessorCuts = newRect->cutOut (*scrapIter);
			for (vector< Rect >::iterator cut = imageProcessorCuts.begin(), endIter3 = imageProcessorCuts.end(); cut != endIter3; cut++){
				imageProcessorDirties.push_back (*cut);
			}
		}
		if (imageProcessorTargetList->size() == 0 && newRect->getWidth() > 0 && newRect->getHeight() > 0){
			imageProcessorDirties.push_back (*newRect);
		}
	}
	//cout << "Enqueueing at " << __LINE__ << endl;
	enqueueDirtyRects (imageProcessorDirties);
	//cout << "Viewport generated " << dirties.size() << " dirty rects." << endl;
	// Now push all the viewports we found into the buffered viewports list.
	targetList->clear();
	for (scrapIter = tempList.begin(), endIter = tempList.end(); scrapIter != endIter; scrapIter++){
		targetList->push_back (*scrapIter);
		//cout << "Adding viewport to bufferedViewports: " << scrapIter->toString() << endl;
		//if (getName() != "") cout << name << " has a viewport of " << scrapIter->toString() << "." << endl;
	}
	imageProcessorTargetList->clear();
	for (scrapIter = imageProcessorTempList.begin(), endIter = imageProcessorTempList.end(); scrapIter != endIter; scrapIter++){
		imageProcessorTargetList->push_back (*scrapIter);
	}	
	//if (getName() != "" && targetList->size() == 0) cout << name << " has no fucking viewports." << endl;
	//cout << "After viewports update, bufferedViewports has " << targetList->size() << " viewports." << endl;
}

void Surface::enqueueDirtyRects (vector< Rect > dirties){
	for (vector< Rect >::iterator next = dirties.begin(); next != dirties.end(); next++){
		//cout << "Enqueueing a dirty rect " << (*next).toString() << endl;
		nextDirtyRects.push_back (*next);
	}
}
void Surface::enqueueDirtyRect (Rect dirty){
	nextDirtyRects.push_back (dirty);
}

void Surface::clippedBlit (const Rect& absoluteSourceRect, SDL_Surface* sourceSurface, SDL_Surface* targetSurface, 
						   const vector<Rect>& viewports, const vector<Rect*>& dirtyRects, const double& offsetX, const double& offsetY){
	//cout << "Surface::clippedBlit() iterating through " << viewports.size() << " viewports and " << dirtyRects.size() << " clipboxes." << endl;
	const Rect& absolute = absoluteSourceRect;
	//cout << "clippedBlit absolute is " << absolute.toString() << endl;
	vector<Rect> myViewports = viewports;
	for (vector< Rect >::iterator nextView = myViewports.begin(); nextView != myViewports.end(); nextView++){
		for (vector< Rect* >::const_iterator nextBox = dirtyRects.begin(); nextBox != dirtyRects.end(); nextBox++){
			Rect clipBox; clipBox.copyFrom (**nextBox);
//cout << "clip box starts as " << clipBox.toString() << endl;
			clipBox.setXY (int (clipBox.getX()), int (clipBox.getY()));
			if (clipBox.getX() < (*nextBox)->getX()){	
				clipBox.setWidth (clipBox.getWidth() + 1); 
				//cout << "offsetting clip x" << endl;
			}
			if (clipBox.getY() < (*nextBox)->getY()){ 
				clipBox.setHeight (clipBox.getHeight() + 1);
				//cout << "offsetting clip y" << endl;
			}
//cout << "clip box becomes as " << clipBox.toString() << endl;
			int correctionOffsetX = (nextView->getX() != floor (nextView->getX()) ? 1 : 0);
			//if (correctionOffsetX == 1) cout << "offsetting view x" << endl;
			//int correctionOffsetX = ((*nextBox)->getX() != floor ((*nextBox)->getX()) ? 1 : 0);
			//int correctionOffsetX = 0;
			int correctionOffsetY = (nextView->getY() != floor (nextView->getY()) ? 1 : 0);
			//if (correctionOffsetY == 1) cout << "offsetting view y" << endl;
			//int correctionOffsetY = ((*nextBox)->getY() != floor ((*nextBox)->getY()) ? 1 : 0);
			//int correctionOffsetY = 0;
			//nextView->setXY (floor (nextView->getX()) + correctionOffsetX, floor (nextView->getY()) + correctionOffsetY);
			//nextView->setWidth (ceil (nextView->getWidth())); nextView->setHeight (ceil (nextView->getHeight()));
			nextView->setXY (int (nextView->getX()), int (nextView->getY()));
			nextView->setWidth (int (nextView->getWidth()) + correctionOffsetX); nextView->setHeight (int (nextView->getHeight()) + correctionOffsetY);
			clipBox.trimTo (*nextView);
//cout << "clip trims to " << clipBox.toString() << endl;
			Rect drawBox = absolute;
			int drawBoxX = int (drawBox.getX());
			int drawBoxY = int (drawBox.getY());
			if (offsetX || offsetY){
				drawBox.setXY (drawBoxX + offsetX, drawBoxY + offsetY);
				drawBoxX += int (offsetX); drawBoxY += int (offsetY);
			}
			Rect cropBox = absolute;
//cout << "cropbox starts as " << cropBox.toString() << endl;
			if (animations && (offsetX || offsetY) && animations->getCurrentSurface() == sourceSurface) 
				cropBox.setXY (cropBox.getX() + offsetX, cropBox.getY() + offsetY);
			cropBox.setXY (int (cropBox.getX()), int (cropBox.getY()));
			cropBox.trimTo (clipBox);
//cout << "cropbox trimmed to " << cropBox.toString() << endl;

			int clipBoxWidth = int (clipBox.getWidth());
			int clipBoxHeight = int (clipBox.getHeight());
			int clipX1 = int (clipBox.getX());
			int clipY1 = int (clipBox.getY());
			int drawX1 = drawBoxX;
			int drawY1 = drawBoxY;
			int drawOffsetX = 0; int drawOffsetY = 0;
			int placeOffsetX = 0; int placeOffsetY = 0;
			if (drawX1 < clipX1) drawOffsetX = clipX1 - drawX1;
			else placeOffsetX = drawX1 - clipX1;
			if (drawY1 < clipY1) drawOffsetY = clipY1 - drawY1;
			else placeOffsetY = drawY1 - clipY1;
			if (animations && animations->getCurrentSurface() == sourceSurface){
				cropBox.setX (drawOffsetX + animations->getCurrentRect().x);
				cropBox.setY (drawOffsetY + animations->getCurrentRect().y);
			}
			else{
				cropBox.setX (drawOffsetX);
				cropBox.setY (drawOffsetY);
			}
//cout << "cropbox becomes " << cropBox.toString() << endl;
			clipBox.setX (clipX1 + placeOffsetX);
			clipBox.setY (clipY1 + placeOffsetY);
			clipBox.setWidth (clipBoxWidth - placeOffsetX);
			clipBox.setHeight (clipBoxHeight - placeOffsetY);

			//cout << "Blitting surface " << sourceSurface << " crop " << cropBox.toString() << " clip " << clipBox.toString() << endl;
			SDL_BlitSurface (sourceSurface, &(cropBox.toSDL_Rect()), targetSurface, &(clipBox.toSDL_Rect()));
		}
	}	
}

bool Surface::compareSurfacePointers (Surface* surface1, Surface* surface2){
	return surface1->getZ() < surface2->getZ();
}

void Surface::postDraw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride, 
						const double& offsetX, const double& offsetY)
{
	if (!getLive() || !getVisible()){
		return;
	}

	if (effects.size() > 0){

		setDefaultColorKey();

    	/////// START EFFECT STACK ///////////
		Rect imageOffset (0, 0, 0, 0);
		if (animations){ 
			imageOffset = animations->getCurrentRect();
		}
		manager->startEffectStack (getSurface(), getAbsoluteVisualRect().toSDL_Rect(), imageOffset.toSDL_Rect());
		// We need to process effects before we actually blit this surface.
		for (vector<Effect*>::iterator iter = effects.begin(); iter != effects.end(); iter++){
			if (!(*iter)->getActive()) continue; // Skip inactive effects.
			(*iter)->processImagePostDraw (manager);
		}
		// TODO: actually handle the dirty rects instead of pushing back the composite rect.
		vector<Rect> dirtyEffectRects;
		dirtyEffectRects.push_back (manager->getCompositeRect());
		forcedAddDirtyRects (dirtyEffectRects);
		//cout << "manager composite rect is " << manager->getCompositeRect().toString() << endl;
	    //////////////////////////////////////

    	/////// END EFFECT STACK /////////
		manager->endEffectStack (getImageProcessorViewports(), offsetX, offsetY);
    	//////////////////////////////////

	}

	list<Surface*> childrenToDraw;
	for (vector<Surface*>::iterator iter = children.begin(); iter != children.end(); iter++){
		childrenToDraw.push_back (*iter);
	}
	childrenToDraw.sort (Surface::compareSurfacePointers);

	list<Surface*>::iterator iter = childrenToDraw.begin();
    for (; iter != childrenToDraw.end(); iter++){
   	    (*iter)->postDraw (manager, screen, dirtyRectsOverride, offsetX, offsetY);
    }
}
void Surface::draw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride, const double& offsetX, const double& offsetY){
	//cout << "drawing object at " << this << endl;
	//if (!surface) cout << "NULL fucking surface!" << endl;

	if (!getLive() || !getVisible()){
		//if (!getLive()) cout << "Surface at " << this << " is not live and can not draw!" << endl;
		//if (!getVisible()) cout << "Surface at " << this << " is not visible and can not draw!" << endl;
		return;
	}
	//Rect absolute = getAbsoluteRect();
	Rect absolute = getAbsoluteVisualRect();

	//cout << "Absolute visual rect is " << absolute.toString() << endl;
	vector<Rect*> originalClipBoxes;
	//getParent()->getRectsToDraw (absolute, originalClipBoxes);
	//targetSurface->getRectsToDraw (absolute, originalClipBoxes);

    this->setDefaultColorKey();

    /////// START EFFECT STACK ///////////
	Rect imageOffset (0, 0, 0, 0);
	if (animations){ 
		imageOffset = animations->getCurrentRect();
	}
	manager->startEffectStack (getSurface(), absolute.toSDL_Rect(), imageOffset.toSDL_Rect());
	// We need to process effects before we actually blit this surface.
	for (vector<Effect*>::iterator iter = effects.begin(); iter != effects.end(); iter++){
		if (!(*iter)->getActive()) continue; // Skip inactive effects.
		(*iter)->processImage (manager);
	}
	// TODO: actually handle the dirty rects instead of pushing back the composite rect.
	vector<Rect> dirtyEffectRects;
	dirtyEffectRects.push_back (manager->getCompositeRect());
	forcedAddDirtyRects (dirtyEffectRects);
	//cout << "manager composite rect is " << manager->getCompositeRect().toString() << endl;
    //////////////////////////////////////


	// If we got an override for dirty rects, use the override instead of asking the screen for dirty rects.
	if (dirtyRectsOverride){
		for (vector<Rect*>::const_iterator iter = dirtyRectsOverride->begin(); iter != dirtyRectsOverride->end(); iter++){
			originalClipBoxes.push_back (*iter);
		}
	}
	else{
		screen->getRectsToDraw (absolute, originalClipBoxes);
	}
	//cout << "Parent gave me " << originalClipBoxes.size() << " rects to draw to." << endl;
	//cout << "Actor " << getType() << endl;

	vector< Rect > viewports = getViewports();
	//cout << "Actor " << getType() << " found " << viewports.size() << " viewports." << endl;
	vector< Rect > cutOutViewports;
	// Occlusion ports always override viewports.
	vector< Rect > occlusionports = getOcclusionports();
	//cout << "Actor " << getType() << " found " << occlusionports.size() << " occlusionports." << endl;
	// For each occlusionport, cut the occlusionport section out of each viewport.
	for (vector< Rect >::iterator nextView = viewports.begin(); nextView != viewports.end(); nextView++){
		// Even if this surface has no actual rect (Text object is an example), we need to push a 
		//	viewport to get it to draw its children. TODO: Fix this silly hack!
		if (occlusionports.size() < 1){
			cutOutViewports.push_back (*nextView);
		}
		else{
			for (vector< Rect >::iterator nextOcc = occlusionports.begin(); nextOcc != occlusionports.end(); nextOcc++){
				vector< Rect > cutouts = nextView->cutOut (*nextOcc);
				// Cutting out a Rect could spawn a bunch of other Rects. These Rects will form the new set of viewports.
				for (vector< Rect >::iterator nextCutOut = cutouts.begin(); nextCutOut != cutouts.end(); nextCutOut++){
					cutOutViewports.push_back (*nextCutOut);
				}
			}
		}
	}
	// I'm going to try allowing image effects to ignore occlusion ports, for now. Otherwise, cut out the viewports here.

	SDL_Surface* sourceSurface = this->surface;	
	if (animations){
		sourceSurface = this->animations->getCurrentSurface();
		/*
		if (drawBox.getX() >= clipBox.getX() && 
				drawBox.getX() < clipBox.getX() + clipBox.getWidth() &&
				drawBox.getY() >= clipBox.getY() && 
				drawBox.getY() < clipBox.getY() + clipBox.getHeight()){ 
			cout << "Drawing animation frame." << endl;
			cout << "Clipbox: " << clipBox.toString() << endl;
			cout << "Surface rect: " << drawBox.toString() << endl;
			cout << "Crop: " << cropBox.toString() << endl;
			cout << "Surface is at " << sourceSurface << endl;
		}
		*/
	}

	//cout << "Made it to cutOutViewports thing." << endl;

	// Drawing actually happens here.
	clippedBlit (absolute, sourceSurface, manager->getTargetSurface(), cutOutViewports, originalClipBoxes, offsetX, offsetY);


    /////// END EFFECT STACK /////////
	manager->endEffectStack (getImageProcessorViewports(), offsetX, offsetY);
    //////////////////////////////////


	list<Surface*> childrenToDraw;
	for (vector<Surface*>::iterator iter = children.begin(); iter != children.end(); iter++){
		childrenToDraw.push_back (*iter);
/*
		unsigned int childIndex = (childrenToDraw.size() - 1) / 2;
		unsigned int windowSize = childIndex;
		while (windowSize){
			windowSize >> 1;
			if (childrenToDraw [childIndex]->getZ() < (*iter)->getZ()){
				childIndex -= windowSize;
			}
			else{
				childIndex += windowSize;
			}
		}
		childrenToDraw.insert (childrenToDraw.begin() + childIndex, *iter);
*/
	}
	
	childrenToDraw.sort (Surface::compareSurfacePointers);

	list<Surface*>::iterator iter = childrenToDraw.begin();
    for (; iter != childrenToDraw.end(); iter++){
   	    (*iter)->draw (manager, screen);
    }
	
    // If this surface has a border, put it on last, using the clipboxes the surface did not mess up.
	//	^ I don't think that's true- I don't think I ever mess up any clipboxes; each box gets copied; i never touch the rects 
	//		on the original list!
	if (hasBorder()){
		//Rect bordViewport = border->getViewport();
		//cout << "Got a bordered viewport back: " << bordViewport.toString();
		//this->border->draw (targetSurface, originalClipBoxes, bordViewport);
		/*
		cout << "Surface drawing border using cutOutViewports. . . " << endl;
		vector<Rect>::iterator myViewports = getViewports().begin();
		for (; myViewports != getViewports().end(); myViewports++){
			cout << myViewports->toString() << endl;
		}
		*/
		this->border->draw (manager, screen);
		//this->border->draw (targetSurface, clipBoxes);
	}
    //printf ("SDL error after blitting: %s .\n", SDL_GetError());
	//cout << "Setting not dirty at " << __LINE__ << endl;
    this->setDirty (false);

	// Buffered rects should ALWAYS have absolute locations because you can't compute 
	//	 absolute location from relative location for a buffered rect; it's possible
	//	 the parent surface has moved since the buffered rect was stored, meaning 
	//	 the absolute location would be off (absolute location is computed from 
	//	 parent's current location).
	Rect* buffered = getBufferedRect (this->currentBuffer);
	if (buffered && this->rect){
		// If this surface has a border, we need to buffer the trimmed portion. That way, next time 
		//	we check if the viewport has grown, when it compares the buffered rect to the rect using 
		//	a border, there's no difference.
		/*
		if (hasBorder()){
			toBuffer.trimTo (border->getViewport());
		}
		*/
		buffered->copyFrom (absolute);
		//buffered->setX (this->getAbsoluteX());
		//buffered->setX (int (this->getAbsoluteVisualX()));
		//buffered->setY (this->getAbsoluteY());
		//buffered->setY (int (this->getAbsoluteVisualY()));
	}
	//if (name != "") cout << name << " is done drawing!" << endl;
	//cout << "done drawing object at " << this << endl;
}

bool Surface::getUseColorKey(){
	return useColorKey;
}
void Surface::setUseColorKey (bool use){
	useColorKey = use;
}

Uint16 Surface::deriveColorKey(){
	if (! getUseColorKey()){
		return 0;
	}	
	SDL_Surface* surface;
	if (animations){
		surface = animations->getCurrentSurface();
	}
	else{
		surface = this->surface;
	}
	Uint16 retKey = 0;
    if (surface){
		vector<int> source = getColorKeySourcePixelCoords();
		int x = source [0]; int y = source [1];
		retKey = Utils::getPixel (surface, x, y);
	}
	return retKey;
}

void Surface::setDefaultColorKey(){
    //printf ("Setting default key.\n");
	SDL_Surface* surface;
	if (animations){
		surface = animations->getCurrentSurface();
	}
	else{
		surface = this->surface;
	}
    if (surface){
		if (! getUseColorKey()){
    	    SDL_SetColorKey (surface, 0, 0);
		}
		else{
			//cout << "This surface is " << this->surface << endl;
			//cout << "First pixel is " << this->surface->pixels << endl;
			//fflush (stdout);

			// TODO: This is inefficient but necessary right now. Fix if time permits (i'm serial, guys)
			colorKey = deriveColorKey();
        	SDL_SetColorKey (surface, SDL_SRCCOLORKEY, colorKey);
		}
	}
}
ResourcePool* Surface::getResourcePool(){
	if (!resources && parent) resources = parent->getResourcePool();
	//cout << "Parent is NULL. Returning default ResourcePool!" << endl;
	// Not pretty, but it's like a cache now, I guess.  :P  
	return resources;
}
void Surface::loadBitmap (string bmpFile, bool getColorKey){
	setUseColorKey (getColorKey);
    //printf ("Surface::loadBitmap() %s.\n", bmpFile.c_str());
	// Commented out because we don't want to dispose of shared surfaces.
	//if (this->surface){
	//	SDL_FreeSurface (this->surface);
	//	this->surface = NULL;
	//}
	//cout << "Setting to dirty at " << __LINE__ << endl;
    this->setDirty (true);
    if (bmpFile != ""){
		ResourcePool* resources = getResourcePool();
		if (resources){
			this->surface = resources->loadBitmap (bmpFile);
		}
		else{
			cout << "Got a NULL ResourcePool!" << endl;
			return;
		}
    }
    if (this->surface){
		//cout << "Created surface at " << this->surface << endl;
        this->rect->setWidth (this->surface->w);
        this->rect->setHeight (this->surface->h);
		colorKey = deriveColorKey();
    }
	//cout << "Surface::loadBitmap() done." << endl;
}

bool Surface::collidesWith (Surface* target, double offsetX, double offsetY, double zTolerance){
	if (!target || ! target->getLive()){
		return false;
	}
	if (zTolerance >= 0 && fabs (target->getZ() - getZ()) > zTolerance) return false;
	Rect* myRect = new Rect();
	myRect->setX (this->getAbsoluteVisualX() + offsetX);
	//cout << "abs visual x: " << getAbsoluteVisualX() << endl;
	//cout << "x: " << getX() << endl;
	myRect->setY (this->getAbsoluteVisualY() + offsetY);
	myRect->setWidth (this->getRect()->getWidth());
	myRect->setHeight (this->getRect()->getHeight());
	//cout << "collider source: " << myRect->toString() << endl;
	Rect* tRect = new Rect();
	tRect->setX (target->getAbsoluteVisualX());
	tRect->setY (target->getAbsoluteVisualY());
	tRect->setWidth (target->getRect()->getWidth());
	tRect->setHeight (target->getRect()->getHeight());
	//cout << "collider target: " << tRect->toString() << endl;
	bool collides = false;
	if (! 
		(
			myRect->getX() + myRect->getWidth() <= tRect->getX() ||
			myRect->getX() >= tRect->getX() + tRect->getWidth() ||
			myRect->getY() + myRect->getHeight() <= tRect->getY() || 
			myRect->getY() >= tRect->getY() + tRect->getWidth()
		)
	){
        // Get the rectanglular area shared by the two surfaces.
	int myTopBorder = int (myRect->getY());
	int myBottomBorder = int (myRect->getY() + myRect->getHeight());
	int tTopBorder = int (tRect->getY());
	int tBottomBorder = int (tRect->getY() + tRect->getHeight());
	int yStart = max (myTopBorder, tTopBorder);
	int yEnd = min (myBottomBorder, tBottomBorder);
	int myLeftBorder = int (myRect->getX());
	int myRightBorder = int (myRect->getX() + myRect->getWidth());
	int tLeftBorder = int (tRect->getX());
	int tRightBorder = int (tRect->getX() + tRect->getWidth());
	int xStart = max (myLeftBorder, tLeftBorder);
	int xEnd = min (myRightBorder, tRightBorder);
	// Now adjust the x and y coordinates so they're relative to 
	//   this surface's location.
	yStart -= int (myRect->getY());
	yEnd -= int (myRect->getY());
	xStart -= int (myRect->getX());
	xEnd -= int (myRect->getX());
	int tOffsetY = int (myRect->getY() - tRect->getY());
	int tOffsetX = int (myRect->getX() - tRect->getX());
	// Get ready for per-pixel collision checks.
	if (SDL_MUSTLOCK (this->surface)){
		SDL_LockSurface (this->surface);
	}
	if (SDL_MUSTLOCK (target->getSurface())){
		SDL_LockSurface (target->getSurface());
	}
	// A collision occurs if a pair of pixels are not the 
	//   respective keycolors of this and the target surface.
	Uint32 myPixel = 0;
	Uint32 tPixel = 0;
	Uint32 myKey = this->surface->format->colorkey;
	Uint32 tKey = target->getSurface()->format->colorkey;
	//printf ("Evaluating collision. . . xStart=%d, yStart=%d, xEnd=%d, yEnd=%d\n", xStart, yStart, xEnd, yEnd);
	for (int y = yStart; !collides && y < yEnd; y++){
		for (int x = xStart; !collides && x < xEnd; x++){
			myPixel = Utils::getPixel (getSurface(), x, y);
			tPixel = Utils::getPixel (target->getSurface(), x + tOffsetX, y + tOffsetY);
			if (myPixel != myKey && tPixel != tKey){
				collides = true;
			}
				
			/*
			if (collides && false){
				printf ("Collision!\n");
				printf ("yStart=%d, yEnd=%d, xStart=%d, xEnd=%d, tOffsetY=%d, tOffsetX=%d, offsetY=%d, offsetX=%d\n", 
					yStart, yEnd, xStart, xEnd, tOffsetY, tOffsetX, offsetY, offsetX);
				printf ("myX=%d, myY=%d, tX=%d, tY=%d, myHeight=%d, myWidth=%d, tHeight=%d, tWidth=%d\n",
					myRect->getX(), myRect->getY(), tRect->getX(), tRect->getY(), 
					myRect->getHeight(), myRect->getWidth(), tRect->getHeight(), tRect->getWidth());
			}
			*/
		}
	}
	// Unlock surfaces following per-pixel collision checks.
	if (SDL_MUSTLOCK (this->surface)){
		SDL_UnlockSurface (this->surface);
	}
	if (SDL_MUSTLOCK (target->getSurface())){
		SDL_UnlockSurface (target->getSurface());
	} 
	}

	delete myRect; myRect = NULL;
	delete tRect; tRect = NULL;

	return collides;
}

void Surface::cleanUpEffects(){
	for (vector< Effect* >::iterator iter = effectsToKill.begin(); iter != effectsToKill.end(); iter++){
		//cout << "Deleting effect at " << *iter << endl;
		removeVisualEffect (*iter);
		delete (*iter);
	}
	effectsToKill.clear();
}

bool Surface::update (unsigned int timePassed){
	//cout << "Actor " << getType() << " is updating." << endl;
	//if (getName() != "") cout << name << " is updating!" << endl;
	if (camera) camera->update (timePassed);

	cleanUpChildren();
	cleanUpEffects();
	if (!getLive()){
		return false;
	}
	if (animations){
		//cout << "Updating animations." << endl;
		if (animations->update (timePassed)){
			//cout << "Setting to dirty at " << __LINE__ << endl;
			setDirty (true);
		}
		//cout << "Done updating animations." << endl;
	}
	if (effects.size()){
		//cout << "Updating effects." << endl;
		//cout << "I'm updating " << effects.size() << " effects." << endl;
		for (vector< Effect* >::iterator iter = effects.begin(); iter != effects.end(); iter++){
			if (!(*iter)->getActive()) continue; // Skip inactive effects.
			(*iter)->update (timePassed);
		}
		//cout << "Done updating effects." << endl;
	}
	//cout << "Calling update viewports at " << __LINE__ << endl;
	updateViewports();
	updateOcclusionports();
	for (unsigned int x = 0; x < children.size(); x++){
    //for (vector< Surface* >::iterator iter = this->children.begin(); iter != this->children.end(); iter++){
		//cout << "Surface at " << this << " updating child at " << *iter << endl;
        //(*iter)->update (timePassed);
		children [x]->update (timePassed);
    }
	if (hasBorder()){
		border->update (timePassed);
	}
	return this->isDirty();
}

double Surface::getPerspectiveX() const{
	if (camera) return camera->getX();
	return 0;
}
double Surface::getPerspectiveY() const{
	if (camera) return camera->getY();
	return 0;
}
double Surface::getFinalPerspectiveX() const{
	if (camera) return camera->getFinalX();
	return 0;
}
double Surface::getFinalPerspectiveY() const{
	if (camera) return camera->getFinalY();
	return 0;
}

double Surface::getAbsoluteVisualX(){
	if (parent){
		return getVisualRect().getX() + getParent()->getAbsoluteVisualX() + Utils::spaceCoordToScreenCoord (-parent->getPerspectiveX(), getZ());
	}
	else{
		return getVisualRect().getX();
	}		
}
double Surface::getAbsoluteVisualY(){
	if (parent){
		return getVisualRect().getY() + getParent()->getAbsoluteVisualY() + Utils::spaceCoordToScreenCoord (-parent->getPerspectiveY(), getZ());
	}
	else{
		return getVisualRect().getY();
	}		
}

double Surface::getAbsoluteX(){
    if (parent){
        return getX() + getParent()->getAbsoluteX();
    }
	else{
        return getX();
    }
}
double Surface::getAbsoluteY(){
    if (parent){
        return getY() + getParent()->getAbsoluteY();
    }
 	else{
        return getY();
    }
}
double Surface::getAbsoluteZ(){
    if (parent){
        return getZ() + getParent()->getAbsoluteZ();
    }
 	else{
        return getZ();
    }
}
bool Surface::hasSurfaceEventObserver (Interface_SurfaceEventObserver* target){
	vector<Interface_SurfaceEventObserver*>::iterator it = find (surfaceEventObservers.begin(), surfaceEventObservers.end(), target);
	if (it == surfaceEventObservers.end()) return false;
	return true;
}
void Surface::addSurfaceEventObserver (Interface_SurfaceEventObserver* notify){
	if (! hasSurfaceEventObserver (notify)) surfaceEventObservers.push_back (notify);
}
void Surface::removeSurfaceEventObserver (Interface_SurfaceEventObserver* notify){
	vector<Interface_SurfaceEventObserver*>::iterator it = find (surfaceEventObservers.begin(), surfaceEventObservers.end(), notify);
	if (it == surfaceEventObservers.end()) return;
	surfaceEventObservers.erase (it);
}
bool Surface::hasAnimationObserver (Interface_AnimationObserver* target){
	for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			next != animationObservers.end(); next++){
		if (*next == target){
			return true;
		}
	}
	return false;
}
void Surface::addAnimationObserver (Interface_AnimationObserver* notify){
	if (! hasAnimationObserver (notify)){
		animationObservers.push_back (notify);
	}
}
void Surface::removeAnimationObserver (Interface_AnimationObserver* notify){
	if (hasAnimationObserver (notify)){
		bool found = false;
		for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
				!found && next != animationObservers.end(); next++){
			if (*next == notify){
				found = true;
				animationObservers.erase (next);
			}
		}
	}
}
void Surface::loadAnimations (string animsSpecFile){
	if (this->animations){
		delete this->animations; this->animations = NULL;
	}
    if (! this->animations){
        //this->animations = new AnimationSet (getResourcePool());
		this->animations = getResourcePool()->loadAnimationSet (animsSpecFile);
		//cout << "Surface::loadAnimations got animation at " << animations << endl;
		getRect()->setWidth (animations->getCurrentRect().w);
		getRect()->setHeight (animations->getCurrentRect().h);
		this->animations->addAnimationObserver (this);
		//cout << "Surface::loadAnimations (" << animsSpecFile << ") set myself up as an observer? " << animations->hasAnimationObserver (this) << endl;
    }
	//cout << "Setting to dirty at " << __LINE__ << endl;
	setDirty (true);
}
