/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SURFACE_H
#define SURFACE_H

#include <string>
#include <vector>
#include <map>
#include <SDL/SDL.h>
#include "rect.h"
#include "interface_animationObserver.h"
#include "interface_effectObserver.h"
#include "interface_drawable.h"
#include "interface_surfaceEventObserver.h"
#include "interface_cameraEventObserver.h"
#include "utils.h"

using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/

class Effect;
class AnimationSet;
class Border;
class ResourcePool;
class Camera;
class TwoDimensionalTransition;
class Surface : public Interface_AnimationObserver, public Interface_EffectObserver, public Interface_Drawable, public Interface_SurfaceEventObserver, 
				public Interface_CameraEventObserver{
    public:
        Surface (Surface* parent = NULL, string bmpFile = string (""), bool getColorKey = true);
		Surface (Surface* parent, SDL_Surface* image);
        virtual ~Surface();
		static bool compareSurfacePointers (Surface* surface1, Surface* surface2);

		virtual const Rect* getScreenRect();
		virtual void effectComplete (Effect* effect);
		virtual void transitionsComplete (Camera* caller);
		virtual void removeVisualEffect (Effect* effect);
		virtual void clearVisualEffects();
		virtual void addVisualEffect (Effect* effect);
		virtual bool hasVisualEffect (Effect* effect);
		virtual bool hasVisualEffects();
		void setColorKey (Uint16);
        virtual bool collidesWith (Surface*, double = 0, double = 0, double = -1);
        virtual void postDraw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride = NULL, 
						   const double& offsetX = 0, const double& offsetY = 0);
        virtual void draw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride = NULL, 
						   const double& offsetX = 0, const double& offsetY = 0);
        virtual bool update (unsigned int timePassed); // Returns true if this surface becomes dirty. False otherwise.
		virtual void getDirtyRects (const Rect& perimeter, vector<Rect>& runningList); // These rects ALWAYS need to use absolute locations.
		virtual void forcedAddDirtyRects (const vector<Rect>& newRects); // These rects ALWAYS need to use absolute locations. For use during draw().
		virtual void getRectsToDraw (const Rect&, vector<Rect*>&);
		virtual double getX(){ return this->x; }
		virtual double getY(){ return this->y; }
		virtual double getZ(){ return this->z; }
		virtual double getPerspectiveX() const;
		virtual double getPerspectiveY() const;
		virtual double getFinalPerspectiveX() const;
		virtual double getFinalPerspectiveY() const;
        void loadBitmap (string bmpFile, bool getColorKey = true);
        void addChildSurface (Surface* child);
        void removeChildSurface (Surface* child);
        void setBorder (Border* border);
        void removeBorder();
        void freeBorder();
        void setDefaultColorKey();
		void switchBuffer (int buffer){ this->currentBuffer = buffer; }
		int getBuffer(){ return this->currentBuffer; }
        virtual void setX (double x){ this->x = x; this->rect->setX (x); }
        virtual void setY (double y){ this->y = y; this->rect->setY (y); }
		virtual void setZ (double z){ this->z = z; }
		virtual void setXY (double x, double y){ setX (x); setY (y); }
		virtual void setScreenX (double x){ setX (getX() + Utils::screenCoordToSpaceCoord (x - getAbsoluteVisualX(), getZ())); }
		virtual void setScreenY (double y){ setY (getY() + Utils::screenCoordToSpaceCoord (y - getAbsoluteVisualY(), getZ())); }
		virtual void setScreenXY (double x, double y){ setScreenX (x); setScreenY (y); }
		void setAnimationTimeLength (AnimationType index, unsigned int milliseconds);
		bool playAnimationFromStart (const AnimationType& index, const InterruptFlag& interrupter = INTERRUPT_SELF, const bool& continuous = false);
		bool resetAnimation (const AnimationType& index, const InterruptFlag& interrupter = INTERRUPT_SELF);
		bool playAnimation (AnimationType index, bool continuous = false);
        SDL_Surface* getSurface();
		void setSurface (SDL_Surface* surface){ this->surface = surface; }
		Rect getVisualRect();
        Rect* getRect();
        vector< Surface* > getChildren() const{ return children; }
        bool isDirty();
        void setParent (Surface* parent){ this->parent = parent; }
        Surface* getParent(){ return this->parent; }
        double getAbsoluteX();
        double getAbsoluteY();
		double getAbsoluteZ();
		double getAbsoluteVisualX();
		double getAbsoluteVisualY();
		Rect getAbsoluteRect();
		Rect getAbsoluteVisualRect();
        bool setDirty (bool isDirty){ this->dirty = isDirty; return true; }
        virtual void animationComplete (Surface* caller, AnimationType animation);
        void loadAnimations (string animsSpecFile);
		bool hasAnimationObserver (Interface_AnimationObserver*);
		void addAnimationObserver (Interface_AnimationObserver*);
		void removeAnimationObserver (Interface_AnimationObserver*);
		bool hasSurfaceEventObserver (Interface_SurfaceEventObserver*);
		void addSurfaceEventObserver (Interface_SurfaceEventObserver*);
		void removeSurfaceEventObserver (Interface_SurfaceEventObserver*);
		void reportDyingRect (const Rect& dying);
		void reportDyingRects (const vector<Rect>& dying);
		void reportDyingChild (const Surface* dying);
		void setVisible (bool visible = true);
		bool getVisible();
		void setLive (bool living);
		bool getLive();
		virtual vector< Rect > getViewports();
		virtual vector< Rect > getImageProcessorViewports();
		void addOcclusionport (Rect port);
		vector< Rect > getOcclusionports();
		void resetOcclusionports(){ occlusionPorts.clear(); }
		bool hasBorder();
		void setResourcePool (ResourcePool* resources){ this->resources = resources; } 
		virtual ResourcePool* getResourcePool();

		virtual vector<int> getColorKeySourcePixelCoords(){  return colorKeySourcePixelCoords; }
		virtual void setColorKeySourcePixelCoords (int x, int y){ 
			colorKeySourcePixelCoords.clear(); colorKeySourcePixelCoords.push_back (x); colorKeySourcePixelCoords.push_back (y);
		}
		void setName (string newName){ name = newName; }
		string getName(){ return name; }

		virtual void surfaceDestroying (Surface* destroying){}
		virtual void perspectiveTransitionsComplete (Surface* perspectivesOwner){}

		int debugLevel;

		bool addPerspectiveTransition (TwoDimensionalTransition* transition);
		void clearPerspectiveTransitions();

		AnimationSet* getAnimations(){ return animations; }

    protected:
		void clippedBlit (const Rect& absoluteSourceRect, SDL_Surface* sourceSurface, SDL_Surface* targetSurface, 
						  const vector<Rect>& viewports, const vector<Rect*>& dirtyRects, const double& offsetX = 0, const double& offsetY = 0);

		void cleanUpChildren();
		bool getUseColorKey();
		void setUseColorKey (bool use = true);
		Uint16 deriveColorKey();
		void clearBufferedRects(){
			for (int x = 0; x < DISPLAY_BUFFERS; x++){ 
				if (bufferedRects [x]){ 
					delete bufferedRects [x]; bufferedRects [x] = NULL;
				}
			}
		}
		void clearBufferedViewports(){
			for (int x = 0; x < DISPLAY_BUFFERS; x++){ 
				if (bufferedViewports [x]){ 
					delete bufferedViewports [x]; bufferedViewports [x] = NULL;
				}
				if (bufferedImageProcessorViewports [x]){
					delete bufferedImageProcessorViewports [x]; bufferedImageProcessorViewports [x] = NULL;
				}
			}
		}
		void clearBufferedOcclusionports(){
			for (int x = 0; x < DISPLAY_BUFFERS; x++){ 
				if (bufferedOcclusionports [x]){ 
					delete bufferedOcclusionports [x]; bufferedOcclusionports [x] = NULL;
				}
			}
		}
		void cleanUpEffects();
        void freeChildren();
        void showChildren();
		void updateViewports();
		void updateOcclusionports();
		vector< Rect >* getBufferedOcclusionports (int buffer);
		vector< Rect >* getBufferedViewports (int buffer);
		vector< Rect >* getBufferedImageProcessorViewports (int buffer);
		Rect* getBufferedRect (int buffer);

		void setCamera (Camera* camera);
		Camera* getCamera();

	private:
		bool visible;
		Uint16 colorKey;
        SDL_Surface* surface;
        Border* border;
        Rect* rect;
		//map< int, Rect* > bufferedRects;
		Rect* bufferedRects [DISPLAY_BUFFERS];
		unsigned int currentBuffer;
		vector< Rect > occlusionPorts;
        vector< Surface* > children;
		vector< Effect* > effects;
		vector< Effect* > effectsToKill;
        bool dirty;
        Surface* parent;
        AnimationSet* animations;
		//map< int, vector< Rect >* > bufferedViewports;
		vector<Rect>* bufferedViewports [DISPLAY_BUFFERS];
		vector<Rect>* bufferedImageProcessorViewports [DISPLAY_BUFFERS];
		//map< int, vector< Rect >* > bufferedOcclusionports;
		vector<Rect>* bufferedOcclusionports [DISPLAY_BUFFERS];
		vector< Interface_AnimationObserver* > animationObservers;
		vector< Interface_SurfaceEventObserver* > surfaceEventObservers;
		vector< Rect > dyingRects;
		void enqueueDirtyRects (vector< Rect > dirties);
		void enqueueDirtyRect (Rect dirty);
		vector< Rect > nextDirtyRects;
		bool useColorKey;
		bool live;
		ResourcePool* resources;
		string name;

		double x;
		double y;
		double z;

		vector<int> colorKeySourcePixelCoords;

		Camera* camera;
};

#endif
