#include "../quadPathFinder.h"
#include "../testUtils.h"
#include "../segmentContainer.h"
#include "../geometry.h"
#include <vector>
#include <list>
#include <iostream>
#include <sstream>
#include <string>

using namespace std;


class PathUnitPropertyFilter{
	public:
		PathUnitPropertyFilter() 
			: x_ (true), y_ (true), distanceTravelled_ (true), distanceLeft_ (true), parent_ (true) { }
		PathUnitPropertyFilter (const bool x, const bool y, const bool distanceTravelled, const bool distanceLeft, const bool parent) 
			: x_ (x), y_ (y), distanceTravelled_ (distanceTravelled), distanceLeft_ (distanceLeft), parent_ (parent) { }
		bool useX() const{ return x_; }
		bool useY() const{ return y_; }
		bool useDistanceTravelled() const{ return distanceTravelled_; }
		bool useDistanceLeft() const{ return distanceLeft_; }
		bool useParent() const{ return parent_; }
	private:
		bool x_; bool y_; bool distanceTravelled_; bool distanceLeft_; bool parent_;
};

string IntRectToString (const IntRect& area){
	ostringstream out;
	out << "(" << area.getX() << "," << area.getY() << ":" << area.getWidth() << "," << area.getHeight() << ")";
	return out.str(); 
}
string PathUnitToString (const PathUnit& unit, const PathUnitPropertyFilter& filter){
	ostringstream out;
	out << "(";
	if (filter.useX()) out << unit.x() << ",";
	if (filter.useY()) out << unit.y(); 
	out << ":";
	if (filter.useDistanceTravelled()) out << unit.distanceTravelled() << ",";
	if (filter.useDistanceLeft()) out << unit.distanceLeft() << ",";
	if (filter.useParent()){
		unsigned int parentValue = (unsigned int) NULL;
		if (unit.parent()) parentValue = (unsigned int) unit.parent();
		out << parentValue;
	}
	out << ")";
	return out.str();
}

bool compareIntRect (const IntRect& one, const IntRect& two){
	return IntRectToString (one) < IntRectToString (two);
}
bool comparePoint (const Point& one, const Point& two){
	return one.toString() < two.toString();
}
bool compareLine (const Line& one, const Line& two){
	return one.toString() < two.toString();
}
bool comparePathUnit (const PathUnit* one, const PathUnit* two){
	if (one->y() < two->y()) return true;
	if (one->y() > two->y()) return false;
	return one->x() < two->x();
}

string IntRectListToString (const list<IntRect>& vec, const bool sort = false){
	list<IntRect> sorted;
	for (list<IntRect>::const_iterator iter = vec.begin(); iter != vec.end(); iter++){
		sorted.push_back (*iter);
	}
	if (sort) sorted.sort (compareIntRect);
	string out;
	for (list<IntRect>::const_iterator iter = sorted.begin(); iter != sorted.end(); iter++){
		out += "[" + IntRectToString (*iter) + "] ";
	}
	return out;
}
string IntRectVectorToString (const vector<IntRect>& vec, const bool sort = false){
	list<IntRect> sorted;
	for (vector<IntRect>::const_iterator iter = vec.begin(); iter != vec.end(); iter++){
		sorted.push_back (*iter);
	}
	if (sort) sorted.sort (compareIntRect);
	string out;
	for (list<IntRect>::const_iterator iter = sorted.begin(); iter != sorted.end(); iter++){
		out += "[" + IntRectToString (*iter) + "] ";
	}
	return out;
}
string PointListToString (const list<Point>& vec){
	list<Point> sorted;
	for (list<Point>::const_iterator iter = vec.begin(); iter != vec.end(); iter++){
		sorted.push_back (*iter);
	}
	//sorted.sort (comparePoint);
	string out;
	for (list<Point>::const_iterator iter = sorted.begin(); iter != sorted.end(); iter++){
		out += "[" + iter->toString() + "] ";
	}
	return out;
}
string PathUnitListToString (const list<PathUnit*>& vec, const PathUnitPropertyFilter& filter, const bool sort = false){
	list<PathUnit*> sorted;
	for (list<PathUnit*>::const_iterator iter = vec.begin(); iter != vec.end(); iter++){
		sorted.push_back (*iter);
	}
	if (sort) sorted.sort (comparePathUnit);
	string out;
	for (list<PathUnit*>::const_iterator iter = sorted.begin(); iter != sorted.end(); iter++){
		out += "[" + PathUnitToString (**iter, filter) + "] ";
	}
	return out;
}
string LineListToString (const list<Line>& vec){
	list<Line> sorted;
	for (list<Line>::const_iterator iter = vec.begin(); iter != vec.end(); iter++){
		sorted.push_back (*iter);
	}
	sorted.sort (compareLine);
	string out;
	for (list<Line>::const_iterator iter = sorted.begin(); iter != sorted.end(); iter++){
		out += iter->toString() + " ";
	}
	return out;
}

class QuadPathFinderTester{
	public:
		QuadPathFinderTester(){
			runTests();
		}
		~QuadPathFinderTester(){ }

		void runTests() const{
			runCachingTests();
			runGetClearedEdgeRectTests();
			runGetInnerPerimeterRectsTests();
			runIntRectTests();
			runQuadrisectTests();
			runSectorizeTests();
			runGetWallsForBoundsTests();
			runGetAdjacentPathUnitsTests();
			runPointPassableTests();
			runRasterizeLineTests();
			runFindPathTests();
		}
		void runCachingTests() const{
			{
				list<IntRect> target;
				target.push_back (IntRect (10, 10, 1, 1));
				target.push_back (IntRect (11, 10, 1, 1));
				target.push_back (IntRect (11, 11, 1, 1));
				list<IntRect> got;
				CachedPath cached (target);
				cout << check ("Cached path test 1.1", true, cached.satisfies (Point (10, 10), Point (11, 11))) << endl;
				cached.getPath (got, Point (10, 10), Point (11, 11));
				cout << check ("Cached path test 1.2", IntRectListToString (target), IntRectListToString (got)) << endl;
				cout << check ("Cached path test 1.3", true, cached.hasPoint (Point (10, 10))) << endl;
				cout << check ("Cached path test 1.4", false, cached.hasPoint (Point (12, 12))) << endl;
				cout << check ("Cached path test 1.5", false, cached.satisfies (Point (10, 10), Point (12, 12))) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				quad.findPath (expected, Point (10, 10), Point (11, 11));
				cout << check ("Cached path test 2.1", 1, quad.cachedPaths_.size()) << endl;
				got.push_back (*(expected.begin()));
				expected.erase (expected.begin());
				quad.findPath (got, Point (expected.begin()->getX(), expected.begin()->getY()), Point (11, 11));
				cout << check ("Cached path test 2.2", IntRectListToString (expected), IntRectListToString (got)) << endl;
			}
		}
		void runGetInnerPerimeterRectsTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect sectorBounds (0, 0, 0, 0);
				quad.getInnerPerimeterRects_ (got, sectorBounds, 1, 1);
				cout << check ("Get inner perimeter rects test 1", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				expected.push_back (IntRect (0, 0, 1, 1));
				IntRect sectorBounds (0, 0, 1, 1);
				quad.getInnerPerimeterRects_ (got, sectorBounds, 1, 1);
				cout << check ("Get inner perimeter rects test 2", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (1, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (1, 1, 1, 1));
				IntRect sectorBounds (0, 0, 2, 2);
				quad.getInnerPerimeterRects_ (got, sectorBounds, 1, 1);
				cout << check ("Get inner perimeter rects test 3", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (1, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (1, 2, 1, 1));
				expected.push_back (IntRect (2, 2, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (2, 1, 1, 1));
				IntRect sectorBounds (0, 0, 3, 3);
				quad.getInnerPerimeterRects_ (got, sectorBounds, 1, 1);
				cout << check ("Get inner perimeter rects test 4", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
		}
		void runGetClearedEdgeRectTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 3, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (1, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				cout << check ("Get cleared edge rects test 1", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 3);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				cout << check ("Get cleared edge rects test 2", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 4, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				expected.push_back (IntRect (3, 0, 1, 1));
				cout << check ("Get cleared edge rects test 3", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 5, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				expected.push_back (IntRect (4, 0, 1, 1));
				cout << check ("Get cleared edge rects test 4", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 4);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 3, 1, 1));
				cout << check ("Get cleared edge rects test 5", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 5);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 4, 1, 1));
				cout << check ("Get cleared edge rects test 6", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (0.5, -0.5), Point (0.5, -0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 4, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (1, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				expected.push_back (IntRect (3, 0, 1, 1));
				cout << check ("Get cleared edge rects test 7", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (0.5, -0.5), Point (0.5, -0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 5, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (1, 0, 1, 1));
				expected.push_back (IntRect (2, 0, 1, 1));
				expected.push_back (IntRect (3, 0, 1, 1));
				expected.push_back (IntRect (4, 0, 1, 1));
				cout << check ("Get cleared edge rects test 8", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (1.5, 0.5), Point (1.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 4);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 3, 1, 1));
				cout << check ("Get cleared edge rects test 9", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (1.5, 0.5), Point (1.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 5);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 1, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 3, 1, 1));
				expected.push_back (IntRect (0, 4, 1, 1));
				cout << check ("Get cleared edge rects test 10", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 2, 8, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 1);
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (4, 2, 1, 1));
				expected.push_back (IntRect (7, 2, 1, 1));
				cout << check ("Get cleared edge rects test 11", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			// Begin character width/height checks
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 2, 8, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_up, 1, 2);
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (1, 2, 1, 1));
				expected.push_back (IntRect (2, 2, 1, 1));
				expected.push_back (IntRect (3, 2, 1, 1));
				expected.push_back (IntRect (4, 2, 1, 1));
				expected.push_back (IntRect (6, 2, 1, 1));
				expected.push_back (IntRect (7, 2, 1, 1));
				cout << check ("Get cleared edge rects test 12", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (1.5, 0.5), Point (1.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 8);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_right, 2, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 3, 1, 1));
				expected.push_back (IntRect (0, 4, 1, 1));
				expected.push_back (IntRect (0, 6, 1, 1));
				expected.push_back (IntRect (0, 7, 1, 1));
				cout << check ("Get cleared edge rects test 13", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (0.5, 2.5), Point (0.5, 2.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 2, 8, 1);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_down, 1, 2);
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (1, 2, 1, 1));
				expected.push_back (IntRect (2, 2, 1, 1));
				expected.push_back (IntRect (3, 2, 1, 1));
				expected.push_back (IntRect (4, 2, 1, 1));
				expected.push_back (IntRect (6, 2, 1, 1));
				expected.push_back (IntRect (7, 2, 1, 1));
				cout << check ("Get cleared edge rects test 14", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (-1.5, 0.5), Point (-1.5, 0.5)));
				list<IntRect> expected;
				list<IntRect> got;
				IntRect edgeBounds (0, 0, 1, 8);
				quad.getClearedEdgeRects_ (got, edgeBounds, QuadPathFinder::facing_left, 2, 1);
				expected.push_back (IntRect (0, 0, 1, 1));
				expected.push_back (IntRect (0, 1, 1, 1));
				expected.push_back (IntRect (0, 2, 1, 1));
				expected.push_back (IntRect (0, 3, 1, 1));
				expected.push_back (IntRect (0, 4, 1, 1));
				expected.push_back (IntRect (0, 6, 1, 1));
				expected.push_back (IntRect (0, 7, 1, 1));
				cout << check ("Get cleared edge rects test 15", IntRectListToString (expected, true), IntRectListToString (got, true)) << endl;
			}
		}
		void runPointPassableTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				cout << check ("Point passable test 1.0", true, quad.pointPassable_ (Point (0, 0), 1, 1)) << endl;
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				cout << check ("Point passable test 1.1", false, quad.pointPassable_ (Point (0, 0), 1, 1)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				walls.insert (Line (Point (4.5, 4.5), Point (4.5, 4.5)));
				cout << check ("Point passable test 2.0", true, quad.pointPassable_ (Point (3, 3), 2, 2)) << endl;
				cout << check ("Point passable test 2.1", true, quad.pointPassable_ (Point (4, 3), 2, 2)) << endl;
				cout << check ("Point passable test 2.2", true, quad.pointPassable_ (Point (3, 4), 2, 2)) << endl;
				cout << check ("Point passable test 2.3", false, quad.pointPassable_ (Point (4, 4), 2, 2)) << endl;
				cout << check ("Point passable test 2.4", false, quad.pointPassable_ (Point (4, 5), 2, 2)) << endl;
				cout << check ("Point passable test 2.5", false, quad.pointPassable_ (Point (5, 4), 2, 2)) << endl;
				cout << check ("Point passable test 2.6", false, quad.pointPassable_ (Point (5, 5), 2, 2)) << endl;
				cout << check ("Point passable test 2.7", true, quad.pointPassable_ (Point (5, 6), 2, 2)) << endl;
				cout << check ("Point passable test 2.8", true, quad.pointPassable_ (Point (6, 5), 2, 2)) << endl;
				cout << check ("Point passable test 2.9", true, quad.pointPassable_ (Point (6, 6), 2, 2)) << endl;
			}
		}
		void runRasterizeLineTests() const{
		/*
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 0; x < 9; x++){
					target.push_back (Point (x, x));
					target.push_back (Point (x, x + 1));
				}
				target.push_back (Point (9, 9));
				quad.rasterizeLine_ (rasterized, Line (Point (0, 0), Point (9, 9)));
				cout << check ("Rasterize line test 1", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 9; x > 0; x--){
					target.insert (target.end(), Point (x, x));
					target.insert (target.end(), Point (x - 1, x));
				}
				target.insert (target.end(), Point (0, 0));
				quad.rasterizeLine_ (rasterized, Line (Point (9, 9), Point (0, 0)));
				cout << check ("Rasterize line test 2", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 0; x < 10; x++){
					target.insert (target.end(), Point (x, 0));
				}
				quad.rasterizeLine_ (rasterized, Line (Point (0, 0), Point (9, 0)));
				cout << check ("Rasterize line test 3", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 0; x < 10; x++){
					target.insert (target.begin(), Point (x, 0));
				}
				quad.rasterizeLine_ (rasterized, Line (Point (9, 0), Point (0, 0)));
				cout << check ("Rasterize line test 4", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 0; x < 9; x++){
					target.push_back (Point (x, 9 - x));
					target.push_back (Point (x, 9 - x - 1));
				}
				target.push_back (Point (9, 0));
				quad.rasterizeLine_ (rasterized, Line (Point (0, 9), Point (9, 0)));
				cout << check ("Rasterize line test 5", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				for (int x = 0; x < 9; x++){
					target.push_back (Point (9 - x, x));
					target.push_back (Point (9 - x - 1, x));
				}
				target.push_back (Point (0, 9));
				quad.rasterizeLine_ (rasterized, Line (Point (9, 0), Point (0, 9)));
				cout << check ("Rasterize line test 6", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				int x;
				for (x = 0; x < 5; x++){
					target.push_back (Point (9 - x, x));
					target.push_back (Point (9 - x - 1, x));
				}
				for ( ; x < 9; x++){
					target.push_back (Point (9 - x - 1, x - 1));
					target.push_back (Point (9 - x - 1, x));
				}
				quad.rasterizeLine_ (rasterized, Line (Point (9, 0), Point (0, 8)));
				cout << check ("Rasterize line test 7", PointListToString (target), PointListToString (rasterized)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<Point> rasterized;
				list<Point> target;
				target.push_back (Point (2, 0));
				target.push_back (Point (1, 0));
				target.push_back (Point (1, 1));
				quad.rasterizeLine_ (rasterized, Line (Point (2, 0), Point (1, 1)));
				cout << check ("Rasterize line test 8", PointListToString (target), PointListToString (rasterized)) << endl;
			}
		*/
		}
		void runFindPathTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x <= 99; x++) target.push_back (IntRect (x, 0, 1, 1));
				quad.findPath (path, Point (0, 0), Point (99, 0));
				cout << check ("Find path test 1", IntRectListToString (target), IntRectListToString (path)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x <= 99; x++) target.push_back (IntRect (99 - x, 0, 1, 1));
				quad.findPath (path, Point (99, 0), Point (0, 0));
				cout << check ("Find path test 2", IntRectListToString (target), IntRectListToString (path)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x <= 99; x++) target.push_back (IntRect (0, x, 1, 1));
				quad.findPath (path, Point (0, 0), Point (0, 99));
				cout << check ("Find path test 3", IntRectListToString (target), IntRectListToString (path)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x <= 99; x++) target.push_back (IntRect (0, 99 - x, 1, 1));
				quad.findPath (path, Point (0, 99), Point (0, 0));
				cout << check ("Find path test 4", IntRectListToString (target), IntRectListToString (path)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x < 99; x++){ 
					target.push_back (IntRect (x, x, 1, 1));
					target.push_back (IntRect (x, x + 1, 1, 1));
				}
				target.push_back (IntRect (99, 99, 1, 1));
				quad.findPath (path, Point (0, 0), Point (99, 99));
				cout << check ("Find path test 5", IntRectListToString (target), IntRectListToString (path)) << endl;				
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				for (int x = 0; x < 99; x++){ 
					target.insert (target.begin(), IntRect (x, x, 1, 1));
					target.insert (target.begin(), IntRect (x, x + 1, 1, 1));
				}
				target.insert (target.begin(), IntRect (99, 99, 1, 1));
				quad.findPath (path, Point (99, 99), Point (0, 0));
				cout << check ("Find path test 6", IntRectListToString (target), IntRectListToString (path)) << endl;				
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				walls.insert (Line (Point (0, 49.5), Point (100, 49.5)));
				walls.insert (Line (Point (0, 51.5), Point (100, 51.5)));
				for (int x = 0; x <= 99; x++){ 
					target.insert (target.end(), IntRect (x, 50, 1, 1));
				}
				quad.findPath (path, Point (0, 50), Point (99, 50));
				cout << check ("Find path test 7", IntRectListToString (target), IntRectListToString (path)) << endl;				
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				walls.insert (Line (Point (1.5, 1.5), Point (1.5, 99.5)));
				for (int x = 99; x > 0; x--){ 
					target.insert (target.end(), IntRect (0, x, 1, 1));
				}
				for (int x = 0; x <= 99; x++){ 
					target.insert (target.end(), IntRect (x, 0, 1, 1));
				}
				quad.findPath (path, Point (0, 99), Point (99, 0));
				cout << check ("Find path test 8", IntRectListToString (target), IntRectListToString (path)) << endl;				
			}
			{
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<IntRect> path;
				list<IntRect> target;
				walls.insert (Line (Point (1.5, 2.5), Point (1.5, 49.5)));
				walls.insert (Line (Point (1.5, 50.5), Point (1.5, 99.5)));
				for (int x = 99; x > 0; x--){ 
					target.insert (target.end(), IntRect (0, x, 1, 1));
				}
				target.insert (target.end(), IntRect (1, 1, 1, 1));
				for (int x = 1; x <= 99; x++){ 
					target.insert (target.end(), IntRect (x, 0, 1, 1));
				}
				quad.findPath (path, Point (0, 99), Point (99, 0), 1, 2);
				cout << check ("Find path test 9", IntRectListToString (target), IntRectListToString (path)) << endl;				
			}
			{
				bool pathNotFound = false;
				try{
					SegmentContainer walls (Rect (0, 0, 100, 100));
					QuadPathFinder quad (walls);
					list<IntRect> path;
					list<IntRect> target;
					walls.insert (Line (Point (1.5, 0.5), Point (1.5, 99.5)));
					for (int x = 99; x > 0; x--){ 
						target.insert (target.end(), IntRect (0, x, 1, 1));
					}
					target.insert (target.end(), IntRect (1, 1, 1, 1));
					for (int x = 1; x <= 99; x++){ 
						target.insert (target.end(), IntRect (x, 0, 1, 1));
					}
					quad.findPath (path, Point (0, 99), Point (99, 0), 1, 1);
				}
				catch (PathFinderPathNotFoundException e){
					pathNotFound = true;
				}
				cout << check ("Find path test 10", true, pathNotFound) << endl;				
			}
			{
				bool pathNotFound = false;
				try{
					SegmentContainer walls (Rect (0, 0, 100, 100));
					QuadPathFinder quad (walls);
					list<IntRect> path;
					list<IntRect> target;
					walls.insert (Line (Point (1.5, 0.5), Point (1.5, 49.5)));
					walls.insert (Line (Point (1.5, 51.5), Point (1.5, 99.5)));
					for (int x = 99; x > 0; x--){ 
						target.insert (target.end(), IntRect (0, x, 1, 1));
					}
					target.insert (target.end(), IntRect (1, 1, 1, 1));
					for (int x = 1; x <= 99; x++){ 
						target.insert (target.end(), IntRect (x, 0, 1, 1));
					}
					quad.findPath (path, Point (0, 99), Point (99, 0), 1, 1);
				}
				catch (PathFinderPathNotFoundException e){
					pathNotFound = true;
				}
				cout << check ("Find path test 11", false, pathNotFound) << endl;				
			}
			{
				bool pathNotFound = false;
				try{
					SegmentContainer walls (Rect (0, 0, 100, 100));
					QuadPathFinder quad (walls);
					list<IntRect> path;
					list<IntRect> target;
					walls.insert (Line (Point (1.5, 0.5), Point (1.5, 49.5)));
					walls.insert (Line (Point (1.5, 51.5), Point (1.5, 99.5)));
					for (int x = 99; x > 0; x--){ 
						target.insert (target.end(), IntRect (0, x, 1, 1));
					}
					target.insert (target.end(), IntRect (1, 1, 1, 1));
					for (int x = 1; x <= 99; x++){ 
						target.insert (target.end(), IntRect (x, 0, 1, 1));
					}
					quad.findPath (path, Point (0, 99), Point (99, 0), 2, 1);
				}
				catch (PathFinderPathNotFoundException e){
					pathNotFound = true;
				}
				cout << check ("Find path test 12", false, pathNotFound) << endl;				
			}
			{
				bool pathNotFound = false;
				try{
					SegmentContainer walls (Rect (0, 0, 100, 100));
					QuadPathFinder quad (walls);
					list<IntRect> path;
					list<IntRect> target;
					walls.insert (Line (Point (1.5, 0.5), Point (1.5, 49.5)));
					walls.insert (Line (Point (1.5, 51.5), Point (1.5, 99.5)));
					for (int x = 99; x > 0; x--){ 
						target.insert (target.end(), IntRect (0, x, 1, 1));
					}
					target.insert (target.end(), IntRect (1, 1, 1, 1));
					for (int x = 1; x <= 99; x++){ 
						target.insert (target.end(), IntRect (x, 0, 1, 1));
					}
					quad.findPath (path, Point (0, 99), Point (99, 0), 1, 2);
				}
				catch (PathFinderPathNotFoundException e){
					pathNotFound = true;
				}
				cout << check ("Find path test 13", true, pathNotFound) << endl;				
			}
			{
				bool pathNotFound = false;
				try{
					SegmentContainer walls (Rect (0, 0, 1500, 1500));
					QuadPathFinder quad (walls);
					list<IntRect> path;
					list<IntRect> target;
					for (int x = 0; x < 1; x++){
						int offset = x * 50 - 50;
						Rect wall (236.36, 1141.24 + offset, 150, 100);
						walls.insert (Line (Point (wall.getUpperLeftPoint()), Point (wall.getUpperRightPoint())));
						walls.insert (Line (Point (wall.getLowerLeftPoint()), Point (wall.getLowerRightPoint())));
						walls.insert (Line (Point (wall.getUpperLeftPoint()), Point (wall.getLowerLeftPoint())));
						walls.insert (Line (Point (wall.getUpperRightPoint()), Point (wall.getLowerRightPoint())));
					}
					for (int x = 0; x < 1; x++){
						list<IntRect> path;
						quad.findPath (path, Point (200, 1179), Point (400 + x, 1179), 50, 50);
						//quad.findPath (path, Point (200, 1179), Point (400 + x, 1179));
					}
				}
				catch (PathFinderPathNotFoundException e){
					pathNotFound = true;
				}
				cout << check ("Find path test 14", true, pathNotFound) << endl;
			}
		}
		void runGetAdjacentPathUnitsTests() const{
			{
				SegmentContainer walls (Rect (4, 4, 3, 3));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (5, 5, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 9, 9, 1, 1);
				PathUnit unit1 (4, 4, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (4, 5, -1, -1); target.push_back (&unit2);
				PathUnit unit3 (4, 6, -1, -1); target.push_back (&unit3);
				PathUnit unit4 (5, 4, -1, -1); target.push_back (&unit4);
				PathUnit unit6 (6, 4, -1, -1); target.push_back (&unit6);
				PathUnit unit7 (6, 5, -1, -1); target.push_back (&unit7);
				PathUnit unit8 (6, 6, -1, -1); target.push_back (&unit8);
				PathUnit unit9 (5, 6, -1, -1); target.push_back (&unit9);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 1.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (4, 4, 4, 4));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (5, 5, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 6, 6, 1, 1);
				PathUnit unit92 (6, 6, -1, -1); target.push_back (&unit92);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 2.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 8, 8));
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (5, 5, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);

				PathUnit unit1 (4, 4, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (5, 4, -1, -1); target.push_back (&unit2);
				PathUnit unit4 (6, 4, -1, -1); target.push_back (&unit4);
				PathUnit unit41 (7, 4, -1, -1); target.push_back (&unit41);
				PathUnit unit5 (4, 7, -1, -1); target.push_back (&unit5);
				PathUnit unit6 (5, 7, -1, -1); target.push_back (&unit6);
				PathUnit unit9 (6, 7, -1, -1); target.push_back (&unit9);
				PathUnit unit10 (7, 7, -1, -1); target.push_back (&unit10);
				PathUnit unit11 (4, 5, -1, -1); target.push_back (&unit11);
				PathUnit unit12 (7, 5, -1, -1); target.push_back (&unit12);
				PathUnit unit13 (4, 6, -1, -1); target.push_back (&unit13);
				PathUnit unit14 (7, 6, -1, -1); target.push_back (&unit14);

				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 3.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 8, 8));
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (4, 5, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);
				PathUnit unit1 (4, 4, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (5, 4, -1, -1); target.push_back (&unit2);
				PathUnit unit3 (6, 4, -1, -1); target.push_back (&unit3);
				PathUnit unit4 (7, 4, -1, -1); target.push_back (&unit4);
				PathUnit unit5 (3, 5, -1, -1); target.push_back (&unit5);
				PathUnit unit6 (7, 5, -1, -1); target.push_back (&unit6);
				PathUnit unit7 (4, 6, -1, -1); target.push_back (&unit7);
				PathUnit unit8 (7, 6, -1, -1); target.push_back (&unit8);
				PathUnit unit9 (4, 7, -1, -1); target.push_back (&unit9);
				PathUnit unit10 (5, 7, -1, -1); target.push_back (&unit10);
				PathUnit unit11 (6, 7, -1, -1); target.push_back (&unit11);
				PathUnit unit12 (7, 7, -1, -1); target.push_back (&unit12);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 4.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 8, 8));
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (4, 4, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);
				PathUnit unit1 (4, 3, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (5, 4, -1, -1); target.push_back (&unit2);
				PathUnit unit3 (6, 4, -1, -1); target.push_back (&unit3);
				PathUnit unit4 (7, 4, -1, -1); target.push_back (&unit4);
				PathUnit unit5 (3, 4, -1, -1); target.push_back (&unit5);
				PathUnit unit51 (4, 5, -1, -1); target.push_back (&unit51);
				PathUnit unit6 (7, 5, -1, -1); target.push_back (&unit6);
				PathUnit unit7 (4, 6, -1, -1); target.push_back (&unit7);
				PathUnit unit8 (7, 6, -1, -1); target.push_back (&unit8);
				PathUnit unit9 (4, 7, -1, -1); target.push_back (&unit9);
				PathUnit unit10 (5, 7, -1, -1); target.push_back (&unit10);
				PathUnit unit11 (6, 7, -1, -1); target.push_back (&unit11);
				PathUnit unit12 (7, 7, -1, -1); target.push_back (&unit12);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 5.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 16, 16));
				walls.insert (Line (Point (0.5, 0.5), Point (0.5, 0.5)));
				walls.insert (Line (Point (8.5, 8.5), Point (8.5, 8.5)));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (4, 4, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);
				PathUnit unit1 (4, 3, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (5, 4, -1, -1); target.push_back (&unit2);
				PathUnit unit3 (6, 4, -1, -1); target.push_back (&unit3);
				PathUnit unit4 (7, 4, -1, -1); target.push_back (&unit4);
				PathUnit unit5 (3, 4, -1, -1); target.push_back (&unit5);
				PathUnit unit51 (4, 5, -1, -1); target.push_back (&unit51);
				PathUnit unit6 (7, 5, -1, -1); target.push_back (&unit6);
				PathUnit unit7 (4, 6, -1, -1); target.push_back (&unit7);
				PathUnit unit8 (7, 6, -1, -1); target.push_back (&unit8);
				PathUnit unit9 (4, 7, -1, -1); target.push_back (&unit9);
				PathUnit unit10 (5, 7, -1, -1); target.push_back (&unit10);
				PathUnit unit11 (6, 7, -1, -1); target.push_back (&unit11);
				PathUnit unit12 (7, 7, -1, -1); target.push_back (&unit12);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 6.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 16, 16));
				walls.insert (Line (Point (0.5, 0.5), Point (3.5, 0.5)));
				walls.insert (Line (Point (0.5, 3.5), Point (3.5, 3.5)));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				PathUnit startPathUnit (1, 1, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);
				PathUnit unit1 (0, 1, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (2, 1, -1, -1); target.push_back (&unit2);
				PathUnit unit4 (1, 2, -1, -1); target.push_back (&unit4);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 7.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
			}
			{
				/////////////////////////
				/* This case is minor (one of the adjacents is getting repeated) and isn't easy to fix, so I'm ignoring it for now.
				/////////////////////////
				SegmentContainer walls (Rect (0, 0, 100, 100));
				QuadPathFinder quad (walls);
				list<PathUnit*> pathUnits;
				list<PathUnit*> target;
				walls.insert (Line (Point (0, 49.5), Point (100, 49.5)));
				walls.insert (Line (Point (0, 51.5), Point (100, 51.5)));
				PathUnit startPathUnit (1, 50, 0, 0);
				quad.getAdjacentPathUnits_ (pathUnits, &startPathUnit, 0, 0, 1, 1);
				PathUnit unit1 (1, 49, -1, -1); target.push_back (&unit1);
				PathUnit unit2 (0, 50, -1, -1); target.push_back (&unit2);
				PathUnit unit3 (2, 50, -1, -1); target.push_back (&unit3);
				PathUnit unit4 (1, 51, -1, -1); target.push_back (&unit4);
				PathUnitPropertyFilter filter (true, true, false, false, false);
				cout << check ("Get adjacent path units test 8.0", PathUnitListToString (target, filter, true), PathUnitListToString (pathUnits, filter, true)) << endl;
				*/
			}
		}
		void runGetWallsForBoundsTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				quad.getWallsForBounds_ (nearbyWalls, IntRect (0, 0, 1, 1));
				list<Line> target;
				cout << check ("Get walls for bounds test 1.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 10, 10));
				Line line1 (Point (4, 5), Point (5, 5));
				walls.insert (line1);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				list<Line> target;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 6, 1, 1));
				cout << check ("Get walls for bounds test 2.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (3, 6, 1, 1));
				cout << check ("Get walls for bounds test 2.1", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 6, 1, 1));
				cout << check ("Get walls for bounds test 2.2", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				target.push_back (line1);
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 4, 1, 1));
				cout << check ("Get walls for bounds test 2.3", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 5, 1, 1));
				cout << check ("Get walls for bounds test 2.4", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (3, 5, 1, 1));
				cout << check ("Get walls for bounds test 2.5", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 5, 1, 1));
				cout << check ("Get walls for bounds test 2.6", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (3, 4, 1, 1));
				cout << check ("Get walls for bounds test 2.7", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 4, 1, 1));
				cout << check ("Get walls for bounds test 2.8", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 10, 10));
				Line line1 (Point (4, 5), Point (5, 5));
				walls.insert (line1);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				list<Line> target;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (3, 6, 2, 2));
				cout << check ("Get walls for bounds test 3.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 6, 2, 2));
				cout << check ("Get walls for bounds test 3.1", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 6, 2, 2));
				cout << check ("Get walls for bounds test 3.2", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				target.push_back (line1);
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (2, 5, 2, 2));
				cout << check ("Get walls for bounds test 3.3", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 5, 2, 2));
				cout << check ("Get walls for bounds test 3.4", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 5, 2, 2));
				cout << check ("Get walls for bounds test 3.5", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (2, 4, 2, 2));
				cout << check ("Get walls for bounds test 3.6", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 4, 2, 2));
				cout << check ("Get walls for bounds test 3.7", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 4, 2, 2));
				cout << check ("Get walls for bounds test 3.8", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 10, 10));
				Line line1 (Point (5, 5), Point (5, 5)); walls.insert (line1);
				Line line2 (Point (6, 5), Point (6, 5)); walls.insert (line2);
				Line line3 (Point (5, 6), Point (5, 6)); walls.insert (line3);
				Line line4 (Point (6, 6), Point (6, 6)); walls.insert (line4);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				list<Line> target;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 4, 1, 1));
				target.clear(); target.push_back (line1);
				cout << check ("Get walls for bounds test 4.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 4, 1, 1));
				target.clear(); target.push_back (line1); target.push_back (line2);
				cout << check ("Get walls for bounds test 4.1", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (6, 4, 1, 1));
				target.clear(); target.push_back (line2);
				cout << check ("Get walls for bounds test 4.2", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 5, 1, 1));
				target.clear(); target.push_back (line1); target.push_back (line3);
				cout << check ("Get walls for bounds test 4.3", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 5, 1, 1));
				target.clear(); target.push_back (line1); target.push_back (line2); target.push_back (line3); target.push_back (line4);
				cout << check ("Get walls for bounds test 4.4", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (6, 5, 1, 1));
				target.clear(); target.push_back (line2); target.push_back (line4);
				cout << check ("Get walls for bounds test 4.5", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (4, 6, 1, 1));
				target.clear(); target.push_back (line3); 
				cout << check ("Get walls for bounds test 4.6", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (5, 6, 1, 1));
				target.clear(); target.push_back (line3); target.push_back (line4);
				cout << check ("Get walls for bounds test 4.7", LineListToString (target), LineListToString (nearbyWalls)) << endl;
				nearbyWalls.clear(); quad.getWallsForBounds_ (nearbyWalls, IntRect (6, 6, 1, 1));
				target.clear(); target.push_back (line4); 
				cout << check ("Get walls for bounds test 4.8", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				Line line1 (Point (2.1, 1), Point (2.1, 2)); walls.insert (line1);
				Line line2 (Point (0.9, 1), Point (0.9, 2)); walls.insert (line2);
				Line line3 (Point (0, 0.9), Point (1, 0.9)); walls.insert (line3);
				Line line4 (Point (0, 2.1), Point (1, 2.1)); walls.insert (line4);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				quad.getWallsForBounds_ (nearbyWalls, IntRect (1, 1, 1, 1));
				list<Line> target;
				cout << check ("Get walls for bounds test 5.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 4, 4));
				Line line1 (Point (0.5, 0.5), Point (0.5, 0.5)); walls.insert (line1);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				quad.getWallsForBounds_ (nearbyWalls, IntRect (0, 1, 8, 1));
				list<Line> target;
				cout << check ("Get walls for bounds test 6.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 10, 4));
				Line line1 (Point (7.5, 0.5), Point (7.5, 0.5)); walls.insert (line1);
				QuadPathFinder quad (walls);
				list<Line> nearbyWalls;
				quad.getWallsForBounds_ (nearbyWalls, IntRect (0, 0, 8, 1));
				list<Line> target; target.push_back (line1);
				cout << check ("Get walls for bounds test 7.0", LineListToString (target), LineListToString (nearbyWalls)) << endl;
			}
		}
		void runIntRectTests() const{
			{
				Rect testRect (0, 0, 0, 0);
				IntRect testIntRect (testRect);
				cout << check ("IntRect test 1.0", 0, testIntRect.getX()) << endl;
				cout << check ("IntRect test 1.1", 0, testIntRect.getY()) << endl;
				cout << check ("IntRect test 1.2", 0, testIntRect.getWidth()) << endl;
				cout << check ("IntRect test 1.3", 0, testIntRect.getHeight()) << endl;
			}
			{
				Rect testRect (0, 0, 1, 1);
				IntRect testIntRect (testRect);
				cout << check ("IntRect test 2.0", 0, testIntRect.getX()) << endl;
				cout << check ("IntRect test 2.1", 0, testIntRect.getY()) << endl;
				cout << check ("IntRect test 2.2", 1, testIntRect.getWidth()) << endl;
				cout << check ("IntRect test 2.3", 1, testIntRect.getHeight()) << endl;
			}
			{
				Rect testRect (0, 0, 1.5, 1.5);
				IntRect testIntRect (testRect);
				cout << check ("IntRect test 3.0", 0, testIntRect.getX()) << endl;
				cout << check ("IntRect test 3.1", 0, testIntRect.getY()) << endl;
				cout << check ("IntRect test 3.2", 2, testIntRect.getWidth()) << endl;
				cout << check ("IntRect test 3.3", 2, testIntRect.getHeight()) << endl;
			}
			{
				Rect testRect (0.5, 0.5, 1, 1);
				IntRect testIntRect (testRect);
				cout << check ("IntRect test 4.0", 0, testIntRect.getX()) << endl;
				cout << check ("IntRect test 4.1", 0, testIntRect.getY()) << endl;
				cout << check ("IntRect test 4.2", 2, testIntRect.getWidth()) << endl;
				cout << check ("IntRect test 4.3", 2, testIntRect.getHeight()) << endl;
			}
		}
		void runQuadrisectTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 0, 0));
				cout << check ("Quadrisect test 1", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 1, 0));
				cout << check ("Quadrisect test 2", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 0, 1));
				cout << check ("Quadrisect test 3", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 1, 1));
				cout << check ("Quadrisect test 4", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 1, 2));
				cout << check ("Quadrisect test 5", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 2, 1));
				cout << check ("Quadrisect test 6", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				target.push_back (IntRect (1, 1, 1, 1));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 2, 2));
				cout << check ("Quadrisect test 7", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 2, 1));
				target.push_back (IntRect (0, 1, 1, 2));
				target.push_back (IntRect (1, 1, 2, 2));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 3, 3));
				cout << check ("Quadrisect test 8", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 2, 1));
				target.push_back (IntRect (0, 1, 1, 2));
				target.push_back (IntRect (1, 1, 2, 2));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 3, 3));
				cout << check ("Quadrisect test 9", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (0, 0, 50, 50));
				target.push_back (IntRect (50, 0, 50, 50));
				target.push_back (IntRect (0, 50, 50, 50));
				target.push_back (IntRect (50, 50, 50, 50));
				quad.quadrisect_ (quadrisected, IntRect (0, 0, 100, 100));
				cout << check ("Quadrisect test 10", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				vector<IntRect> quadrisected;
				vector<IntRect> target;
				target.push_back (IntRect (25, 0, 25, 25));
				target.push_back (IntRect (50, 0, 25, 25));
				target.push_back (IntRect (25, 25, 25, 25));
				target.push_back (IntRect (50, 25, 25, 25));
				quad.quadrisect_ (quadrisected, IntRect (25, 0, 50, 50));
				cout << check ("Quadrisect test 11", IntRectVectorToString (target, true), IntRectVectorToString (quadrisected, true)) << endl;
			}
		}
		void runSectorizeTests() const{
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				quad.sectorize_ (sectorized, IntRect (0, 0, 0, 0), 1, 1);
				cout << check ("Sectorize test 1", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				quad.sectorize_ (sectorized, IntRect (0, 0, 0, 1), 1, 1);
				cout << check ("Sectorize test 2", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				quad.sectorize_ (sectorized, IntRect (0, 0, 1, 0), 1, 1);
				cout << check ("Sectorize test 3", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 1, 1), 1, 1);
				cout << check ("Sectorize test 4", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 2, 1), 1, 1);
				cout << check ("Sectorize test 5", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 1, 2), 1, 1);
				cout << check ("Sectorize test 6", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				target.push_back (IntRect (1, 1, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 2, 2), 1, 1);
				cout << check ("Sectorize test 7", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				target.push_back (IntRect (2, 0, 1, 1));
				target.push_back (IntRect (3, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				target.push_back (IntRect (0, 2, 1, 1));
				target.push_back (IntRect (3, 1, 1, 1));
				target.push_back (IntRect (3, 2, 1, 1));
				target.push_back (IntRect (0, 3, 1, 1));
				target.push_back (IntRect (1, 3, 1, 1));
				target.push_back (IntRect (2, 3, 1, 1));
				target.push_back (IntRect (3, 3, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 4, 4), 1, 1);
				cout << check ("Sectorize test 8", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
			{
				SegmentContainer walls (Rect (0, 0, 0, 0));
				QuadPathFinder quad (walls);
	
				list<IntRect> sectorized;
				list<IntRect> target;
				target.push_back (IntRect (0, 0, 1, 1));
				target.push_back (IntRect (1, 0, 1, 1));
				target.push_back (IntRect (2, 0, 1, 1));
				target.push_back (IntRect (3, 0, 1, 1));
				target.push_back (IntRect (4, 0, 1, 1));
				target.push_back (IntRect (0, 1, 1, 1));
				target.push_back (IntRect (0, 2, 1, 1));
				target.push_back (IntRect (0, 3, 1, 1));
				target.push_back (IntRect (0, 4, 1, 1));
				target.push_back (IntRect (4, 1, 1, 1));
				target.push_back (IntRect (4, 2, 1, 1));
				target.push_back (IntRect (4, 3, 1, 1));
				target.push_back (IntRect (4, 4, 1, 1));
				target.push_back (IntRect (1, 4, 1, 1));
				target.push_back (IntRect (2, 4, 1, 1));
				target.push_back (IntRect (3, 4, 1, 1));
				quad.sectorize_ (sectorized, IntRect (0, 0, 5, 5), 1, 1);
				cout << check ("Sectorize test 9", IntRectListToString (target, true), IntRectListToString (sectorized, true)) << endl;
			}
		}
};

int main (int numArgs, char** args){
	QuadPathFinderTester test;

	return 0;
}
