/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "wallpaper.h"
#include "imageProcessingSurfaceManager.h"
#include <iostream>

#define WALLPAPER_DESIRED_DIMENSION_MULTIPLIER 2


Wallpaper::Wallpaper (Surface* parent, const string& definition, const double& x, const double& y, const double& width, const double& height, 
		const bool& useColorKey) : Surface (parent){
	init (definition, x, y, width, height, useColorKey);
}
Wallpaper::~Wallpaper(){
}

void Wallpaper::init (const string& definition, const double& x, const double& y, const double& width, const double& height, const bool& useColorKey){
	setUseColorKey (useColorKey);
	sample = NULL;
	this->definition = definition;
	heightAlreadyWallpapered = 0;
	widthAlreadyWallpapered = 0;
	//cout << "Wallpaper::Wallpaper() Setting width and height to " << width << ", " << height << endl;
	getRect()->setWidth (width);
	getRect()->setHeight (height);
	setXY (x, y);
	//cout << "Wallpaper resources is at " << resources << endl;
	setUpWallpaperSource();
}

void Wallpaper::setDefinitionFile (string definition){
	this->definition = definition;
	setUpWallpaperSource();
}

Rect Wallpaper::getSourceImageDimensions(){
	if (sample) return *(sample->getRect());
	return Rect (0, 0, 0, 0);
}

void Wallpaper::setUpWallpaperSource(){
	//cout << "Setting up wallpaper source, using \"" << definition << "\"" << endl;
	if (definition == "") return;
	if (sample){ 
		delete sample; sample = NULL;
	}
	sample = new Surface (NULL, "", getUseColorKey());
	sample->setResourcePool (getResourcePool());
	//cout << "use color key? " << getUseColorKey() << endl;
	sample->loadBitmap (definition, getUseColorKey());
	sample->setVisible (false);
	//cout << "Setting dirty in setUpWallpaperSource()" << endl;
	setDirty (true);
	//cout << "Done setting up wallpaper source." << endl;
}

void Wallpaper::setWidth (double width){
	getRect()->setWidth (width);
}
void Wallpaper::setHeight (double height){
	getRect()->setHeight (height);
}

bool Wallpaper::update (unsigned int timePassed){
	//cout << "Wallpaper updating!" << endl;
	if (isDirty() || ! getBufferedRect (getBuffer())->equals (*getRect())){
		//cout << "Wallpaper dirty. Filling span." << endl;
		fillSpan();
		return Surface::update (timePassed);
	}
	return Surface::update (timePassed);
}

void Wallpaper::draw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride, 
		const double& offsetX, const double& offsetY){
	Rect* myRect = getRect();
	if (myRect->getHeight() < 1 || myRect->getWidth() < 1){
		//cout << "wallpaper with definition " << definition << " has 0-size rect: " << myRect->toString() << endl;
		return;
	}
	//cout << "Wallpaper is drawing using rect " << myRect->toString() << endl;
	//cout << "WALLPAPER drawing. has " << children.size() << " children." << endl;
	//cout << "WALLPAPER using definition " << definition << endl;
	//SDL_ClearError();
	//vector<Rect> clipBox; clipBox.push_back (Rect (0, 0, 640, 480));
	//Surface::draw (target, clipBox, offsetX, offsetY);
	Surface::draw (manager, screen, dirtyRectsOverride, offsetX, offsetY);
	//cout << "SDL error: " << SDL_GetError() << endl;
	//cout << "WALLPAPER done drawing." << endl;
}

void Wallpaper::fillSpan(){
	if (!sample) return;
	Rect* myRect = getRect();
	//cout << "In Wallpaper::fillSpan(), definition is " << definition << ", myRect is " << myRect->toString() << endl;
	//cout << "Already wallpapered: " << widthAlreadyWallpapered << ", " << heightAlreadyWallpapered << endl;
	if (myRect->getHeight() < 1 || myRect->getWidth() < 1) return;
	int desiredWidth = int (myRect->getWidth());
	int desiredHeight = int (myRect->getHeight());
	if (widthAlreadyWallpapered > desiredWidth && heightAlreadyWallpapered > desiredHeight) return;

	if (!getSurface()){
		SDL_Surface* base = sample->getSurface();
		// NOTE: careful here! after conversion from int to double width and height, possible to have a very subtle memory problem 
		// 	when drawing to this new RGB surface.
		SDL_Surface* newImage = SDL_CreateRGBSurface (base->flags, desiredWidth, desiredHeight, base->format->BitsPerPixel, 
			base->format->Rmask, base->format->Gmask, base->format->Bmask, base->format->Amask);
		setSurface (newImage);
	}
	int sampleWidth = int (sample->getRect()->getWidth());
	int sampleHeight = int (sample->getRect()->getHeight());
	if (getSurface()->w < desiredWidth + sampleWidth || getSurface()->h < desiredHeight + sampleHeight){
		//TODO: should probably move this to utils or something (if I use it again somewhere.)
		SDL_Surface* base = sample->getSurface();
		int newWidth = getSurface()->w << 1; int newHeight = getSurface()->h << 1;
		while (newWidth < desiredWidth + sampleWidth) newWidth = newWidth << 1;
		while (newHeight < desiredHeight + sampleWidth) newHeight = newHeight << 1;
		SDL_Surface* newImage = SDL_CreateRGBSurface (base->flags, newWidth, newHeight, base->format->BitsPerPixel, 
			base->format->Rmask, base->format->Gmask, base->format->Bmask, base->format->Amask);
		// Don't forget to print the old surface onto the new one because the new one has no image!
		SDL_BlitSurface (getSurface(), NULL, newImage, NULL);
		SDL_FreeSurface (getSurface());
		setSurface (newImage);
	}

	// We may as well start with a surface already since we need the image dimensions and 
	//	we can assume this rect needs at least one child to fill it anyway.
	//cout << "Getting ready to make the first child." << endl;
	//Surface* child = new Surface (this, definition);
	Surface* child = sample;
	//cout << "Got a surface. Getting child's rect." << endl;
	Rect* childRect = child->getRect();
	//cout << "Got rect. Getting width and height of rect." << endl;
	int imageWidth = int (childRect->getWidth()); int imageHeight = int (childRect->getHeight());
	//cout << "Made the first child." << endl;

	// We'll fill up the whole rect with tiled images, going left to right, top to bottom.
	// I'm going to split this up into different cases because my hunch is that'll make it 
	// 	easier to follow along when I look back on this and have to figure out what the 
	//	hell I was thinking when I implemented it, despite how wasteful it looks.  :)
	bool widthDone = widthAlreadyWallpapered > desiredWidth;
	bool heightDone = heightAlreadyWallpapered > desiredHeight;
	int baseWidth = widthAlreadyWallpapered;
	// First, cover the case in which we need to add more tiles to each row.
	// Be careful- in case we want to add more rows after this, only fill up 
	//	to the height we've already wallpapered. We'll get the rest in the 
	//	next case.
	//child->setVisible (true); // Because I'm blitting instead of drawing, I can ignore this.
	//cout << "Getting ready to do the width part." << endl;
	//SDL_ClearError();
	if (!widthDone){
		desiredHeight = heightAlreadyWallpapered;
		heightAlreadyWallpapered = 0;
		while (heightAlreadyWallpapered < desiredHeight){
			while (widthAlreadyWallpapered < desiredWidth){
				SDL_Rect destRectRuinedByBlitting;
				destRectRuinedByBlitting.x = widthAlreadyWallpapered;
				destRectRuinedByBlitting.y = heightAlreadyWallpapered;
				SDL_BlitSurface (child->getSurface(), NULL, getSurface(), &destRectRuinedByBlitting);
				//cout << "Blitted wallpaper source to " << widthAlreadyWallpapered << ", " << heightAlreadyWallpapered << endl;
				widthAlreadyWallpapered += imageWidth;
			}
			heightAlreadyWallpapered += imageHeight;
			widthAlreadyWallpapered = baseWidth;
		}
	}
	//cout << "Getting ready to do the height part." << endl;
	// Next, cover the case in which we need to add more rows.
	if (!heightDone){
		desiredHeight = int (getRect()->getHeight());
		while (heightAlreadyWallpapered < desiredHeight){
			widthAlreadyWallpapered = 0;
			while (widthAlreadyWallpapered < desiredWidth){
				SDL_Rect destRectRuinedByBlitting;
				destRectRuinedByBlitting.x = widthAlreadyWallpapered;
				destRectRuinedByBlitting.y = heightAlreadyWallpapered;
				SDL_BlitSurface (child->getSurface(), NULL, getSurface(), &destRectRuinedByBlitting);
				//cout << "Blitted wallpaper source to " << widthAlreadyWallpapered << ", " << heightAlreadyWallpapered << endl;
				widthAlreadyWallpapered += imageWidth;
			}
			heightAlreadyWallpapered += imageHeight;
		}
	}
	//cout << "Child surface is " << child->getSurface() << ". My surface is " << getSurface() << endl;
	//cout << "SDL error: " << SDL_GetError() << endl;
	//child->setVisible (false); // Because I'm blitting instead of drawing, I can ignore this.
	//cout << "Getting rid of extra child." << endl;

	// We'll always have an unplaced child left over. We'll just kill it.
	//removeChildSurface (child);

	//cout << "Papered " << widthAlreadyWallpapered << " width and " << heightAlreadyWallpapered << " height." << endl;
	//cout << "Wallpaper's dimensions: " << getRect()->getWidth() << ", " << getRect()->getHeight() << endl;
	//cout << "In Wallpaper::fillSpan(), definition is " << definition << ", myRect is " << myRect->toString() << ". made it through." << endl;
}

