#ifndef WAYPOINT_PLAN_H
#define WAYPOINT_PLAN_H


#include "geometry.h"
#include "utils.h"
#include "utilTypes.h"


class WaypointPlan{
	friend class WaypointPlanTester;

	public:
		WaypointPlan (const std::vector<Point>& plan) : plan_ (plan), progressToNextWaypoint_ (0), currentStep_ (0), totalProgress_ (0) {
			resetCurrentStepSize();
		}
		~WaypointPlan(){}

		void update (double distance){
			//cout << "updating " << distance << endl;
			//cout << "progress to next is " << progressToNextWaypoint_ << endl;
			totalProgress_ += distance;
			progressToNextWaypoint_ += distance;
			while (progressToNextWaypoint_ >= getCurrentStepSize() && !finishedPath()){
				advanceStep();
				//cout << "progress to next is " << progressToNextWaypoint_ << endl;
				//cout << "current step now " << currentStep_ << endl;
			}
		}
		Optional<Point> getLastWaypoint() const{
			if (plan_.size() < 1) return Optional<Point>();
			return Optional<Point> (plan_ [currentStep_]);
		}
		Optional<Point> getCurrentPoint() const{
			if (plan_.size() < 1) return Optional<Point>();
			if (plan_.size() < 2) return plan_ [currentStep_];
			if (currentStep_ >= plan_.size() - 1){
				return plan_ [currentStep_];
			}
			double x1 = plan_ [currentStep_].getX();
			double x2 = plan_ [currentStep_ + 1].getX();
			double y1 = plan_ [currentStep_].getY();
			double y2 = plan_ [currentStep_ + 1].getY();
			double distance = Utils::get2dDistance (plan_ [currentStep_], plan_ [currentStep_ + 1]);
			double weight = 1.0 - (progressToNextWaypoint_ / distance);
			double recipWeight = 1.0 - weight;
			return Optional<Point> (Point (x1 * weight + x2 * recipWeight, y1 * weight + y2 * recipWeight));
		}
		double getTotalProgress(){ return totalProgress_; }
		double getProgressToNextWaypoint(){ return progressToNextWaypoint_; }
		bool finishedPath() const{ return int (currentStep_) >= int (plan_.size()) - 1 && getDistanceToNextWaypoint() == 0; }

	private:
		double getDistanceToNextWaypoint() const{
			if (int (currentStep_) >= int (plan_.size()) - 1) return 0;
			return Utils::get2dDistance (plan_ [currentStep_], plan_ [currentStep_ + 1]) - progressToNextWaypoint_;
		}
		double getCurrentStepSize(){
			return currentStepSize_;
		}
		void advanceStep(){
			if (currentStep_ + 1 < plan_.size()){
				progressToNextWaypoint_ -= getCurrentStepSize();
				currentStep_++;
			}
			resetCurrentStepSize();
		}
		size_t getPlanSize() const{
			return plan_.size();
		}
		void resetCurrentStepSize(){
			if (currentStep_ + 1 < plan_.size()){
				currentStepSize_ = plan_ [currentStep_].vectorToOtherPoint (plan_ [currentStep_ + 1]).getLength();
			}
			else{
				currentStepSize_ = 0;
			}
		}

		std::vector<Point> plan_;
		double progressToNextWaypoint_;
		unsigned int currentStep_;
		double totalProgress_;
		double currentStepSize_;
};


#endif
